/*
  $RCSfile: momotifu.c,v $       $Revision: 1.29 $
  $State: V7_1 $                  $Date: 2004/11/04 17:49:16 $
*/
#include <Xm/ArrowB.h>
#include <Xm/ArrowBG.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/Command.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/DrawnB.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/MainW.h>
#include <Xm/MenuShell.h>
#include <Xm/MessageB.h>
#include <Xm/PanedW.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/Separator.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>

#include "utmacrou.h"
#include "utstrngu.h"
#include "utstckpu.h"
#include "utmemryu.h"
#include "utlicenu.h"

#include "momotifu.h"
#include "modefltw.h"
#include "mopushbw.h"
#include "motextfw.h"
#include "motextw.h"
#include "molistw.h"

#ifdef BPTPC
#define LPRNTF
#endif

#include "utprntfu.h"

#ifdef DEBUGEvents
static char *eventNames[] = {
   "",
   "",
   "KeyPress",
   "KeyRelease",
   "ButtonPress",
   "ButtonRelease",      /*  5 */
   "MotionNotify",
   "EnterNotify",
   "LeaveNotify",
   "FocusIn",
   "FocusOut",           /* 10 */
   "KeymapNotify",
   "Expose",
   "GraphicsExpose",
   "NoExpose",
   "VisibilityNotify",   /* 15 */
   "CreateNotify",
   "DestroyNotify",
   "UnmapNotify",
   "MapNotify",
   "MapRequest",         /* 20 */
   "ReparentNotify",
   "ConfigureNotify",
   "ConfigureRequest",
   "GravityNotify",
   "ResizeRequest",      /* 25 */
   "CirculateNotify",
   "CirculateRequest",
   "PropertyNotify",
   "SelectionClear",
   "SelectionRequest",   /* 30 */
   "SelectionNotify",
   "ColormapNotify",
   "ClientMessage",
   "MappingNotify"
};
#endif

static pStackOfPtrs momotifParentStack = NULL;

#ifdef __STDC__
void moWaitWhileManaged(Widget w)
#else
void moWaitWhileManaged(w)
Widget w;
#endif
{
   XEvent      buttonOrKeyboardEvent;
   Window      window;
   Widget      oldDefaultParent;
   Widget      newDefaultParent = XtParent(w);
   XtInputMask inputMask;

   if(momotifParentStack == NULL) {
      momotifParentStack = utCreateStackOfPtrs();
      oldDefaultParent = moGetDefaultParentWidget();
      if(oldDefaultParent != NULL)
         utPushStackOfPtrs(momotifParentStack,(pVoid)oldDefaultParent);
   }
   utPushStackOfPtrs(momotifParentStack,(pVoid)newDefaultParent);
   moSetDefaultParentWidget(newDefaultParent);

   do {
      inputMask = XtAppPending(moGetDefaultAppContext());
      if       ((inputMask & XtIMXEvent) != 0) {
         if(XtAppPeekEvent(moGetDefaultAppContext(),&buttonOrKeyboardEvent)) {
            window = XtWindow(w);
#ifdef DEBUGEvents
            switch(buttonOrKeyboardEvent.type) {
               case KeyPress:
               case KeyRelease:
                  window = buttonOrKeyboardEvent.xkey.window;
                  utConsolePrint(" Processing %s event on %s\n",eventNames[buttonOrKeyboardEvent.type],
                                                                moGetWidgetName(XtWindowToWidget(moGetDefaultDisplay(),window)));
                  break;
               case ButtonPress:
               case ButtonRelease:
                  window = buttonOrKeyboardEvent.xbutton.window;
                  utConsolePrint(" Processing %s event on %s\n",eventNames[buttonOrKeyboardEvent.type],
                                                                moGetWidgetName(XtWindowToWidget(moGetDefaultDisplay(),window)));
                  break;
               default:
                  utConsolePrint(" Processing %s event\n",eventNames[buttonOrKeyboardEvent.type]);
                  break;
            }
#endif
         }
         XtAppProcessEvent(moGetDefaultAppContext(),XtIMXEvent);
      } else if((inputMask & XtIMTimer) != 0) {
#ifdef DEBUGEvents
         XtAppPeekEvent(moGetDefaultAppContext(),&buttonOrKeyboardEvent);
         utConsolePrint(" Timer event type =  %d\n",buttonOrKeyboardEvent.type);
#endif
         XtAppProcessEvent(moGetDefaultAppContext(),XtIMTimer);
      } else if((inputMask & XtIMAlternateInput) != 0) {
#ifdef DEBUGEvents
         utConsolePrint(" Alternate input event\n");
#endif
         XtAppProcessEvent(moGetDefaultAppContext(),XtIMAlternateInput);
      }
   } while(XtIsManaged(w));

   newDefaultParent = (Widget)utPopStackOfPtrs(momotifParentStack);
   if(utSizeOfStackOfPtrs(momotifParentStack) > 0)
      newDefaultParent = (Widget)utShowNextStackOfPtrs(momotifParentStack);
   else
      newDefaultParent = (Widget)NULL;
   moSetDefaultParentWidget(newDefaultParent);

   while(XCheckMaskEvent(moGetDefaultDisplay(),
                         KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask,
                         &buttonOrKeyboardEvent)) {
#ifdef DEBUGEvents
      if((buttonOrKeyboardEvent.type == 4) || (buttonOrKeyboardEvent.type == 5)) {
         window = buttonOrKeyboardEvent.xbutton.window;
         utConsolePrint(" Discarding %s event on %s\n",eventNames[buttonOrKeyboardEvent.type],
                                                       moGetWidgetName(XtWindowToWidget(moGetDefaultDisplay(),window)));
      }
#endif
   }

   moProcessExposeEvents();
}


#ifdef __STDC__
void moDestroyManagedParentStack()
#else
void moDestroyManagedParentStack()
#endif
{
   if(momotifParentStack != NULL)
      utDestroyStackOfPtrs(momotifParentStack);
}


#ifdef __STDC__
void moManageWithControl(Widget w)
#else
void moManageWithControl(w)
Widget w;
#endif
{
   XtManageChild(w);
   moWaitWhileManaged(w);
}


#ifdef __STDC__
void moSetSensitive(Widget w)
#else
void moSetSensitive(w)
Widget w;
#endif
{
   XtSetSensitive(w,TRUE);
}


#ifdef __STDC__
void moSetInsensitive(Widget w)
#else
void moSetInsensitive(w)
Widget w;
#endif
{
   XtSetSensitive(w,FALSE);
}


#ifdef __STDC__
utLogical moIsSensitive(Widget w)
#else
utLogical moIsSensitive(Widget w)
Widget w;
#endif
{
   utLogical isSensitive = utTrue;

   if(!XtIsSensitive(w))
      isSensitive = utFalse;

   return(isSensitive);
}


#ifdef __STDC__
void moSetTitleOfDB(Widget wDB,
                    char *title)
#else
void moSetTitleOfDB(wDB,
                    title)
Widget  wDB; 
char   *title;
#endif
{
   int ac;
   Arg al[1];

   ac = 0;
   XtSetArg(al[ac],XmNtitle,title); ac++;
   XtSetValues(wDB,al,ac);
}


#ifdef __STDC__
Pixel moGetForegroundPixel(Widget w)
#else
Pixel moGetForegroundPixel(w)
Widget w; 
#endif
{
   int   ac;
   Arg   al[1];
   Pixel fg;

   ac = 0;
   XtSetArg(al[ac],XmNforeground,&fg); ac++;
   XtGetValues(w,al,ac);

   return(fg);
}


#ifdef __STDC__
void moSetForegroundPixel(Widget w,
                          Pixel fg)
#else
void moSetForegroundPixel(w,
                          fg)
Widget w; 
Pixel  fg;
#endif
{
   int   ac;
   Arg   al[1];
   Pixel currentForeground = moGetForegroundPixel(w);

   if(currentForeground != fg) {
      ac = 0;
      XtSetArg(al[ac],XmNforeground,fg); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
Pixel moGetBackgroundPixel(Widget w)
#else
Pixel moGetBackgroundPixel(w)
Widget w; 
#endif
{
   int   ac;
   Arg   al[1];
   Pixel bg;

   ac = 0;
   XtSetArg(al[ac],XmNbackground,&bg); ac++;
   XtGetValues(w,al,ac);

   return(bg);
}


#ifdef __STDC__
void moSetBackgroundPixel(Widget w,
                          Pixel bg)
#else
void moSetBackgroundPixel(w,
                          bg)
Widget w; 
Pixel  bg;
#endif
{
   int   ac;
   Arg   al[1];
   Pixel currentBackground = moGetBackgroundPixel(w);

   if(currentBackground != bg) {
      ac = 0;
      XtSetArg(al[ac],XmNbackground,bg); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
void moGetSizeOfWidget(Widget w,
                       double *width,
                       double *height)
#else
void moGetSizeOfWidget(w,
                       width,
                       height)
Widget  w;
double *width;
double *height;
#endif
{
   Arg       al[2];
   int       ac;
   Dimension widgetWidth;
   Dimension widgetHeight;

   assert(w != NULL);

   ac = 0;
   XtSetArg(al[ac],XmNwidth,&widgetWidth); ac++;
   XtSetArg(al[ac],XmNheight,&widgetHeight); ac++;
   XtGetValues(w,al,ac);

   *width  = widgetWidth;
   *height = widgetHeight;
}


#ifdef __STDC__
void moSetSizeOfWidget(Widget w,
                       double width,
                       double height)
#else
void moSetSizeOfWidget(w,
                       width,
                       height)
Widget w;
double width;
double height;
#endif
{
   Arg       al[2];
   int       ac;
   Dimension widgetWidth  = (Dimension)width;
   Dimension widgetHeight = (Dimension)height;

   assert(w != NULL);

   ac = 0;
   XtSetArg(al[ac],XmNwidth,widgetWidth); ac++;
   XtSetArg(al[ac],XmNheight,widgetHeight); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
void moGetLocationOfWidget(Widget w,
                           double *xLeft,
                           double *yTop)
#else
void moGetLocationOfWidget(w,
                           xLeft,
                           yTop)
Widget  w;
double *xLeft;
double *yTop;
#endif
{
   Arg      al[2];
   int      ac;
   Position widgetX;
   Position widgetY;

   assert(w != NULL);

   ac = 0;
   XtSetArg(al[ac],XmNx,&widgetX); ac++;
   XtSetArg(al[ac],XmNy,&widgetY); ac++;
   XtGetValues(w,al,ac);

   *xLeft = widgetX;
   *yTop  = widgetY;
}


#ifdef __STDC__
void moSetLocationOfWidget(Widget w,
                           double xLeft,
                           double yTop)
#else
void moSetLocationOfWidget(w,
                           xLeft,
                           yTop)
Widget w;
double xLeft;
double yTop;
#endif
{
   Arg      al[2];
   int      ac;
   Position widgetX = (Position)xLeft;
   Position widgetY = (Position)yTop;

   assert(w != NULL);

   ac = 0;
   XtSetArg(al[ac],XmNx,widgetX); ac++;
   XtSetArg(al[ac],XmNy,widgetY); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
Widget moGetParentDialogShell(Widget w)
#else
Widget moGetParentDialogShell(w)
Widget w;
#endif
{
   Widget parentWidget;

   assert(w != NULL);

   parentWidget = w;
   while(!XmIsDialogShell(parentWidget) && !XtIsApplicationShell(parentWidget))
      parentWidget = XtParent(parentWidget);

   return(parentWidget);
}


#ifdef __STDC__
void moGetSizeOfDialogShellParent(Widget w,
                                  double *width,
                                  double *height)
#else
void moGetSizeOfDialogShellParent(w,
                                  width,
                                  height)
Widget  w;
double *width;
double *height;
#endif
{
   Widget parentWidget = moGetParentDialogShell(w);

   moGetSizeOfWidget(parentWidget,width,height);
}


#ifdef __STDC__
void moGetLocatOfDialogShellParent(Widget w,
                                   double *xLeft,
                                   double *yTop)
#else
void moGetLocatOfDialogShellParent(w,
                                   xLeft,
                                   yTop)
Widget  w;
double *xLeft;
double *yTop;
#endif
{
   Widget parentWidget = moGetParentDialogShell(w);

   moGetLocationOfWidget(parentWidget,xLeft,yTop);
}


#ifdef __STDC__
double moGetScreenWidth()
#else
double moGetScreenWidth()
#endif
{
   Display *display = moGetDefaultDisplay();
   double   screenWidth = 5000.;

   if(display != NULL)
      screenWidth = (double)XDisplayWidth(display,DefaultScreen(display));

   return(screenWidth);
}


#ifdef __STDC__
double moGetScreenHeight()
#else
double moGetScreenHeight()
#endif
{
   Display *display = moGetDefaultDisplay();
   double   screenHeight = 5000.;

   if(display != NULL)
      screenHeight = (double)XDisplayHeight(display,DefaultScreen(display));

   return(screenHeight);
}


#ifdef __STDC__
void moSetEditFunctionBtnSensitivity(Widget list,
                                     Widget addButton,
                                     Widget insertButton,
                                     Widget changeButton,
                                     Widget deleteButton,
                                     Widget copyButton,
                                     Widget pasteButton,
                                     Widget textField1,
                                     Widget textField2,
                                     Widget textField3,
                                     Widget textField4,
                                     Widget textField5,
                                     Widget textField6,
                                     Widget choiceList,
                                     Widget flagSelectionList,
                                     pVoid clipBoard)
#else
void moSetEditFunctionBtnSensitivity(list,
                                     addButton,
                                     insertButton,
                                     changeButton,
                                     deleteButton,
                                     copyButton,
                                     pasteButton,
                                     textField1,
                                     textField2,
                                     textField3,
                                     textField4,
                                     textField5,
                                     textField6,
                                     choiceList,
                                     flagSelectionList,
                                     clipBoard)
Widget list;
Widget addButton;
Widget insertButton;
Widget changeButton;
Widget deleteButton;
Widget copyButton;
Widget pasteButton;
Widget textField1;
Widget textField2;
Widget textField3;
Widget textField4;
Widget textField5;
Widget textField6;
Widget choiceList;
Widget flagSelectionList;
pVoid  clipBoard;
#endif
{
   utLogical  isAnyTextFieldEmpty = utFalse;
   utLogical  isListItemSelected;
   pSarray    selectedFlag;
   char      *selectedFlagString;
   utLogical  isRunTimeVersion = utFalse;

   assert(XmIsList(list));
   if(addButton != NULL)
      assert(XmIsPushButton(addButton) || XmIsPushButtonGadget(addButton));
   if(insertButton != NULL)
      assert(XmIsPushButton(insertButton) || XmIsPushButtonGadget(insertButton));
   if(changeButton != NULL)
      assert(XmIsPushButton(changeButton) || XmIsPushButtonGadget(changeButton));
   if(deleteButton != NULL)
      assert(XmIsPushButton(deleteButton) || XmIsPushButtonGadget(deleteButton));
   if(copyButton != NULL)
      assert(XmIsPushButton(copyButton) || XmIsPushButtonGadget(copyButton));
   if(pasteButton != NULL)
      assert(XmIsPushButton(pasteButton) || XmIsPushButtonGadget(pasteButton));
   if(textField1 != NULL)
      assert(XmIsTextField(textField1) || XmIsText(textField1));
   if(textField2 != NULL)
      assert(XmIsTextField(textField2) || XmIsText(textField2));
   if(textField3 != NULL)
      assert(XmIsTextField(textField3) || XmIsText(textField3));
   if(textField4 != NULL)
      assert(XmIsTextField(textField4) || XmIsText(textField4));
   if(textField5 != NULL)
      assert(XmIsTextField(textField5) || XmIsText(textField5));
   if(textField6 != NULL)
      assert(XmIsTextField(textField6) || XmIsText(textField6));
   if(choiceList != NULL)
      assert(XmIsList(choiceList));
   if(flagSelectionList != NULL)
      assert(XmIsList(flagSelectionList));

   if(!isAnyTextFieldEmpty)
      if(textField1 != NULL)
         if(XmIsTextField(textField1)) {
            if(moIsEmptyTF(textField1))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField1))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(textField2 != NULL)
         if(XmIsTextField(textField2)) {
            if(moIsEmptyTF(textField2))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField2))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(textField3 != NULL)
         if(XmIsTextField(textField3)) {
            if(moIsEmptyTF(textField3))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField3))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(textField4 != NULL)
         if(XmIsTextField(textField4)) {
            if(moIsEmptyTF(textField4))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField4))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(textField5 != NULL)
         if(XmIsTextField(textField5)) {
            if(moIsEmptyTF(textField5))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField5))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(textField6 != NULL)
         if(XmIsTextField(textField6)) {
            if(moIsEmptyTF(textField6))
               isAnyTextFieldEmpty = utTrue;
         } else {
            if(moIsEmptyTX(textField6))
               isAnyTextFieldEmpty = utTrue;
         }

   if(!isAnyTextFieldEmpty)
      if(choiceList != NULL)
         isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(flagSelectionList != NULL) {
         isAnyTextFieldEmpty = utTrue;
      }

   isListItemSelected = utFalse;

   if(isAnyTextFieldEmpty) {
      if(addButton != NULL)
         moSetInsensitive(addButton);
      if(insertButton != NULL)
         moSetInsensitive(insertButton);
      if(changeButton != NULL)
         moSetInsensitive(changeButton);
   } else {
      if(addButton != NULL)
         if(isRunTimeVersion)
            moSetInsensitive(addButton);
         else
            moSetSensitive(addButton);
      if(insertButton != NULL)
         if(isRunTimeVersion)
            moSetInsensitive(insertButton);
         else
            moSetSensitive(insertButton);
      if(changeButton != NULL)
         if(isListItemSelected)
            moSetSensitive(changeButton);
         else
            moSetInsensitive(changeButton);
   }
 
   if(deleteButton != NULL)
      if(isListItemSelected)
         moSetSensitive(deleteButton);
      else
         moSetInsensitive(deleteButton);
 
   if(copyButton != NULL)
      if(isListItemSelected && !isRunTimeVersion)
         moSetSensitive(copyButton);
      else
         moSetInsensitive(copyButton);

   if(pasteButton != NULL)
      if(isListItemSelected && (clipBoard != NULL) && !isRunTimeVersion)
         moSetSensitive(pasteButton);
      else
         moSetInsensitive(pasteButton);
}


#ifdef __STDC__
void moSetEditFileFuncBtnSensitivity(Widget list,
                                     Widget addFileButton,
                                     Widget insertFileButton,
                                     Widget textField1,
                                     Widget textField2,
                                     Widget textField3,
                                     Widget textField4,
                                     Widget textField5,
                                     Widget textField6,
                                     Widget choiceList,
                                     Widget flagSelectionList)
#else
void moSetEditFileFuncBtnSensitivity(list,
                                     addFileButton,
                                     insertFileButton,
                                     textField1,
                                     textField2,
                                     textField3,
                                     textField4,
                                     textField5,
                                     textField6,
                                     choiceList,
                                     flagSelectionList)
Widget list;
Widget addFileButton;
Widget insertFileButton;
Widget textField1;
Widget textField2;
Widget textField3;
Widget textField4;
Widget textField5;
Widget textField6;
Widget choiceList;
Widget flagSelectionList;
#endif
{
   utLogical  isAnyTextFieldEmpty = utFalse;
   pSarray    selectedFlag;
   char      *selectedFlagString;
   utLogical  isRunTimeVersion = utFalse;

   assert(XmIsList(list));
   if(addFileButton != NULL)
      assert(XmIsPushButton(addFileButton) || XmIsPushButtonGadget(addFileButton));
   if(insertFileButton != NULL)
      assert(XmIsPushButton(insertFileButton) || XmIsPushButtonGadget(insertFileButton));
   if(textField1 != NULL)
      assert(XmIsTextField(textField1));
   if(textField2 != NULL)
      assert(XmIsTextField(textField2));
   if(textField3 != NULL)
      assert(XmIsTextField(textField3));
   if(textField4 != NULL)
      assert(XmIsTextField(textField4));
   if(textField5 != NULL)
      assert(XmIsTextField(textField5));
   if(textField6 != NULL)
      assert(XmIsTextField(textField6));
   if(choiceList != NULL)
      assert(XmIsList(choiceList));
   if(flagSelectionList != NULL)
      assert(XmIsList(flagSelectionList));

   if(textField1 != NULL)
      if(moIsEmptyTF(textField1))
         isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(textField2 != NULL)
         if(moIsEmptyTF(textField2))
            isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(textField3 != NULL)
         if(moIsEmptyTF(textField3))
            isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(textField4 != NULL)
         if(moIsEmptyTF(textField4))
            isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(textField5 != NULL)
         if(moIsEmptyTF(textField5))
            isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(textField6 != NULL)
         if(moIsEmptyTF(textField6))
            isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(choiceList != NULL)
          isAnyTextFieldEmpty = utTrue;
   if(!isAnyTextFieldEmpty)
      if(flagSelectionList != NULL) {
         isAnyTextFieldEmpty = utTrue;
      }

   if(isAnyTextFieldEmpty) {
      if(addFileButton != NULL)
         moSetInsensitive(addFileButton);
      if(insertFileButton != NULL)
         moSetInsensitive(insertFileButton);
   } else {
      if(addFileButton != NULL)
         if(isRunTimeVersion)
            moSetInsensitive(addFileButton);
         else
            moSetSensitive(addFileButton);
      if(insertFileButton != NULL)
         if(isRunTimeVersion)
            moSetInsensitive(insertFileButton);
         else
            moSetSensitive(insertFileButton);
   }
}


#ifdef __STDC__
long moGetEventMask(Widget w)
#else
long moGetEventMask(w)
Widget w;
#endif
{
   long               eventMask = -1;
   Display           *display = moGetDefaultDisplay();
   Window             window;
   XWindowAttributes  windowAttributes;

   assert(XtIsWidget(w));

   if(XtIsRealized(w)) {
      window = XtWindow(w);
      if(XGetWindowAttributes(display,window,&windowAttributes))
         eventMask = windowAttributes.your_event_mask;
   } else
      utConsolePrint("moGetEventMask: Widget %s is not realized\n",moGetWidgetName(w));

   return(eventMask);
}


#ifdef __STDC__
void moSetEventMask(Widget w,
                    long eventMask)
#else
void moSetEventMask(w,
                    eventMask)
Widget w;
long   eventMask;
#endif
{
   Display              *display = moGetDefaultDisplay();
   Window                window;
   XSetWindowAttributes  windowAttributes;

   assert(XtIsWidget(w));

   if(XtIsRealized(w)) {
      window = XtWindow(w);
      windowAttributes.event_mask = eventMask;
      XChangeWindowAttributes(display,window,CWEventMask,&windowAttributes);
   } else
      utConsolePrint("moSetEventMask: Widget %s is not realized\n",moGetWidgetName(w));
}


#ifdef __STDC__
void moPrintEventMask(FILE *fp,
                      long eventMask)
#else
void moPrintEventMask(fp,
                      eventMask)
FILE *fp;
long  eventMask;
#endif
{
   if(eventMask != NoEventMask) {
      utConsolePrint("The events are recognized\n");
      if((eventMask & KeyPressMask) != 0)
         utConsolePrint("   KeyPress\n");
      if((eventMask & KeyReleaseMask) != 0)
         utConsolePrint("   KeyRelease\n");
      if((eventMask & ButtonPressMask) != 0)
         utConsolePrint("   ButtonPress\n");
      if((eventMask & ButtonReleaseMask) != 0)
         utConsolePrint("   ButtonRelease\n");
      if((eventMask & EnterWindowMask) != 0)
         utConsolePrint("   EnterWindow\n");
      if((eventMask & LeaveWindowMask) != 0)
         utConsolePrint("   LeaveWindow\n");
      if((eventMask & PointerMotionMask) != 0)
         utConsolePrint("   PointerMotion\n");
      if((eventMask & PointerMotionHintMask) != 0)
         utConsolePrint("   PointerMotionHint\n");
      if((eventMask & Button1MotionMask) != 0)
         utConsolePrint("   Button1Motion\n");
      if((eventMask & Button2MotionMask) != 0)
         utConsolePrint("   Button2Motion\n");
      if((eventMask & Button3MotionMask) != 0)
         utConsolePrint("   Button3Motion\n");
      if((eventMask & Button4MotionMask) != 0)
         utConsolePrint("   Button4Motion\n");
      if((eventMask & Button5MotionMask) != 0)
         utConsolePrint("   Button5Motion\n");
      if((eventMask & ButtonMotionMask) != 0)
         utConsolePrint("   ButtonMotion\n");
      if((eventMask & KeymapStateMask) != 0)
         utConsolePrint("   KeymapState\n");
      if((eventMask & ExposureMask) != 0)
         utConsolePrint("   Exposure\n");
      if((eventMask & VisibilityChangeMask) != 0)
         utConsolePrint("   VisibilityChange\n");
      if((eventMask & StructureNotifyMask) != 0)
         utConsolePrint("   StructureNotify\n");
      if((eventMask & ResizeRedirectMask) != 0)
         utConsolePrint("   ResizeRedirect\n");
      if((eventMask & SubstructureNotifyMask) != 0)
         utConsolePrint("   SubstructureNotify\n");
      if((eventMask & SubstructureRedirectMask) != 0)
         utConsolePrint("   SubstructureRedirect\n");
      if((eventMask & FocusChangeMask) != 0)
         utConsolePrint("   FocusChange\n");
      if((eventMask & PropertyChangeMask) != 0)
         utConsolePrint("   PropertyChange\n");
      if((eventMask & ColormapChangeMask) != 0)
         utConsolePrint("   ColormapChange\n");
      if((eventMask & OwnerGrabButtonMask) != 0)
         utConsolePrint("   OwnerGrabButton\n");
   }
}


#ifdef __STDC__
void moSetTraversalOn(Widget w)
#else
void moSetTraversalOn(w)
Widget w; 
#endif
{
   int ac;
   Arg al[1];

   ac = 0;
   XtSetArg(al[ac],XmNtraversalOn,TRUE); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
void moSetTraversalOff(Widget w)
#else
void moSetTraversalOff(w)
Widget w; 
#endif
{
   int ac;
   Arg al[1];

   if((XmIsTextField(w) || XmIsText(w)) && (XmVersion < 1002)) {
   } else {
      ac = 0;
      XtSetArg(al[ac],XmNtraversalOn,FALSE); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
void moSetWidgetFont(Widget w,
                     char *fontName)
#else
void moSetWidgetFont(w,
                     fontName)
Widget  w;
char   *fontName;
#endif
{
   Arg      al[1];
   int      ac;
   XrmValue from_value,to_value;

   from_value.addr = fontName;
   from_value.size = utStringLength(from_value.addr)+1;
   to_value.addr = NULL;
   XtConvertAndStore(w,XmRString,&from_value,XmRFontList,&to_value);

   ac = 0;
   if(to_value.addr) {
      XtSetArg(al[ac],XmNfontList,*(unsigned int *)to_value.addr); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
char *moGetWidgetName(Widget w)
#else
char *moGetWidgetName(w)
Widget w;
#endif
{
   assert(XtIsWidget(w) || XmIsGadget(w));

   return(XtName(w));
}


#ifdef __STDC__
void moRingBell()
#else
void moRingBell()
#endif
{
   Display *display = moGetDefaultDisplay();

   XBell(display,0);
}


#ifdef __STDC__
char *moGetResourceValue(char *programName,
                         char *resourceName)
#else
char *moGetResourceValue(programName,
                         resourceName)
char *programName;
char *resourceName;
#endif
{
   Display *display = moGetDefaultDisplay();
   char    *resourceString = NULL;

   if(display != NULL)
      resourceString = XGetDefault(display,programName,resourceName);

   return(utAssignString(resourceString));
}

static XErrorHandler XErrorHandlerFunction;

#ifdef __STDC__
int moXerrorHandler(Display *display,
                    XErrorEvent *errorEvent)
#else
int moXerrorHandler(display,
                    errorEvent)
Display     *display;
XErrorEvent *errorEvent;
#endif
{
   char buf[MAX_STRING_LENGTH+1];

   XGetErrorText(display,errorEvent->error_code,
                 buf,MAX_STRING_LENGTH);
   fprintf(stderr," Xerror Code %s\n",buf);

   XErrorHandlerFunction(display,errorEvent);

   return(1);
}


#ifdef __STDC__
void moSetErrorHandler()
#else
void moSetErrorHandler()
#endif
{
   XErrorHandlerFunction = XSetErrorHandler(moXerrorHandler);
}


#ifdef __STDC__
void moProcessExposeEvents()
#else
void moProcessExposeEvents()
#endif
{
   XEvent exposeEvent;

   while(XCheckMaskEvent(moGetDefaultDisplay(),
                         ExposureMask,
                         &exposeEvent)) {
      XtDispatchEvent(&exposeEvent);
   }
}


#ifdef __STDC__
void moProcessStructureNotifyEvents()
#else
void moProcessStructureNotifyEvents()
#endif
{
   XEvent structureNotifyEvent;

   while(XCheckMaskEvent(moGetDefaultDisplay(),
                         StructureNotifyMask | SubstructureNotifyMask | ExposureMask,
                         &structureNotifyEvent)) {
      XtDispatchEvent(&structureNotifyEvent);
   }
}


#ifdef __STDC__
void moHorizontallyCenterWidgetInFM(Widget w)
#else
void moHorizontallyCenterWidgetInFM(w)
Widget w;
#endif
{
   Widget parentFM = XtParent(w);
   double widgetWidth,widgetHeight;
   double parentWidth,parentHeight;
   double widgetOffsetLeft;
   Arg    al[1];
   int    ac = 0;

   assert(XtIsWidget(w));
   assert(XmIsForm(parentFM));

   moGetSizeOfWidget(parentFM,
                     &parentWidth,&parentHeight);
   moGetSizeOfWidget(w,
                     &widgetWidth,&widgetHeight);

   if(parentWidth > widgetWidth) {
      widgetOffsetLeft = (parentWidth-widgetWidth)/2.;
      XtSetArg(al[ac],XmNleftOffset,(int)widgetOffsetLeft); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
void moVerticallyCenterWidgetInFM(Widget w)
#else
void moVerticallyCenterWidgetInFM(w)
Widget w;
#endif
{
   Widget parentFM = XtParent(w);
   double widgetWidth,widgetHeight;
   double parentWidth,parentHeight;
   double widgetOffsetTop;
   Arg    al[1];
   int    ac = 0;

   assert(XtIsWidget(w));
   assert(XmIsForm(parentFM));

   moGetSizeOfWidget(parentFM,
                     &parentWidth,&parentHeight);
   moGetSizeOfWidget(w,
                     &widgetWidth,&widgetHeight);

   if(parentHeight > widgetHeight) {
      widgetOffsetTop = (parentHeight-widgetHeight)/2.;
      XtSetArg(al[ac],XmNtopOffset,(int)widgetOffsetTop); ac++;
      XtSetValues(w,al,ac);
   }
}


#ifdef __STDC__
double moGetLeftOffset(Widget w)
#else
double moGetLeftOffset(w)
Widget w;
#endif
{
   int widgetOffsetLeft;
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNleftOffset,&widgetOffsetLeft); ac++;
   XtGetValues(w,al,ac);

   return((double)widgetOffsetLeft);
}


#ifdef __STDC__
void moSetLeftOffset(Widget w,
                     double widgetOffsetLeft)
#else
void moSetLeftOffset(w,
                     widgetOffsetLeft)
Widget w;
double widgetOffsetLeft;
#endif
{
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNleftOffset,(int)widgetOffsetLeft); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
double moGetRightOffset(Widget w)
#else
double moGetRightOffset(w)
Widget w;
#endif
{
   int widgetOffsetRight;
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNrightOffset,&widgetOffsetRight); ac++;
   XtGetValues(w,al,ac);

   return((double)widgetOffsetRight);
}


#ifdef __STDC__
void moSetRightOffset(Widget w,
                      double widgetOffsetRight)
#else
void moSetRightOffset(w,
                      widgetOffsetRight)
Widget w;
double widgetOffsetRight;
#endif
{
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNrightOffset,(int)widgetOffsetRight); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
double moGetTopOffset(Widget w)
#else
double moGetTopOffset(w)
Widget w;
#endif
{
   int widgetOffsetTop;
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNtopOffset,&widgetOffsetTop); ac++;
   XtGetValues(w,al,ac);

   return((double)widgetOffsetTop);
}


#ifdef __STDC__
void moSetTopOffset(Widget w,
                    double widgetOffsetTop)
#else
void moSetTopOffset(w,
                    widgetOffsetTop)
Widget w;
double widgetOffsetTop;
#endif
{
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNtopOffset,(int)widgetOffsetTop); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
double moGetBottomOffset(Widget w)
#else
double moGetBottomOffset(w)
Widget w;
#endif
{
   int widgetOffsetBottom;
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNbottomOffset,&widgetOffsetBottom); ac++;
   XtGetValues(w,al,ac);

   return((double)widgetOffsetBottom);
}


#ifdef __STDC__
void moSetBottomOffset(Widget w,
                       double widgetOffsetBottom)
#else
void moSetBottomOffset(w,
                       widgetOffsetBottom)
Widget w;
double widgetOffsetBottom;
#endif
{
   Arg al[1];
   int ac = 0;

   assert(XtIsWidget(w));

   XtSetArg(al[ac],XmNbottomOffset,(int)widgetOffsetBottom); ac++;
   XtSetValues(w,al,ac);
}


#ifdef __STDC__
void moSwitchMouseButtons2and3()
#else
void moSwitchMouseButtons2and3()
#endif
{
   Display       *display = moGetDefaultDisplay();
   unsigned char  mouseMap[10];
   int            i1,nMouseMap;
   int            logicalMouseButton;
   char          *switchMouseButtons2and3 = moGetResourceValue("BATCHES",
                                                               "SwitchMouseButtons2and3");

   if(utEqualStrings(switchMouseButtons2and3,"True")) {
      nMouseMap = XGetPointerMapping(display,mouseMap,5);
      for(i1=0 ; i1<nMouseMap ; i1++) {
         logicalMouseButton = mouseMap[i1];
         if(logicalMouseButton == 2)
            mouseMap[i1] = 3;
         if(logicalMouseButton == 3)
            mouseMap[i1] = 2;
      }
      XSetPointerMapping(display,mouseMap,nMouseMap);
   }

   utFreeBytes((pVoid)switchMouseButtons2and3);
}


#ifdef __STDC__
void moPrintWidgetClass(Widget w)
#else
void moPrintWidgetClass(w)
Widget w;
#endif
{
   utConsolePrint("Widget %s belongs to class ",moGetWidgetName(w));

   if     (XmIsArrowButton(w))
      utConsolePrint("ArrowButton\n");
   else if(XmIsArrowButtonGadget(w))
      utConsolePrint("ArrowButtonGadget\n");
   else if(XmIsBulletinBoard(w))
      utConsolePrint("BulletinBoard\n");
   else if(XmIsCascadeButton(w))
      utConsolePrint("CascadeButton\n");
   else if(XmIsCascadeButtonGadget(w))
      utConsolePrint("CascadeButtonGadget\n");
   else if(XmIsCommand(w))
      utConsolePrint("Command\n");
   else if(XmIsDialogShell(w))
      utConsolePrint("DialogShell\n");
   else if(XmIsDrawingArea(w))
      utConsolePrint("DrawingArea\n");
   else if(XmIsDrawnButton(w))
      utConsolePrint("DrawnButton\n");
   else if(XmIsFileSelectionBox(w))
      utConsolePrint("FileSelectionBox\n");
   else if(XmIsForm(w))
      utConsolePrint("Form\n");
   else if(XmIsFrame(w))
      utConsolePrint("Frame\n");
   else if(XmIsLabel(w))
      utConsolePrint("Label\n");
   else if(XmIsLabelGadget(w))
      utConsolePrint("LabelGadget\n");
   else if(XmIsList(w))
      utConsolePrint("List\n");
   else if(XmIsMainWindow(w))
      utConsolePrint("MainWindow\n");
   else if(XmIsMenuShell(w))
      utConsolePrint("MenuShell\n");
   else if(XmIsMessageBox(w))
      utConsolePrint("MessageBox\n");
   else if(XmIsPanedWindow(w))
      utConsolePrint("PanedWindow\n");
   else if(XmIsPushButton(w))
      utConsolePrint("PushButton\n");
   else if(XmIsPushButtonGadget(w))
      utConsolePrint("PushButtonGadget\n");
   else if(XmIsRowColumn(w))
      utConsolePrint("RowColumn\n");
   else if(XmIsScale(w))
      utConsolePrint("Scale\n");
   else if(XmIsScrollBar(w))
      utConsolePrint("ScrollBar\n");
   else if(XmIsScrolledWindow(w))
      utConsolePrint("ScrolledWindow\n");
   else if(XmIsSelectionBox(w))
      utConsolePrint("SelectionBox\n");
   else if(XmIsSeparatorGadget(w))
      utConsolePrint("SeparatorGadget\n");
   else if(XmIsSeparator(w))
      utConsolePrint("Separator\n");
   else if(XmIsText(w))
      utConsolePrint("Text\n");
   else if(XmIsTextField(w))
      utConsolePrint("TextField\n");
   else if(XmIsToggleButton(w))
      utConsolePrint("ToggleButton\n");
   else if(XmIsToggleButtonGadget(w))
      utConsolePrint("ToggleButtonGadget\n");
   else if(XmIsVendorShell(w))
      utConsolePrint("VendorShell\n");
}


#ifdef __STDC__
void moDestroyWidgetChildren(Widget w)
#else
void moDestroyWidgetChildren(w)
Widget w;
#endif
{
   Cardinal   nChildren;
   WidgetList childWidgets;
   Arg        al[2];
   int        ac;
   int        i1;

   ac = 0;
   XtSetArg(al[ac],XmNnumChildren,&nChildren); ac++;
   XtSetArg(al[ac],XmNchildren,&childWidgets); ac++;
   XtGetValues(w,al,ac);

   for(i1=0 ; i1<(int)nChildren ; i1++)
      XtDestroyWidget(childWidgets[i1]);
}

static long momotifInputMask = KeyPressMask | KeyReleaseMask |
                               ButtonPressMask | ButtonReleaseMask;

#ifdef __STDC__
void moDisableInput(Widget w)
#else
void moDisableInput(w)
Widget w;
#endif
{
   long eventMask;
   long noInputMask = ~momotifInputMask;
   long noInputEventMask;

   eventMask = moGetEventMask(w);
   noInputEventMask = eventMask & noInputMask;
   moSetEventMask(w,noInputEventMask);

   moSetTraversalOff(w);
}


#ifdef __STDC__
void moEnableInput(Widget w)
#else
void moEnableInput(w)
Widget w;
#endif
{
   long eventMask;
   long inputEventMask;

   eventMask = moGetEventMask(w);
   inputEventMask = eventMask | momotifInputMask;
   moSetEventMask(w,inputEventMask);

   moSetTraversalOn(w);
}


