/*
  $RCSfile: motextw.c,v $       $Revision: 1.9 $
  $State: V7_1 $                  $Date: 2004/11/04 17:49:36 $
*/
#include <Xm/Form.h>
#include <Xm/FileSB.h>

#include "utmemryu.h"
#include "utstrngu.h"

#include "momotifu.h"
#include "motextw.h"
#include "mosbutnu.h"

/* to take care of lprintf */
#ifdef BPTPC
#define LPRNTF
#endif
#include "utprntfu.h"

#ifdef __STDC__
void moClearTX(Widget wText)
#else
void moClearTX(wText)
Widget wText;
#endif
{
   assert(wText != NULL);
   assert(XmIsText(wText));

   moSetCstringToTX(wText,"");
}


#ifdef __STDC__
void moSetCstringToTX(Widget wText,
                      char *s)
#else
void moSetCstringToTX(wText,
                      s)
Widget  wText;
char   *s;
#endif
{
   assert(wText != NULL);
   assert(XmIsText(wText));

   if(!utIsNullString(s))
      XmTextSetString(wText,s);
   else
      XmTextSetString(wText,"");
}


#ifdef __STDC__
void moCopyToCstringFromTX(Widget wText,
                           char* buf,
                           int bufSize)
#else
void moCopyToCstringFromTX(wText,
                           buf,
                           bufSize)
Widget  wText;
char   *buf;
int     bufSize;
#endif
{
   char *s;
  
   assert(wText != NULL);
   assert(XmIsText(wText));
   assert(buf != NULL);

   s = moGetCstringFromTX(wText);
   assert(utStringLength(s) < (bufSize - 1));
   strncpy(buf,s,bufSize);
   utFreeBytes((pVoid)s);
}


#ifdef __STDC__
char *moGetCstringFromTX(Widget wText)
#else
char *moGetCstringFromTX(wText)
Widget wText;
#endif
{
   char *xts;
   char *cstring;

   assert(wText != NULL);
   assert(XmIsText(wText));

   xts = XmTextGetString(wText);
   cstring = utAssignString(xts);
   XtFree(xts);

   return(cstring);
}


#ifdef __STDC__
void moSetEditableTX(Widget wText)
#else
void moSetEditableTX(wText)
Widget wText;
#endif
{
   int ac;
   Arg al[1];

   assert(wText != NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNeditable,TRUE); ac++;
   XtSetValues(wText,al,ac);
}


#ifdef __STDC__
void moSetUneditableTX(Widget wText)
#else
void moSetUneditableTX(wText)
Widget wText;
#endif
{
   int ac;
   Arg al[1];

   assert(wText != NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNeditable,FALSE); ac++;
   XtSetValues(wText,al,ac);
}


#ifdef __STDC__
void moSetCursorVisibleOnTX(Widget wText)
#else
void moSetCursorVisibleOnTX(wText)
Widget wText;
#endif
{
   int ac;
   Arg al[1];

   assert(wText!= NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNcursorPositionVisible,TRUE); ac++;
   XtSetValues(wText,al,ac);
}


#ifdef __STDC__
void moSetCursorVisibleOffTX(Widget wText)
#else
void moSetCursorVisibleOffTX(wText)
Widget wText;
#endif
{
   int ac;
   Arg al[1];

   assert(wText!= NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNcursorPositionVisible,FALSE); ac++;
   XtSetValues(wText,al,ac);
}


#ifdef __STDC__
int moGetMaxLengthTX(Widget wText)
#else
int moGetMaxLengthTX(wText)
Widget wText;
#endif
{
   int ac;
   Arg al[1];
   int maxLength;

   assert(wText != NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNmaxLength,&maxLength); ac++;
   XtGetValues(wText,al,ac);

   return(maxLength);
}


#ifdef __STDC__
void moSetMaxLengthTX(Widget wText,int maxLength)
#else
void moSetMaxLengthTX(wText,maxLength)
Widget wText;
int    maxLength;
#endif
{
   int ac;
   Arg al[1];

   assert(wText != NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNmaxLength,maxLength); ac++;
   XtSetValues(wText,al,ac);
}

static long motextDefaultEventMask = -1;
static long motextDefaultBackground = -1;
static long motextInputMask = KeyPressMask | KeyReleaseMask |
                              ButtonPressMask | ButtonReleaseMask;

#ifdef __STDC__
void moDisableInputTX(Widget wText)
#else
void moDisableInputTX(wText)
Widget wText;
#endif
{
   Widget form;
   Pixel  background;
   long   eventMask;
   long   noInputMask = ~motextInputMask;
   long   noInputEventMask;

   assert(wText != NULL);
   assert(XmIsText(wText));

   eventMask = moGetEventMask(wText);
   noInputEventMask = eventMask & noInputMask;
   moSetEventMask(wText,noInputEventMask);
   if(motextDefaultEventMask == -1)
      motextDefaultEventMask = eventMask;

   moSetCursorVisibleOffTX(wText);
   
   form = XtParent(wText);
   while(!XmIsForm(form) && !XmIsFileSelectionBox(form))
      form = XtParent(form);
 
   if(motextDefaultBackground == -1) {
      motextDefaultBackground = (long)moGetBackgroundPixel(wText);
/*    utConsolePrint("Default text pixel = %ld, set by Disable(%s)\n",motextDefaultBackground,
                                                                      moGetWidgetName(wText)); */
   }
   background = moGetBackgroundPixel(form);
   moSetBackgroundPixel(wText,background);
   moSetTraversalOff(wText);
}


#ifdef __STDC__
void moEnableInputTX(Widget wText)
#else
void moEnableInputTX(wText)
Widget wText;
#endif
{
   long            eventMask;
   long            inputEventMask;
   utLogical       isSbuttonWidget;
   ColorIdentifier sbuttonColorIdentifier;

   assert(wText != NULL);
   assert(XmIsText(wText));

   isSbuttonWidget = moIsSbuttonWidget(wText);
   if(isSbuttonWidget) {
      if(motextDefaultEventMask >= 0) {
         eventMask = moGetEventMask(wText);
         inputEventMask = eventMask | motextInputMask;
         moSetEventMask(wText,inputEventMask);

         sbuttonColorIdentifier = moGetSbuttonColorIdentifier();
         moSetBackgroundPixel(wText,(Pixel)sbuttonColorIdentifier);
         moSetTraversalOn(wText);
      }
   } else {
      if((motextDefaultEventMask >= 0) && (motextDefaultBackground >= 0)) {
         eventMask = moGetEventMask(wText);
         inputEventMask = eventMask | motextInputMask;
         moSetEventMask(wText,inputEventMask);

         moSetBackgroundPixel(wText,(Pixel)motextDefaultBackground);
      } else {
         if(motextDefaultBackground == -1) {
            motextDefaultBackground = (long)moGetBackgroundPixel(wText);
/*          utConsolePrint("Default text pixel = %ld, set by Enable(%s)\n",motextDefaultBackground,
                                                                           moGetWidgetName(wText)); */
         }
         if(motextDefaultEventMask == -1)
            motextDefaultEventMask = moGetEventMask(wText);
      }
      moSetTraversalOn(wText);
   }
   if(moIsEditableTX(wText))
      moSetCursorVisibleOnTX(wText);
}


#ifdef __STDC__
utLogical moIsEditableTX(Widget wText)
#else
utLogical moIsEditableTX(wText)
Widget wText;
#endif
{
   int     ac;
   Arg     al[1];
   Boolean isEditable;

   assert(wText != NULL);
   assert(XmIsText(wText));

   ac = 0;
   XtSetArg(al[ac],XmNeditable,&isEditable); ac++;
   XtGetValues(wText,al,ac);

   if(isEditable)
      return(utTrue);
   else
      return(utFalse);
}


#ifdef __STDC__
void moUnderlineStringTX(Widget wText,
                         int startPosition,
                         int endPosition)
#else
void moUnderlineStringTX(wText,
                         startPosition,
                         endPosition)
Widget wText;
int    startPosition;
int    endPosition;
#endif
{
   XmTextPosition left  = (XmTextPosition)startPosition;
   XmTextPosition right = (XmTextPosition)endPosition;

   assert(wText != NULL);
   assert(XmIsText(wText));

   XmTextSetHighlight(wText,left,right,XmHIGHLIGHT_SECONDARY_SELECTED);
}


#ifdef __STDC__
char *moGetSelectionTX(Widget wText)
#else
char *moGetSelectionTX(wText)
Widget wText;
#endif
{
   char *returnString = NULL;
   char *string;

   assert(wText != NULL);
   assert(XmIsText(wText));

   string = XmTextGetSelection(wText);

   returnString = utAssignString(string);
   XtFree(string);

   return(returnString);
}


#ifdef __STDC__
utLogical moIsEmptyTX(Widget wText)
#else
utLogical moIsEmptyTX(wText)
Widget wText;
#endif
{
   utLogical  isEmptyTX = utFalse;
   char      *string;

   assert(wText != NULL);
   assert(XmIsText(wText));

   string = moGetCstringFromTX(wText);
   if(utIsNullString(string))
      isEmptyTX = utTrue;
   else
      if(utHasAllBlanks(string))
         isEmptyTX = utTrue;

   utFreeBytes((pVoid)string);

   return(isEmptyTX);
}


#ifdef __STDC__
int moGetNumberOfColumnsTX(Widget wText)
#else
int moGetNumberOfColumnsTX(wText)
Widget wText;
#endif
{
   int ac = 0;
   Arg al[1];
   int numberOfColumns;

   assert(wText != NULL);
   assert(XmIsText(wText));

   XtSetArg(al[ac],XmNcolumns,&numberOfColumns); ac++;
   XtGetValues(wText,al,ac);

   return(numberOfColumns);
}


#ifdef __STDC__
void moSetNumberOfColumnsTX(Widget wText,
                            int numberOfColumns)
#else
void moSetNumberOfColumnsTX(wText,
                            numberOfColumns)
Widget wText;
int    numberOfColumns;
#endif
{
   int ac = 0;
   Arg al[1];

   assert(wText != NULL);
   assert(XmIsText(wText));

   XtSetArg(al[ac],XmNcolumns,numberOfColumns); ac++;
   XtSetValues(wText,al,ac);
}


#ifdef __STDC__
int moGetNumberOfRowsTX(Widget wText)
#else
int moGetNumberOfRowsTX(wText)
Widget wText;
#endif
{
   int ac = 0;
   Arg al[1];
   int numberOfRows;

   assert(wText != NULL);
   assert(XmIsText(wText));

   XtSetArg(al[ac],XmNrows,&numberOfRows); ac++;
   XtGetValues(wText,al,ac);

   return(numberOfRows);
}


#ifdef __STDC__
void moSetNumberOfRowsTX(Widget wText,
                         int numberOfRows)
#else
void moSetNumberOfRowsTX(wText,
                         numberOfRows)
Widget wText;
int    numberOfRows;
#endif
{
   int ac = 0;
   Arg al[1];

   assert(wText != NULL);
   assert(XmIsText(wText));

   XtSetArg(al[ac],XmNrows,numberOfRows); ac++;
   XtSetValues(wText,al,ac);
}


