/*
  $RCSfile: utfilesu.h,v $       $Revision: 1.9 $
  $State: V7_1 $                  $Date: 2004/11/04 17:51:09 $
*/
#ifndef _UTFILESU_H
#define _UTFILESU_H

#ifdef __cplusplus
   extern "C" {
#endif

#include <stdio.h>

#include "uttypesu.h"
#include "utsarryu.h"

#define HEADBUFFERSIZE                 6
#define FILEBUFFERSIZE               512
#define UNIVERSALPATHDELIMITER       ":"
#define ARCHIVEDFILEHEADERDELIMITER  "%"

#ifdef BPTUNIX
#define EXECUTABLEFILEEXTENSION      "x"
#define OBJECTFILEEXTENSION          "o"
#define LIBRARYFILEEXTENSION         "a"
#endif
#ifdef BPTVMS
#define EXECUTABLEFILEEXTENSION      "EXE;*"
#define OBJECTFILEEXTENSION          "OBJ;*"
#define LIBRARYFILEEXTENSION         "OLB;*"
#endif
#ifdef BPTPC
#define EXECUTABLEFILEEXTENSION      "EXE"
#define OBJECTFILEEXTENSION          "OBJ"
#define LIBRARYFILEEXTENSION         "LIB"
#endif

#define ALLFILES   "*"

typedef enum {
   FilePermissionReadOnly,
   FilePermissionReadWrite
} utFilePermission;

typedef enum {
   FileOpenStatusRead,
   FileOpenStatusWrite,
   FileOpenStatusAppend,
   FileOpenStatusReadWrite
} utFileOpenStatus;

typedef enum {
   FileTypeUnknown   = 0,
   FileTypeDirectory = 1,
   FileTypeRegular   = 2
} utFileType;

typedef fpos_t FilePosition,*pFilePosition;

#ifdef __STDC__
   extern long           utGetFileTime(char *fileName);
   extern long           utGetFileSize(char *fileName);
   extern utLogical      utIsFileTypeDirectory(char *fileName);
   extern utLogical      utIsFileTypeRegular(char *fileName);
   extern char          *utGetExtensionFromFile(char *fileName);
   extern char          *utGetDirectoryFromFile(char *fileName);
   extern char          *utGetBaseFromFile(char *fileName);
   extern char          *utGetBaseWithExtensionFromFile(char *fileName);
   extern utLogical      utRemoveFile(char *fullPathName);
   extern utLogical      utRemoveDirectory(char *pathName);
   extern utLogical      utFileExist(char *fileName);
   extern utLogical      utIsFileWriteable(char *fileName);
   extern utLogical      utIsDirectorySearchable(char *dirname);
   extern char          *utMakeFileName(char *directory,
                                        char *basename, char *extension);
   extern pSarray        utDirectoryList(char *directory,
                                         utFilePermission filePermission);
   extern pSarray        utDirectoryListDirectories(char *directory,
                                                    utFilePermission filePermission);
   extern pSarray        utDirectoryListRegularFiles(char *directory,
                                                     utFilePermission filePermission);
   extern pSarray        utDirListPatternRegularFiles(char *directory,
                                                      char *pattern,
                                                      utLogical ignoreCase,
                                                      utFilePermission filePermission);
   extern char          *utGetParentDirectory(char *dirname);
   extern char          *utMakeSubDirectoryName(char *directory,
                                                char *subDirectory);
   extern utLogical      utCreateSubDirectory(char *parentDirectory,
                                              char *subDirectory);
   extern void           utGetFileTree(char *directory,
                                       pSarray fileTree,
                                       utFilePermission filePermission);
   extern utLogical      utRemoveFileTree(char *directory,
                                          utFilePermission filePermission);
   extern utLogical      utCopyAsciiFile(char *originalFile,
                                         char *destinationFile);
   extern utLogical      utConcatenateAsciiFile(char *originalFile,
                                                FILE *fpDestination);
   extern char          *utFileToString(char *fileName);
   extern char          *utGetCurrentWorkingDirectory();
   extern utLogical      utRenameFile(char *oldPathName,
                                      char *newPathName);
   extern utLogical      utArchiveDirectoryTreeIntoFile(char *directory,
                                                        utFilePermission filePermission,
                                                        char *archivedFilePathName);
   extern utLogical      utRetrieveDirectoryTreeArchive(char *archivedFilePathName,
                                                        char *directoryPath);
   extern FILE          *utFileOpen(char *filePathName,
                                    utFileOpenStatus fileOpenStatus);
   extern utLogical      utFileClose(FILE *fp);
   extern char          *utNativeToUniversalFilePath(char *filePath);
   extern char          *utUniversalToNativeFilePath(char *filePath);
   extern void           utSaveUniversalFilePath(char *universalFilePath,
                                                 FILE *fp);
   extern char          *utLoadUniversalFilePath(FILE *fp);
   extern utLogical      utCopyDirectoryTree(char *originalDirectory,
                                             utFilePermission filePermission,
                                             char *destinationDirectory);
   extern char          *utGetUniqueFileName(char *directory,
                                             char *fileNamePrefix);
   extern utLogical      utIsFileAnExecutableFile(char *filePathName);
   extern utLogical      utIsFileAnObjectFile(char *filePathName);
   extern utLogical      utIsFileALibraryFile(char *filePathName);
   extern pFilePosition  utCreateFilePosition();
   extern void           utFreeFilePosition(pFilePosition *filePosition);
   extern pFilePosition  utAssignFilePosition(pFilePosition filePosition);
   extern pFilePosition  utGetFilePosition(FILE *fp);
   extern utLogical      utRecordFilePosition(FILE *fp,
                                              pFilePosition filePosition);
   extern utLogical      utSetFilePosition(FILE *fp,
                                           pFilePosition filePosition);
#else
   extern long           utGetFileTime();
   extern long           utGetFileSize();
   extern utLogical      utIsFileTypeDirectory();
   extern utLogical      utIsFileTypeRegular();
   extern char          *utGetExtensionFromFile();
   extern char          *utGetDirectoryFromFile();
   extern char          *utGetBaseFromFile();
   extern char          *utGetBaseWithExtensionFromFile();
   extern utLogical      utRemoveFile();
   extern utLogical      utRemoveDirectory();
   extern utLogical      utFileExist();
   extern utLogical      utIsFileWriteable();
   extern utLogical      utIsDirectorySearchable();
   extern char          *utMakeFileName();
   extern pSarray        utDirectoryList();
   extern pSarray        utDirectoryListDirectories();
   extern pSarray        utDirectoryListRegularFiles();
   extern pSarray        utDirListPatternRegularFiles();
   extern char          *utGetParentDirectory();
   extern char          *utMakeSubDirectoryName();
   extern utLogical      utCreateSubDirectory();
   extern void           utGetFileTree();
   extern utLogical      utRemoveFileTree();
   extern utLogical      utCopyAsciiFile();
   extern utLogical      utConcatenateAsciiFile();
   extern char          *utFileToString();
   extern char          *utGetCurrentWorkingDirectory();
   extern utLogical      utRenameFile();
   extern utLogical      utArchiveDirectoryTreeIntoFile();
   extern utLogical      utRetrieveDirectoryTreeArchive();
   extern FILE          *utFileOpen();
   extern utLogical      utFileClose();
   extern char          *utNativeToUniversalFilePath();
   extern char          *utUniversalToNativeFilePath();
   extern void           utSaveUniversalFilePath();
   extern char          *utLoadUniversalFilePath();
   extern utLogical      utCopyDirectoryTree();
   extern char          *utGetUniqueFileName();
   extern utLogical      utIsFileAnExecutableFile();
   extern utLogical      utIsFileAnObjectFile();
   extern utLogical      utIsFileALibraryFile();
   extern pFilePosition  utCreateFilePosition();
   extern void           utFreeFilePosition();
   extern pFilePosition  utAssignFilePosition();
   extern pFilePosition  utGetFilePosition();
   extern utLogical      utRecordFilePosition();
   extern utLogical      utSetFilePosition();
#endif

#ifdef __cplusplus
   }
#endif

#endif
