/*
  $RCSfile: utlistiu.c,v $       $Revision: 1.1 $
  $State: V6_0 $                  $Date: 1997/04/21 11:01:20 $
*/
#include "uttypesu.h"
#include "utlistiu.h"
#include "utlistip.h"
#include "utmemryu.h"

#ifdef __STDC__
pListOfInts utCreateListOfInts(int nBlockSize)
#else
pListOfInts utCreateListOfInts(nBlockSize)
   int nBlockSize;
#endif
{
  pListOfInts list;

  list = (pListOfInts) utGetBytes(sizeof(ListOfInts));
  utInitializeListOfInts(list, nBlockSize);
  return list;
}

#ifdef __STDC__
static void utInitializeListOfInts(pListOfInts list, int nBlockSize)
#else
static void utInitializeListOfInts(list, nBlockSize)
pListOfInts list; 
int         nBlockSize;
#endif
{
  assert(nBlockSize > 0);

  list->StructType   = STRUCTTYPE_LISTOFINTS;
  list->m_nCount     = 0;
  list->m_pNodeHead  = NULL;
  list->m_pNodeTail  = NULL;
  list->m_pNodeFree  = NULL;
  list->m_pBlocks    = NULL;
  list->m_nBlockSize = nBlockSize;
}

#ifdef __STDC__
void utDestroyListOfInts(pListOfInts list)
#else
void utDestroyListOfInts(list)
pListOfInts list;
#endif
{
   CNode_I *pNode;

   for(pNode=list->m_pNodeHead ; pNode!=NULL ; pNode=pNode->pNext)
      pNode->data = 0;

   list->m_nCount = 0;
   list->m_pNodeHead = NULL;
   list->m_pNodeTail = NULL;
   list->m_pNodeFree = NULL;
   utFreeDataChainListOfInts(list->m_pBlocks);
   list->m_pBlocks = NULL;

   utFreeBytes((pVoid)list);
}

#ifdef __STDC__
static CNode_I* utNewNodeListOfInts(pListOfInts list, CNode_I* pPrev, 
                                                         CNode_I* pNext)
#else
static CNode_I* utNewNodeListOfInts(list, pPrev, pNext)
   pListOfInts list;
   CNode_I* pPrev; 
   CNode_I* pNext;
#endif
{
  int i;
  CNode_I* pNode;
 
  if (list->m_pNodeFree == NULL)
  {
     /* add another block */
     CNode_I * newBlock = utCreateNewBlockListOfInts(list);
     /* chain them into free list */
     for (i = 0, pNode = newBlock; i < list->m_nBlockSize-1;
                                                     i++, pNode++)
     {
        pNode->pNext = list->m_pNodeFree;
        list->m_pNodeFree = pNode;
     }
  }

  assert(list->m_pNodeFree != NULL);  /* we must have something */
  pNode = list->m_pNodeFree;
  list->m_pNodeFree = list->m_pNodeFree->pNext;
  list->m_nCount++;
  assert (list->m_nCount > 0);  /* make sure we don't overflow */
  pNode->pPrev = pPrev;
  pNode->pNext = pNext;
  pNode->data = 0;

  return(pNode);
}

#ifdef __STDC__
static void utFreeNodeListOfInts(pListOfInts list, CNode_I* pNode)
#else
static void utFreeNodeListOfInts(list, pNode)
   pListOfInts list; 
   CNode_I* pNode;
#endif
{
   pNode->data = 0;
   pNode->pNext = list->m_pNodeFree;
   list->m_pNodeFree = pNode;
   list->m_nCount--;
   /* make sure we don't underflow */
   assert(list->m_nCount >= 0);  
}

#ifdef __STDC__
pVoid utAddHeadListOfInts(pListOfInts list, int newElement)
#else
pVoid utAddHeadListOfInts(list, newElement)
   pListOfInts list; 
   int newElement;
#endif
{
   CNode_I* pNewNode = utNewNodeListOfInts(list, NULL, list->m_pNodeHead);

   assert(list != NULL);

   pNewNode->data = newElement; 
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = pNewNode;
   else
      list->m_pNodeTail = pNewNode;
   list->m_pNodeHead = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
pVoid utAddTailListOfInts(pListOfInts list, int newElement)
#else
pVoid utAddTailListOfInts(list, newElement)
   pListOfInts list;  
   int newElement;
#endif
{
   CNode_I* pNewNode = utNewNodeListOfInts(list, list->m_pNodeTail, NULL);

   assert (list != NULL);

   pNewNode->data = newElement;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   list->m_pNodeTail = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utAddListToHeadListOfInts(pListOfInts list, pListOfInts pNewList)
#else
void utAddListToHeadListOfInts(list, pNewList)
   pListOfInts list;
   pListOfInts pNewList;
#endif
{
   pVoid pos;

   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements to head (maintain order) */
   pos = utGetTailPosListOfInts(pNewList);
   while (pos != NULL)
   {
     utAddHeadListOfInts(list, ((CNode_I*) pos)->data);
     pos = utGetPrevPosListOfInts(pos);
   }  
}

#ifdef __STDC__
void utAddListToTailListOfInts(pListOfInts list, pListOfInts pNewList)
#else
void utAddListToTailListOfInts(list, pNewList)
   pListOfInts list;
   pListOfInts pNewList;
#endif
{
   pVoid pos;

   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements */
   pos = utGetHeadPosListOfInts(pNewList);
   while (pos)
   {
     utAddTailListOfInts(list, ((CNode_I*) pos)->data);
     pos = utGetNextPosListOfInts(pos);
   }
}

#ifdef __STDC__
void utRemoveHeadListOfInts(pListOfInts list)
#else
void utRemoveHeadListOfInts(list)
  pListOfInts list;
#endif
{
   CNode_I* pOldNode;

   assert(list != NULL);
   /* don't call on empty list !!! */
   assert(list->m_pNodeHead != NULL);  

   pOldNode = list->m_pNodeHead;

   list->m_pNodeHead = pOldNode->pNext;
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = NULL;
   else
      list->m_pNodeTail = NULL;
   utFreeNodeListOfInts(list, pOldNode);

   return;
}

#ifdef __STDC__
void utRemoveTailListOfInts(pListOfInts list)
#else
void utRemoveTailListOfInts(list)
   pListOfInts list;
#endif
{
   CNode_I* pOldNode;

   /* don't call on empty list !!! */
   assert(list->m_pNodeTail != NULL);  

   pOldNode = list->m_pNodeTail;

   list->m_pNodeTail = pOldNode->pPrev;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = NULL;
   else
      list->m_pNodeHead = NULL;
   utFreeNodeListOfInts(list, pOldNode);

   return;
}

#ifdef __STDC__
pVoid utInsertBeforeListOfInts(pListOfInts list,
                               pVoid position, 
                               int newElement)
#else
pVoid utInsertBeforeListOfInts(list, position, newElement)
    pListOfInts list;
    pVoid position; 
    int newElement;
#endif
{
   CNode_I* pOldNode;
   CNode_I* pNewNode;
   /* insert before nothing->head of the list */
   if (position == NULL)
      return utAddHeadListOfInts(list, newElement); 

   /* utInsert it before position */
   pOldNode = (CNode_I *) position;
   pNewNode = utNewNodeListOfInts(list, pOldNode->pPrev, pOldNode);
   pNewNode->data = newElement;

   if (pOldNode->pPrev != NULL)
      pOldNode->pPrev->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   pOldNode->pPrev = pNewNode;
   return (pVoid) pNewNode;
}


#ifdef __STDC__
pVoid utInsertAfterListOfInts(pListOfInts list,
                              pVoid position, 
                              int newElement)
#else
pVoid utInsertAfterListOfInts(list, position, newElement)
   pListOfInts list;
   pVoid position; 
   int newElement;
#endif
{
   CNode_I* pOldNode;
   CNode_I* pNewNode;
   /* insert after nothing->tail of the list */
   if (position == NULL)
      return utAddTailListOfInts(list, newElement); 

   /* Insert it before position */
   pOldNode = (CNode_I *) position;
   pNewNode = utNewNodeListOfInts(list, pOldNode, pOldNode->pNext);
   pNewNode->data = newElement;

   if (pOldNode->pNext != NULL)
      pOldNode->pNext->pPrev = pNewNode;
   else
   list->m_pNodeTail = pNewNode;
   pOldNode->pNext = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utRemoveAtListOfInts(pListOfInts list, pVoid position)
#else
void utRemoveAtListOfInts(list, position)
   pListOfInts list;
   pVoid position;
#endif
{
   CNode_I* pOldNode = (CNode_I *) position;

   /* remove pOldNode from list */
   if (pOldNode == list->m_pNodeHead)
      list->m_pNodeHead = pOldNode->pNext;
   else
      pOldNode->pPrev->pNext = pOldNode->pNext;
   if (pOldNode == list->m_pNodeTail)
      list->m_pNodeTail = pOldNode->pPrev;
   else
      pOldNode->pNext->pPrev = pOldNode->pPrev;
   utFreeNodeListOfInts(list, pOldNode);
}


/* slow operations */

#ifdef __STDC__
pVoid utGetAtListOfInts(pListOfInts list, int nIndex)
#else
pVoid utGetAtListOfInts(list, nIndex) 
   pListOfInts list;
   int nIndex;
#endif
{
   CNode_I* pNode;

   assert(nIndex >= 0);

   if (nIndex >= list->m_nCount)
      return NULL;  /* went too far */

   pNode = list->m_pNodeHead;
   while (nIndex--)
   {
      pNode = pNode->pNext;
   }
   return (pVoid) pNode;
}

#ifdef __STDC__
pVoid utFindItemListOfInts(pListOfInts list,
                           pVoid startAfter,
                           int searchValue) 
#else
pVoid utFindItemListOfInts(list, startAfter, searchValue)
   pListOfInts list; 
   pVoid startAfter;
   int searchValue; 
#endif
{
  CNode_I* pNode = (CNode_I *) startAfter;
  if (pNode == NULL)
    pNode = list->m_pNodeHead;  /* start at head */
  else
    pNode = pNode->pNext;  /* start after the one specified */

  for (; pNode != NULL; pNode = pNode->pNext)
    if (pNode->data == searchValue)
      return (pVoid) pNode;
  return NULL;
}

#ifdef __STDC__
int utGetCountListOfInts(pListOfInts list)
#else
int utGetCountListOfInts(list)
   pListOfInts list;
#endif
{
  return list->m_nCount;
}

#ifdef __STDC__
pVoid utGetHeadPosListOfInts(pListOfInts list)
#else
pVoid utGetHeadPosListOfInts(list)
   pListOfInts list;
#endif
{
  return (pVoid) list->m_pNodeHead;
}

#ifdef __STDC__
pVoid utGetTailPosListOfInts(pListOfInts list)
#else
pVoid utGetTailPosListOfInts(list)
   pListOfInts list;
#endif
{
  assert (list != NULL);
  return (pVoid) list->m_pNodeTail;
}

#ifdef __STDC__
pVoid utGetPrevPosListOfInts(pVoid pos)
#else
pVoid utGetPrevPosListOfInts(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) ((CNode_I *) pos)->pPrev;
}

#ifdef __STDC__
pVoid utGetNextPosListOfInts(pVoid pos)
#else
pVoid utGetNextPosListOfInts(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) (((CNode_I *) pos)->pNext);
}

#ifdef __STDC__
int utGetHeadListOfInts(pListOfInts list)
#else
int utGetHeadListOfInts(list)
  pListOfInts list;
#endif
{
  return utGetDataListOfInts(utGetHeadPosListOfInts(list));
}
 
#ifdef __STDC__
int utGetTailListOfInts(pListOfInts list)
#else
int utGetTailListOfInts(list)
  pListOfInts list;
#endif
{
  return utGetDataListOfInts(utGetTailPosListOfInts(list));
}
 

#ifdef __STDC__
int utGetNextListOfInts(pVoid* pos)
#else
int utGetNextListOfInts(pos)
  pVoid* pos;
#endif
{
  pVoid prevPos;
  assert(!pos || !*pos); /* do not call with null pos */
  prevPos = *pos;
  *pos = utGetNextPosListOfInts(pos);
  return utGetDataListOfInts(utGetNextPosListOfInts(prevPos));
}

#ifdef __STDC__
int utGetPrevListOfInts(pVoid* pos)
#else
int utGetPrevListOfInts(pos)
  pVoid* pos;
#endif
{
  pVoid oldPos;
  assert(!pos || !*pos); /* do not call with null pos */
  oldPos = *pos;
  *pos = utGetPrevPosListOfInts(pos);
  return utGetDataListOfInts(utGetPrevPosListOfInts(oldPos));
}

#ifdef __STDC__
int utGetDataListOfInts(pVoid pos)
#else
int utGetDataListOfInts(pos)
  pVoid pos;
#endif
{
  assert (pos != NULL);
  return ((CNode_I *) pos)->data;
}

#ifdef __STDC__
static CNode_I* utCreateNewBlockListOfInts(pListOfInts list)
#else
static CNode_I* utCreateNewBlockListOfInts(list)
pListOfInts list;
#endif
{
   CPlex_I *p;
   CNode_I *node;
   int      nAssoc = list->m_nBlockSize;
   int      i1;

   assert (nAssoc > 0);

   p = (CPlex_I*)utGetBytes(sizeof(CPlex_I) + sizeof(CNode_I) * nAssoc);
   node = (CNode_I *)(p+1);
   for(i1=0 ; i1<nAssoc ; i1++) {
      node->StructType = STRUCTTYPE_CNODE_I;
      node->pNext      = NULL;
      node->pPrev      = NULL;
      node->data       = 0;
      node++;
   }
   p->StructType = STRUCTTYPE_CPLEX_I;
   p->pNext      = list->m_pBlocks;

   list->m_pBlocks = p;

   return((CNode_I *)(p + 1));
}
 

#ifdef __STDC__
static void utFreeDataChainListOfInts(CPlex_I* p)
#else
static void utFreeDataChainListOfInts(p)
   CPlex_I* p;
#endif
{
   CPlex_I* pNext;
 
   while (p != NULL)
   {
        pNext = p->pNext;
        utFreeBytes((pVoid)p);
        p = pNext;
   }
}
 
