/*
  $RCSfile: utlistlu.c,v $       $Revision: 1.1 $
  $State: V6_0 $                  $Date: 1997/04/21 11:01:23 $
*/
#include "uttypesu.h"
#include "utlistlu.h"
#include "utlistlp.h"
#include "utmemryu.h"

#ifdef __STDC__
pListOfLongs utCreateListOfLongs(int nBlockSize)
#else
pListOfLongs utCreateListOfLongs(nBlockSize)
   int nBlockSize;
#endif
{
  pListOfLongs list;

  list = (pListOfLongs) utGetBytes(sizeof(ListOfLongs));
  utInitializeListOfLongs(list, nBlockSize);
  return list;
}

#ifdef __STDC__
static void utInitializeListOfLongs(pListOfLongs list, int nBlockSize)
#else
static void utInitializeListOfLongs(list, nBlockSize)
   pListOfLongs list; 
   int nBlockSize;
#endif
{
  assert(nBlockSize > 0);

  list->StructType   = STRUCTTYPE_LISTOFLONGS;
  list->m_nCount     = 0;
  list->m_pNodeHead  = NULL;
  list->m_pNodeTail  = NULL;
  list->m_pNodeFree  = NULL;
  list->m_pBlocks    = NULL;
  list->m_nBlockSize = nBlockSize;
}

#ifdef __STDC__
void utDestroyListOfLongs(pListOfLongs list)
#else
void utDestroyListOfLongs(list)
pListOfLongs list;
#endif
{
   CNode_L *pNode;

   for(pNode=list->m_pNodeHead ; pNode!=NULL ; pNode=pNode->pNext)
      pNode->data = 0;

   list->m_nCount = 0;
   list->m_pNodeHead = NULL;
   list->m_pNodeTail = NULL;
   list->m_pNodeFree = NULL;
   utFreeDataChainListOfLongs(list->m_pBlocks);
   list->m_pBlocks = NULL;

   utFreeBytes((pVoid)list);
}

#ifdef __STDC__
static CNode_L* utNewNodeListOfLongs(pListOfLongs list, CNode_L* pPrev, 
                                                         CNode_L* pNext)
#else
static CNode_L* utNewNodeListOfLongs(list, pPrev, pNext)
   pListOfLongs list;
   CNode_L* pPrev; 
   CNode_L* pNext;
#endif
{
  int i;
  CNode_L* pNode;
 
  if (list->m_pNodeFree == NULL)
  {
     /* add another block */
     CNode_L * newBlock = utCreateNewBlockListOfLongs(list);
     /* chain them into free list */
     for (i = 0, pNode = newBlock; i < list->m_nBlockSize-1;
                                                     i++, pNode++)
     {
        pNode->pNext = list->m_pNodeFree;
        list->m_pNodeFree = pNode;
     }
  }

  assert(list->m_pNodeFree != NULL);  /* we must have something */
  pNode = list->m_pNodeFree;
  list->m_pNodeFree = list->m_pNodeFree->pNext;
  list->m_nCount++;
  assert (list->m_nCount > 0);  /* make sure we don't overflow */
  pNode->pPrev = pPrev;
  pNode->pNext = pNext;
  pNode->data = 0;
  return pNode;
}

#ifdef __STDC__
static void utFreeNodeListOfLongs(pListOfLongs list, CNode_L* pNode)
#else
static void utFreeNodeListOfLongs(list, pNode)
   pListOfLongs list; 
   CNode_L* pNode;
#endif
{
   pNode->data = 0;
   pNode->pNext = list->m_pNodeFree;
   list->m_pNodeFree = pNode;
   list->m_nCount--;
   /* make sure we don't underflow */
   assert(list->m_nCount >= 0);  
}

#ifdef __STDC__
pVoid utAddHeadListOfLongs(pListOfLongs list, long newElement)
#else
pVoid utAddHeadListOfLongs(list, newElement)
   pListOfLongs list; 
   long newElement;
#endif
{
   CNode_L* pNewNode = utNewNodeListOfLongs(list, NULL, list->m_pNodeHead);

   assert(list != NULL);

   pNewNode->data = newElement; 
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = pNewNode;
   else
      list->m_pNodeTail = pNewNode;
   list->m_pNodeHead = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
pVoid utAddTailListOfLongs(pListOfLongs list, long newElement)
#else
pVoid utAddTailListOfLongs(list, newElement)
   pListOfLongs list;  
   long newElement;
#endif
{
   CNode_L* pNewNode = utNewNodeListOfLongs(list, list->m_pNodeTail, NULL);

   assert (list != NULL);

   pNewNode->data = newElement;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   list->m_pNodeTail = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utAddListToHeadListOfLongs(pListOfLongs list, pListOfLongs pNewList)
#else
void utAddListToHeadListOfLongs(list, pNewList)
   pListOfLongs list;
   pListOfLongs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements to head (maintain order) */
   pos = utGetTailPosListOfLongs(pNewList);
   while (pos != NULL)
   {
     utAddHeadListOfLongs(list, ((CNode_L*) pos)->data);
     pos = utGetPrevPosListOfLongs(pos);
   }  
}

#ifdef __STDC__
void utAddListToTailListOfLongs(pListOfLongs list, pListOfLongs pNewList)
#else
void utAddListToTailListOfLongs(list, pNewList)
   pListOfLongs list;
   pListOfLongs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements */
   pos = utGetHeadPosListOfLongs(pNewList);
   while (pos)
   {
     utAddTailListOfLongs(list, ((CNode_L*) pos)->data);
     pos = utGetNextPosListOfLongs(pos);
   }
}

#ifdef __STDC__
void utRemoveHeadListOfLongs(pListOfLongs list)
#else
void utRemoveHeadListOfLongs(list)
  pListOfLongs list;
#endif
{
   CNode_L* pOldNode;

   assert(list != NULL);
   /* don't call on empty list !!! */
   assert(list->m_pNodeHead != NULL);  

   pOldNode = list->m_pNodeHead;

   list->m_pNodeHead = pOldNode->pNext;
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = NULL;
   else
      list->m_pNodeTail = NULL;
   utFreeNodeListOfLongs(list, pOldNode);

   return;
}

#ifdef __STDC__
void utRemoveTailListOfLongs(pListOfLongs list)
#else
void utRemoveTailListOfLongs(list)
   pListOfLongs list;
#endif
{
   CNode_L* pOldNode;

   /* don't call on empty list !!! */
   assert(list->m_pNodeTail != NULL);  

   pOldNode = list->m_pNodeTail;

   list->m_pNodeTail = pOldNode->pPrev;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = NULL;
   else
      list->m_pNodeHead = NULL;
   utFreeNodeListOfLongs(list, pOldNode);

   return;
}

#ifdef __STDC__
pVoid utInsertBeforeListOfLongs(pListOfLongs list,
                                pVoid position, 
                                long newElement)
#else
pVoid utInsertBeforeListOfLongs(list, position, newElement)
    pListOfLongs list;
    pVoid position; 
    long newElement;
#endif
{
   CNode_L* pOldNode;
   CNode_L* pNewNode;
   /* insert before nothing->head of the list */
   if (position == NULL)
      return utAddHeadListOfLongs(list, newElement); 

   /* utInsert it before position */
   pOldNode = (CNode_L *) position;
   pNewNode = utNewNodeListOfLongs(list, pOldNode->pPrev, pOldNode);
   pNewNode->data = newElement;

   if (pOldNode->pPrev != NULL)
      pOldNode->pPrev->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   pOldNode->pPrev = pNewNode;
   return (pVoid) pNewNode;
}


#ifdef __STDC__
pVoid utInsertAfterListOfLongs(pListOfLongs list,
                               pVoid position, 
                               long newElement)
#else
pVoid utInsertAfterListOfLongs(list, position, newElement)
   pListOfLongs list;
   pVoid position; 
   long newElement;
#endif
{
   CNode_L* pOldNode;
   CNode_L* pNewNode;
   /* insert after nothing->tail of the list */
   if (position == NULL)
      return utAddTailListOfLongs(list, newElement); 

   /* Insert it before position */
   pOldNode = (CNode_L *) position;
   pNewNode = utNewNodeListOfLongs(list, pOldNode, pOldNode->pNext);
   pNewNode->data = newElement;

   if (pOldNode->pNext != NULL)
      pOldNode->pNext->pPrev = pNewNode;
   else
   list->m_pNodeTail = pNewNode;
   pOldNode->pNext = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utRemoveAtListOfLongs(pListOfLongs list, pVoid position)
#else
void utRemoveAtListOfLongs(list, position)
   pListOfLongs list;
   pVoid position;
#endif
{
   CNode_L* pOldNode = (CNode_L *) position;

   /* remove pOldNode from list */
   if (pOldNode == list->m_pNodeHead)
      list->m_pNodeHead = pOldNode->pNext;
   else
      pOldNode->pPrev->pNext = pOldNode->pNext;
   if (pOldNode == list->m_pNodeTail)
      list->m_pNodeTail = pOldNode->pPrev;
   else
      pOldNode->pNext->pPrev = pOldNode->pPrev;
   utFreeNodeListOfLongs(list, pOldNode);
}


/* slow operations */

#ifdef __STDC__
pVoid utGetAtListOfLongs(pListOfLongs list, int nIndex)
#else
pVoid utGetAtListOfLongs(list, nIndex) 
   pListOfLongs list;
   int nIndex;
#endif
{
   CNode_L* pNode;

   assert(nIndex >= 0);

   if (nIndex >= list->m_nCount)
      return NULL;  /* went too far */

   pNode = list->m_pNodeHead;
   while (nIndex--)
   {
      pNode = pNode->pNext;
   }
   return (pVoid) pNode;
}

#ifdef __STDC__
pVoid utFindItemListOfLongs(pListOfLongs list,
                            pVoid startAfter,
                            long searchValue) 
#else
pVoid utFindItemListOfLongs(list, startAfter, searchValue)
   pListOfLongs list; 
   pVoid startAfter;
   long searchValue; 
#endif
{
  CNode_L* pNode = (CNode_L *) startAfter;
  if (pNode == NULL)
    pNode = list->m_pNodeHead;  /* start at head */
  else
    pNode = pNode->pNext;  /* start after the one specified */

  for (; pNode != NULL; pNode = pNode->pNext)
    if (pNode->data == searchValue)
      return (pVoid) pNode;
  return NULL;
}

#ifdef __STDC__
int utGetCountListOfLongs(pListOfLongs list)
#else
int utGetCountListOfLongs(list)
   pListOfLongs list;
#endif
{
  return list->m_nCount;
}

#ifdef __STDC__
pVoid utGetHeadPosListOfLongs(pListOfLongs list)
#else
pVoid utGetHeadPosListOfLongs(list)
   pListOfLongs list;
#endif
{
  return (pVoid) list->m_pNodeHead;
}

#ifdef __STDC__
pVoid utGetTailPosListOfLongs(pListOfLongs list)
#else
pVoid utGetTailPosListOfLongs(list)
   pListOfLongs list;
#endif
{
  assert (list != NULL);
  return (pVoid) list->m_pNodeTail;
}

#ifdef __STDC__
pVoid utGetPrevPosListOfLongs(pVoid pos)
#else
pVoid utGetPrevPosListOfLongs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) ((CNode_L *) pos)->pPrev;
}

#ifdef __STDC__
pVoid utGetNextPosListOfLongs(pVoid pos)
#else
pVoid utGetNextPosListOfLongs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) (((CNode_L *) pos)->pNext);
}

#ifdef __STDC__
long utGetHeadListOfLongs(pListOfLongs list)
#else
long utGetHeadListOfLongs(list)
  pListOfLongs list;
#endif
{
  return utGetDataListOfLongs(utGetHeadPosListOfLongs(list));
}
 
#ifdef __STDC__
long utGetTailListOfLongs(pListOfLongs list)
#else
long utGetTailListOfLongs(list)
  pListOfLongs list;
#endif
{
  return utGetDataListOfLongs(utGetTailPosListOfLongs(list));
}
 

#ifdef __STDC__
long utGetNextListOfLongs(pVoid* pos)
#else
long utGetNextListOfLongs(pos)
  pVoid* pos;
#endif
{
  pVoid prevPos;
  assert(!pos || !*pos); /* do not call with null pos */
  prevPos = *pos;
  *pos = utGetNextPosListOfLongs(pos);
  return utGetDataListOfLongs(utGetNextPosListOfLongs(prevPos));
}

#ifdef __STDC__
long utGetPrevListOfLongs(pVoid* pos)
#else
long utGetPrevListOfLongs(pos)
  pVoid* pos;
#endif
{
  pVoid oldPos;
  assert(!pos || !*pos); /* do not call with null pos */
  oldPos = *pos;
  *pos = utGetPrevPosListOfLongs(pos);
  return utGetDataListOfLongs(utGetPrevPosListOfLongs(oldPos));
}

#ifdef __STDC__
long utGetDataListOfLongs(pVoid pos)
#else
long utGetDataListOfLongs(pos)
  pVoid pos;
#endif
{
  assert (pos != NULL);
  return ((CNode_L *) pos)->data;
}

#ifdef __STDC__
static CNode_L* utCreateNewBlockListOfLongs(pListOfLongs list)
#else
static CNode_L* utCreateNewBlockListOfLongs(list)
pListOfLongs list;
#endif
{
   CPlex_L *p;
   CNode_L *node;
   int      nAssoc = list->m_nBlockSize;
   int      i1;

   assert (nAssoc > 0);

   p = (CPlex_L*)utGetBytes(sizeof(CPlex_L) + sizeof(CNode_L) * nAssoc);
   node = (CNode_L *)(p+1);
   for(i1=0 ; i1<nAssoc ; i1++) {
      node->StructType = STRUCTTYPE_CNODE_L;
      node->pNext      = NULL;
      node->pPrev      = NULL;
      node->data       = 0;
      node++;
   }
   p->StructType = STRUCTTYPE_CPLEX_L;
   p->pNext      = list->m_pBlocks;

   list->m_pBlocks = p;

   return((CNode_L *)(p + 1));
}
 
 
#ifdef __STDC__
static void utFreeDataChainListOfLongs(CPlex_L* p)
#else
static void utFreeDataChainListOfLongs(p)
   CPlex_L* p;
#endif
{
   CPlex_L* pNext;
 
   while (p != NULL)
   {
        pNext = p->pNext;
        utFreeBytes((pVoid)p);
        p = pNext;
   }
}
 
