/*
  $RCSfile: utlistpu.c,v $       $Revision: 1.2 $
  $State: Exp $                  $Date: 1997/05/16 09:50:46 $
*/
#include "uttypesu.h"
#include "utlistpu.h"
#include "utlistpp.h"
#include "utmemryu.h"

#ifdef __STDC__
pListOfPtrs utCreateListOfPtrs(int nBlockSize)
#else
pListOfPtrs utCreateListOfPtrs(nBlockSize)
   int nBlockSize;
#endif
{
  pListOfPtrs list;

  list = (pListOfPtrs)utGetBytes(sizeof(ListOfPtrs));
  utInitializeListOfPtrs(list, nBlockSize);
  return list;
}

#ifdef __STDC__
static void utInitializeListOfPtrs(pListOfPtrs list, int nBlockSize)
#else
static void utInitializeListOfPtrs(list, nBlockSize)
pListOfPtrs list; 
int         nBlockSize;
#endif
{
  assert(nBlockSize > 0);

  list->StructType   = STRUCTTYPE_LISTOFPTRS;
  list->m_nCount     = 0;
  list->m_pNodeHead  = NULL;
  list->m_pNodeTail  = NULL;
  list->m_pNodeFree  = NULL;
  list->m_pBlocks    = NULL;
  list->m_nBlockSize = nBlockSize;
}


#ifdef __STDC__
void utDestroyListOfPtrs(pListOfPtrs list)
#else
void utDestroyListOfPtrs(list)
pListOfPtrs list;
#endif
{
   CNode_P *pNode;

   for(pNode=list->m_pNodeHead ; pNode!=NULL ; pNode=pNode->pNext)
      pNode->data = NULL;

   list->m_nCount = 0;
   list->m_pNodeHead = NULL;
   list->m_pNodeTail = NULL;
   list->m_pNodeFree = NULL;
   utFreeDataChainListOfPtrs(list->m_pBlocks);
   list->m_pBlocks = NULL;

   utFreeBytes((pVoid)list);
}


#ifdef __STDC__
static CNode_P* utNewNodeListOfPtrs(pListOfPtrs list, CNode_P* pPrev, 
                                                         CNode_P* pNext)
#else
static CNode_P* utNewNodeListOfPtrs(list, pPrev, pNext)
   pListOfPtrs list;
   CNode_P* pPrev; 
   CNode_P* pNext;
#endif
{
  int i;
  CNode_P* pNode;
 
  if (list->m_pNodeFree == NULL)
  {
     /* add another block */
     CNode_P * newBlock = utCreateNewBlockListOfPtrs(list);
     /* chain them into free list */
     for (i = 0, pNode = newBlock; i < list->m_nBlockSize-1;
                                                     i++, pNode++)
     {
        pNode->pNext = list->m_pNodeFree;
        list->m_pNodeFree = pNode;
     }
  }

  assert(list->m_pNodeFree != NULL);  /* we must have something */
  pNode = list->m_pNodeFree;
  list->m_pNodeFree = list->m_pNodeFree->pNext;
  list->m_nCount++;
  assert (list->m_nCount > 0);  /* make sure we don't overflow */
  pNode->pPrev = pPrev;
  pNode->pNext = pNext;
  pNode->data = NULL;
  return pNode;
}

#ifdef __STDC__
static void utFreeNodeListOfPtrs(pListOfPtrs list, CNode_P* pNode)
#else
static void utFreeNodeListOfPtrs(list, pNode)
   pListOfPtrs list; 
   CNode_P* pNode;
#endif
{
   pNode->data = NULL;
   pNode->pNext = list->m_pNodeFree;
   list->m_pNodeFree = pNode;
   list->m_nCount--;
   /* make sure we don't underflow */
   assert(list->m_nCount >= 0);  
}

#ifdef __STDC__
pVoid utAddHeadListOfPtrs(pListOfPtrs list, pVoid newElement)
#else
pVoid utAddHeadListOfPtrs(list, newElement)
   pListOfPtrs list; 
   pVoid newElement;
#endif
{
   CNode_P* pNewNode = utNewNodeListOfPtrs(list, NULL, list->m_pNodeHead);

   assert(list != NULL);
   assert(newElement != NULL);

   pNewNode->data = newElement; 
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = pNewNode;
   else
      list->m_pNodeTail = pNewNode;
   list->m_pNodeHead = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
pVoid utAddTailListOfPtrs(pListOfPtrs list, pVoid newElement)
#else
pVoid utAddTailListOfPtrs(list, newElement)
   pListOfPtrs list;  
   pVoid newElement;
#endif
{
   CNode_P* pNewNode = utNewNodeListOfPtrs(list, list->m_pNodeTail, NULL);

   assert (list != NULL);
   assert (newElement != NULL);

   pNewNode->data = newElement;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   list->m_pNodeTail = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utAddListToHeadListOfPtrs(pListOfPtrs list, pListOfPtrs pNewList)
#else
void utAddListToHeadListOfPtrs(list, pNewList)
   pListOfPtrs list;
   pListOfPtrs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements to head (maintain order) */
   pos = utGetTailPosListOfPtrs(pNewList);
   while (pos != NULL)
   {
     utAddHeadListOfPtrs(list, ((CNode_P*) pos)->data);
     pos = utGetPrevPosListOfPtrs(pos);
   }  
}

#ifdef __STDC__
void utAddListToTailListOfPtrs(pListOfPtrs list, pListOfPtrs pNewList)
#else
void utAddListToTailListOfPtrs(list, pNewList)
   pListOfPtrs list;
   pListOfPtrs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements */
   pos = utGetHeadPosListOfPtrs(pNewList);
   while (pos)
   {
     utAddTailListOfPtrs(list, ((CNode_P*) pos)->data);
     pos = utGetNextPosListOfPtrs(pos);
   }
}

#ifdef __STDC__
void utRemoveHeadListOfPtrs(pListOfPtrs list)
#else
void utRemoveHeadListOfPtrs(list)
  pListOfPtrs list;
#endif
{
   CNode_P* pOldNode;

   assert(list != NULL);
   /* don't call on empty list !!! */
   assert(list->m_pNodeHead != NULL);  

   pOldNode = list->m_pNodeHead;

   list->m_pNodeHead = pOldNode->pNext;
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = NULL;
   else
      list->m_pNodeTail = NULL;
   utFreeNodeListOfPtrs(list, pOldNode);

   return;
}

#ifdef __STDC__
void utRemoveTailListOfPtrs(pListOfPtrs list)
#else
void utRemoveTailListOfPtrs(list)
   pListOfPtrs list;
#endif
{
   CNode_P* pOldNode;

   /* don't call on empty list !!! */
   assert(list->m_pNodeTail != NULL);  

   pOldNode = list->m_pNodeTail;

   list->m_pNodeTail = pOldNode->pPrev;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = NULL;
   else
      list->m_pNodeHead = NULL;
   utFreeNodeListOfPtrs(list, pOldNode);

   return;
}

#ifdef __STDC__
pVoid utInsertBeforeListOfPtrs(pListOfPtrs list,
                               pVoid position, 
                               pVoid newElement)
#else
pVoid utInsertBeforeListOfPtrs(list, position, newElement)
    pListOfPtrs list;
    pVoid position; 
    pVoid newElement;
#endif
{
   CNode_P* pOldNode;
   CNode_P* pNewNode;
   /* insert before nothing->head of the list */
   if (position == NULL)
      return utAddHeadListOfPtrs(list, newElement); 

   /* utInsert it before position */
   pOldNode = (CNode_P *) position;
   pNewNode = utNewNodeListOfPtrs(list, pOldNode->pPrev, pOldNode);
   pNewNode->data = newElement;

   if (pOldNode->pPrev != NULL)
      pOldNode->pPrev->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   pOldNode->pPrev = pNewNode;
   return (pVoid) pNewNode;
}


#ifdef __STDC__
pVoid utInsertAfterListOfPtrs(pListOfPtrs list,
                              pVoid position, 
                              pVoid newElement)
#else
pVoid utInsertAfterListOfPtrs(list, position, newElement)
   pListOfPtrs list;
   pVoid position; 
   pVoid newElement;
#endif
{
   CNode_P* pOldNode;
   CNode_P* pNewNode;
   /* insert after nothing->tail of the list */
   if (position == NULL)
      return utAddTailListOfPtrs(list, newElement); 

   /* Insert it before position */
   pOldNode = (CNode_P *) position;
   pNewNode = utNewNodeListOfPtrs(list, pOldNode, pOldNode->pNext);
   pNewNode->data = newElement;

   if (pOldNode->pNext != NULL)
      pOldNode->pNext->pPrev = pNewNode;
   else
   list->m_pNodeTail = pNewNode;
   pOldNode->pNext = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utRemoveAtListOfPtrs(pListOfPtrs list, pVoid position)
#else
void utRemoveAtListOfPtrs(list, position)
   pListOfPtrs list;
   pVoid position;
#endif
{
   CNode_P* pOldNode = (CNode_P *) position;

   /* remove pOldNode from list */
   if (pOldNode == list->m_pNodeHead)
      list->m_pNodeHead = pOldNode->pNext;
   else
      pOldNode->pPrev->pNext = pOldNode->pNext;
   if (pOldNode == list->m_pNodeTail)
      list->m_pNodeTail = pOldNode->pPrev;
   else
      pOldNode->pNext->pPrev = pOldNode->pPrev;
   utFreeNodeListOfPtrs(list, pOldNode);
}


/* slow operations */

#ifdef __STDC__
pVoid utGetAtListOfPtrs(pListOfPtrs list, int nIndex)
#else
pVoid utGetAtListOfPtrs(list, nIndex) 
   pListOfPtrs list;
   int nIndex;
#endif
{
   CNode_P* pNode;

   assert(nIndex >= 0);

   if (nIndex >= list->m_nCount)
      return NULL;  /* went too far */

   pNode = list->m_pNodeHead;
   while (nIndex--)
   {
      pNode = pNode->pNext;
   }
   return (pVoid) pNode;
}

#ifdef __STDC__
pVoid utFindItemListOfPtrs(pListOfPtrs list,
                           pVoid startAfter,
                           pVoid searchValue) 
#else
pVoid utFindItemListOfPtrs(list,
                           startAfter,
                           searchValue)
pListOfPtrs list; 
pVoid startAfter;
pVoid searchValue; 
#endif
{
  CNode_P* pNode = (CNode_P *) startAfter;
  if (pNode == NULL)
    pNode = list->m_pNodeHead;  /* start at head */
  else
    pNode = pNode->pNext;  /* start after the one specified */

  for (; pNode != NULL; pNode = pNode->pNext)
    if (pNode->data == searchValue)
      return (pVoid) pNode;
  return NULL;
}

#ifdef __STDC__
int utGetCountListOfPtrs(pListOfPtrs list)
#else
int utGetCountListOfPtrs(list)
   pListOfPtrs list;
#endif
{
  assert (list != NULL);
  return list->m_nCount;
}

#ifdef __STDC__
pVoid utGetHeadPosListOfPtrs(pListOfPtrs list)
#else
pVoid utGetHeadPosListOfPtrs(list)
   pListOfPtrs list;
#endif
{
  assert (list != NULL);
  return (pVoid) list->m_pNodeHead;
}

#ifdef __STDC__
pVoid utGetTailPosListOfPtrs(pListOfPtrs list)
#else
pVoid utGetTailPosListOfPtrs(list)
   pListOfPtrs list;
#endif
{
  assert (list != NULL);
  return (pVoid) list->m_pNodeTail;
}

#ifdef __STDC__
pVoid utGetPrevPosListOfPtrs(pVoid pos)
#else
pVoid utGetPrevPosListOfPtrs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) ((CNode_P *) pos)->pPrev;
}

#ifdef __STDC__
pVoid utGetNextPosListOfPtrs(pVoid pos)
#else
pVoid utGetNextPosListOfPtrs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) (((CNode_P *) pos)->pNext);
}

#ifdef __STDC__
pVoid utGetHeadListOfPtrs(pListOfPtrs list)
#else
pVoid utGetHeadListOfPtrs(list)
  pListOfPtrs list;
#endif
{
  assert (list != NULL);
  return utGetDataListOfPtrs(utGetHeadPosListOfPtrs(list));
}
 
#ifdef __STDC__
pVoid utGetTailListOfPtrs(pListOfPtrs list)
#else
pVoid utGetTailListOfPtrs(list)
  pListOfPtrs list;
#endif
{
  assert (list != NULL);
  return utGetDataListOfPtrs(utGetTailPosListOfPtrs(list));
}
 

#ifdef __STDC__
pVoid utGetNextListOfPtrs(pVoid* pos)
#else
pVoid utGetNextListOfPtrs(pos)
  pVoid* pos;
#endif
{
  pVoid prevPos;
  assert(!pos || !*pos); /* do not call with null pos */
  prevPos = *pos;
  *pos = utGetNextPosListOfPtrs(pos);
  return utGetDataListOfPtrs(utGetNextPosListOfPtrs(prevPos));
}

#ifdef __STDC__
pVoid utGetPrevListOfPtrs(pVoid* pos)
#else
pVoid utGetPrevListOfPtrs(pos)
  pVoid* pos;
#endif
{
  pVoid oldPos;
  assert(!pos || !*pos); /* do not call with null pos */
  oldPos = *pos;
  *pos = utGetPrevPosListOfPtrs(pos);
  return utGetDataListOfPtrs(utGetPrevPosListOfPtrs(oldPos));
}

#ifdef __STDC__
pVoid utGetDataListOfPtrs(pVoid pos)
#else
pVoid utGetDataListOfPtrs(pos)
  pVoid pos;
#endif
{
  assert (pos != NULL);
  return ((CNode_P *) pos)->data;
}

#ifdef __STDC__
static CNode_P* utCreateNewBlockListOfPtrs(pListOfPtrs list)
#else
static CNode_P* utCreateNewBlockListOfPtrs(list)
pListOfPtrs list;
#endif
{
   CPlex_P *p;
   CNode_P *node;
   int      nAssoc = list->m_nBlockSize;
   int      i1;

   assert (nAssoc > 0);

   p = (CPlex_P*)utGetBytes(sizeof(CPlex_P) + sizeof(CNode_P) * nAssoc);
   node = (CNode_P *)(p+1);
   for(i1=0 ; i1<nAssoc ; i1++) {
      node->StructType = STRUCTTYPE_CNODE_P;
      node->pNext      = NULL;
      node->pPrev      = NULL;
      node->data       = NULL;
      node++;
   }
   p->StructType = STRUCTTYPE_CPLEX_P;
   p->pNext      = list->m_pBlocks;

   list->m_pBlocks = p;

   return((CNode_P *)(p + 1));
}
 
 
#ifdef __STDC__
static void utFreeDataChainListOfPtrs(CPlex_P *p)
#else
static void utFreeDataChainListOfPtrs(p)
CPlex_P *p;
#endif
{
   CPlex_P* pNext;
 
   while (p != NULL) {
      pNext = p->pNext;
      utFreeBytes((pVoid)p);
      p = pNext;
   }
}

/*****************************************************
 * search for an item and remove its first occurrence.
 * return true (1) if found and removed th item.
 * return false (0) otherwise.
 *****************************************************/
 
#ifdef __STDC__
utLogical utRemoveItemListOfPtrs(pListOfPtrs list,pVoid item)
#else
utLogical utRemoveItemListOfPtrs(list, item)
  pListOfPtrs list;
  pVoid item;
#endif
{
  pVoid pos;

  assert (list != NULL);
  assert (item != NULL);

  pos = utFindItemListOfPtrs(list, NULL, item);
  if (pos == NULL) return(utFalse);
  utRemoveAtListOfPtrs(list, pos);  

  return(utTrue);
}


#ifdef __STDC__
pVoid utGetPtrAtPositionInListOfPtrs(pListOfPtrs list, int nIndex)
#else
pVoid utGetPtrAtPositionInListOfPtrs(list, nIndex) 
pListOfPtrs list;
int         nIndex;
#endif
{
   return utGetDataListOfPtrs(utGetAtListOfPtrs(list,nIndex));
}
