/*
  $RCSfile: utlistsu.c,v $       $Revision: 1.1 $
  $State: V6_0 $                  $Date: 1997/04/21 11:01:29 $
*/
#include "uttypesu.h"
#include "utmemryu.h"
#include "utstrngu.h"
#include "utlistsu.h"
#include "utlistsp.h"

#ifdef __STDC__
pListOfStrs utCreateListOfStrs(int nBlockSize)
#else
pListOfStrs utCreateListOfStrs(nBlockSize)
   int nBlockSize;
#endif
{
  pListOfStrs list;

  list = (pListOfStrs) utGetBytes(sizeof(ListOfStrs));
  utInitializeListOfStrs(list, nBlockSize);
  return list;
}

#ifdef __STDC__
static void utInitializeListOfStrs(pListOfStrs list, int nBlockSize)
#else
static void utInitializeListOfStrs(list, nBlockSize)
   pListOfStrs list; 
   int nBlockSize;
#endif
{
  assert(nBlockSize > 0);

  list->StructType   = STRUCTTYPE_LISTOFSTRS;
  list->m_nCount     = 0;
  list->m_pNodeHead  = NULL;
  list->m_pNodeTail  = NULL;
  list->m_pNodeFree  = NULL;
  list->m_pBlocks    = NULL;
  list->m_nBlockSize = nBlockSize;
}

#ifdef __STDC__
void utDestroyListOfStrs(pListOfStrs list)
#else
void utDestroyListOfStrs(list)
pListOfStrs list;
#endif
{
   CNode_S *pNode;

   for(pNode=list->m_pNodeHead ; pNode!=NULL ; pNode=pNode->pNext) {
      utFreeBytes((pVoid)pNode->data);
      pNode->data = NULL;
   }

   list->m_nCount = 0;
   list->m_pNodeHead = NULL;
   list->m_pNodeTail = NULL;
   list->m_pNodeFree = NULL;
   utFreeDataChainListOfStrs(list->m_pBlocks);
   list->m_pBlocks = NULL;

   utFreeBytes((pVoid)list);
}

#ifdef __STDC__
static CNode_S* utNewNodeListOfStrs(pListOfStrs list, CNode_S* pPrev, 
                                                         CNode_S* pNext)
#else
static CNode_S* utNewNodeListOfStrs(list, pPrev, pNext)
   pListOfStrs list;
   CNode_S* pPrev; 
   CNode_S* pNext;
#endif
{
  int i;
  CNode_S* pNode;
 
  if (list->m_pNodeFree == NULL)
  {
     /* add another block */
     CNode_S * newBlock = utCreateNewBlockListOfStrs(list);
     /* chain them into free list */
     for (i = 0, pNode = newBlock; i < list->m_nBlockSize-1;
                                                     i++, pNode++)
     {
        pNode->pNext = list->m_pNodeFree;
        list->m_pNodeFree = pNode;
     }
  }

  assert(list->m_pNodeFree != NULL);  /* we must have something */
  pNode = list->m_pNodeFree;
  list->m_pNodeFree = list->m_pNodeFree->pNext;
  list->m_nCount++;
  assert (list->m_nCount > 0);  /* make sure we don't overflow */
  pNode->pPrev = pPrev;
  pNode->pNext = pNext;
  pNode->data = NULL;

  return pNode;
}

#ifdef __STDC__
static void utFreeNodeListOfStrs(pListOfStrs list, CNode_S* pNode)
#else
static void utFreeNodeListOfStrs(list, pNode)
   pListOfStrs list; 
   CNode_S* pNode;
#endif
{
   pNode->data = NULL;
   pNode->pNext = list->m_pNodeFree;
   list->m_pNodeFree = pNode;
   list->m_nCount--;
   /* make sure we don't underflow */
   assert(list->m_nCount >= 0);  
}

#ifdef __STDC__
pVoid utAddHeadListOfStrs(pListOfStrs list, char* newElement)
#else
pVoid utAddHeadListOfStrs(list, newElement)
   pListOfStrs list; 
   char* newElement;
#endif
{
   CNode_S* pNewNode = utNewNodeListOfStrs(list, NULL, list->m_pNodeHead);

   assert(list != NULL);
   assert(newElement != NULL);

   pNewNode->data = utAssignString(newElement); 
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = pNewNode;
   else
      list->m_pNodeTail = pNewNode;
   list->m_pNodeHead = pNewNode;

   return (pVoid) pNewNode;
}

#ifdef __STDC__
pVoid utAddTailListOfStrs(pListOfStrs list, char* newElement)
#else
pVoid utAddTailListOfStrs(list, newElement)
   pListOfStrs list;  
   char* newElement;
#endif
{
   CNode_S* pNewNode = utNewNodeListOfStrs(list, list->m_pNodeTail, NULL);

   assert (list != NULL);
   assert (newElement != NULL);

   pNewNode->data = utAssignString(newElement);
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   list->m_pNodeTail = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utAddListToHeadListOfStrs(pListOfStrs list, pListOfStrs pNewList)
#else
void utAddListToHeadListOfStrs(list, pNewList)
   pListOfStrs list;
   pListOfStrs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements to head (maintain order) */
   pos = utGetTailPosListOfStrs(pNewList);
   while (pos != NULL)
   {
     utAddHeadListOfStrs(list, ((CNode_S*) pos)->data);
     pos = utGetPrevPosListOfStrs(pos);
   }  
}

#ifdef __STDC__
void utAddListToTailListOfStrs(pListOfStrs list, pListOfStrs pNewList)
#else
void utAddListToTailListOfStrs(list, pNewList)
   pListOfStrs list;
   pListOfStrs pNewList;
#endif
{
   pVoid pos;
   assert(list != NULL);
   assert(pNewList != NULL);

   /* add a list of same elements */
   pos = utGetHeadPosListOfStrs(pNewList);
   while (pos)
   {
     utAddTailListOfStrs(list, ((CNode_S*) pos)->data);
     pos = utGetNextPosListOfStrs(pos);
   }
}

#ifdef __STDC__
void utRemoveHeadListOfStrs(pListOfStrs list)
#else
void utRemoveHeadListOfStrs(list)
  pListOfStrs list;
#endif
{
   CNode_S* pOldNode;

   assert(list != NULL);
   /* don't call on empty list !!! */
   assert(list->m_pNodeHead != NULL);  

   pOldNode = list->m_pNodeHead;

   list->m_pNodeHead = pOldNode->pNext;
   if (list->m_pNodeHead != NULL)
      list->m_pNodeHead->pPrev = NULL;
   else
      list->m_pNodeTail = NULL;
   utFreeNodeListOfStrs(list, pOldNode);

   return;
}

#ifdef __STDC__
void utRemoveTailListOfStrs(pListOfStrs list)
#else
void utRemoveTailListOfStrs(list)
   pListOfStrs list;
#endif
{
   CNode_S* pOldNode;

   /* don't call on empty list !!! */
   assert(list->m_pNodeTail != NULL);  

   pOldNode = list->m_pNodeTail;

   list->m_pNodeTail = pOldNode->pPrev;
   if (list->m_pNodeTail != NULL)
      list->m_pNodeTail->pNext = NULL;
   else
      list->m_pNodeHead = NULL;
   utFreeNodeListOfStrs(list, pOldNode);

   return;
}

#ifdef __STDC__
pVoid utInsertBeforeListOfStrs(pListOfStrs list,
                               pVoid position, 
                               char* newElement)
#else
pVoid utInsertBeforeListOfStrs(list, position, newElement)
    pListOfStrs list;
    pVoid position; 
    char* newElement;
#endif
{
   CNode_S* pOldNode;
   CNode_S* pNewNode;
   /* insert before nothing->head of the list */
   if (position == NULL)
      return utAddHeadListOfStrs(list, newElement); 

   /* utInsert it before position */
   pOldNode = (CNode_S *) position;
   pNewNode = utNewNodeListOfStrs(list, pOldNode->pPrev, pOldNode);
   pNewNode->data = utAssignString(newElement);

   if (pOldNode->pPrev != NULL)
      pOldNode->pPrev->pNext = pNewNode;
   else
      list->m_pNodeHead = pNewNode;
   pOldNode->pPrev = pNewNode;
   return (pVoid) pNewNode;
}


#ifdef __STDC__
pVoid utInsertAfterListOfStrs(pListOfStrs list,
                              pVoid position, 
                              char* newElement)
#else
pVoid utInsertAfterListOfStrs(list, position, newElement)
   pListOfStrs list;
   pVoid position; 
   char* newElement;
#endif
{
   CNode_S* pOldNode;
   CNode_S* pNewNode;
   /* insert after nothing->tail of the list */
   if (position == NULL)
      return utAddTailListOfStrs(list, newElement); 

   /* Insert it before position */
   pOldNode = (CNode_S *) position;
   pNewNode = utNewNodeListOfStrs(list, pOldNode, pOldNode->pNext);
   pNewNode->data = utAssignString(newElement);

   if (pOldNode->pNext != NULL)
      pOldNode->pNext->pPrev = pNewNode;
   else
   list->m_pNodeTail = pNewNode;
   pOldNode->pNext = pNewNode;
   return (pVoid) pNewNode;
}

#ifdef __STDC__
void utRemoveAtListOfStrs(pListOfStrs list, pVoid position)
#else
void utRemoveAtListOfStrs(list, position)
   pListOfStrs list;
   pVoid position;
#endif
{
   CNode_S* pOldNode = (CNode_S *) position;

   /* remove pOldNode from list */
   if (pOldNode == list->m_pNodeHead)
      list->m_pNodeHead = pOldNode->pNext;
   else
      pOldNode->pPrev->pNext = pOldNode->pNext;
   if (pOldNode == list->m_pNodeTail)
      list->m_pNodeTail = pOldNode->pPrev;
   else
      pOldNode->pNext->pPrev = pOldNode->pPrev;
   utFreeNodeListOfStrs(list, pOldNode);
}


/* slow operations */

#ifdef __STDC__
pVoid utGetAtListOfStrs(pListOfStrs list, int nIndex)
#else
pVoid utGetAtListOfStrs(list, nIndex) 
   pListOfStrs list;
   int nIndex;
#endif
{
   CNode_S* pNode;

   assert(nIndex >= 0);

   if (nIndex >= list->m_nCount)
      return NULL;  /* went too far */

   pNode = list->m_pNodeHead;
   while (nIndex--)
   {
      pNode = pNode->pNext;
   }
   return (pVoid) pNode;
}

#ifdef __STDC__
pVoid utFindItemListOfStrs(pListOfStrs list,
                           pVoid startAfter,
                           char* searchValue) 
#else
pVoid utFindItemListOfStrs(list, startAfter, searchValue)
   pListOfStrs list; 
   pVoid startAfter;
   char* searchValue; 
#endif
{
  CNode_S* pNode = (CNode_S *) startAfter;
  if (pNode == NULL)
    pNode = list->m_pNodeHead;  /* start at head */
  else
    pNode = pNode->pNext;  /* start after the one specified */

  for (; pNode != NULL; pNode = pNode->pNext)
    if (utEqualStrings(pNode->data,searchValue))
      return (pVoid) pNode;
  return NULL;
}

#ifdef __STDC__
int utGetCountListOfStrs(pListOfStrs list)
#else
int utGetCountListOfStrs(list)
   pListOfStrs list;
#endif
{
  return list->m_nCount;
}

#ifdef __STDC__
pVoid utGetHeadPosListOfStrs(pListOfStrs list)
#else
pVoid utGetHeadPosListOfStrs(list)
   pListOfStrs list;
#endif
{
  return (pVoid) list->m_pNodeHead;
}

#ifdef __STDC__
pVoid utGetTailPosListOfStrs(pListOfStrs list)
#else
pVoid utGetTailPosListOfStrs(list)
   pListOfStrs list;
#endif
{
  assert (list != NULL);
  return (pVoid) list->m_pNodeTail;
}

#ifdef __STDC__
pVoid utGetPrevPosListOfStrs(pVoid pos)
#else
pVoid utGetPrevPosListOfStrs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) ((CNode_S *) pos)->pPrev;
}

#ifdef __STDC__
pVoid utGetNextPosListOfStrs(pVoid pos)
#else
pVoid utGetNextPosListOfStrs(pos)
   pVoid pos;
#endif
{
   assert (pos != NULL);
   return (pVoid) (((CNode_S *) pos)->pNext);
}

#ifdef __STDC__
char* utGetHeadListOfStrs(pListOfStrs list)
#else
char* utGetHeadListOfStrs(list)
  pListOfStrs list;
#endif
{
  return utGetDataListOfStrs(utGetHeadPosListOfStrs(list));
}
 
#ifdef __STDC__
char* utGetTailListOfStrs(pListOfStrs list)
#else
char* utGetTailListOfStrs(list)
  pListOfStrs list;
#endif
{
  return utGetDataListOfStrs(utGetTailPosListOfStrs(list));
}
 

#ifdef __STDC__
char* utGetNextListOfStrs(pVoid* pos)
#else
char* utGetNextListOfStrs(pos)
  pVoid* pos;
#endif
{
  pVoid prevPos;
  assert(!pos || !*pos); /* do not call with null pos */
  prevPos = *pos;
  *pos = utGetNextPosListOfStrs(pos);
  return utGetDataListOfStrs(utGetNextPosListOfStrs(prevPos));
}

#ifdef __STDC__
char* utGetPrevListOfStrs(pVoid* pos)
#else
char* utGetPrevListOfStrs(pos)
  pVoid* pos;
#endif
{
  pVoid oldPos;
  assert(!pos || !*pos); /* do not call with null pos */
  oldPos = *pos;
  *pos = utGetPrevPosListOfStrs(pos);
  return utGetDataListOfStrs(utGetPrevPosListOfStrs(oldPos));
}

#ifdef __STDC__
char* utGetDataListOfStrs(pVoid pos)
#else
char* utGetDataListOfStrs(pos)
  pVoid pos;
#endif
{
  assert (pos != NULL);
  return ((CNode_S *) pos)->data;
}

#ifdef __STDC__
static CNode_S* utCreateNewBlockListOfStrs(pListOfStrs list)
#else
static CNode_S* utCreateNewBlockListOfStrs(list)
pListOfStrs list;
#endif
{
   CPlex_S *p;
   CNode_S *node;
   int      nAssoc = list->m_nBlockSize;
   int      i1;

   assert (nAssoc > 0);

   p = (CPlex_S*)utGetBytes(sizeof(CPlex_S) + sizeof(CNode_S) * nAssoc);
   node = (CNode_S *)(p+1);
   for(i1=0 ; i1<nAssoc ; i1++) {
      node->StructType = STRUCTTYPE_CNODE_S;
      node->pNext      = NULL;
      node->pPrev      = NULL;
      node->data       = NULL;
      node++;
   }
   p->StructType = STRUCTTYPE_CPLEX_S;
   p->pNext      = list->m_pBlocks;

   list->m_pBlocks = p;

   return((CNode_S *)(p + 1));
}
 
 
#ifdef __STDC__
static void utFreeDataChainListOfStrs(CPlex_S* p)
#else
static void utFreeDataChainListOfStrs(p)
CPlex_S* p;
#endif
{
   CPlex_S* pNext;
 
   while (p != NULL) {
      pNext = p->pNext;
      utFreeBytes((pVoid)p);
      p = pNext;
   }
}
 
