/*
  $RCSfile: utsarryu.c,v $       $Revision: 1.13 $
  $State: V7_1 $                  $Date: 2004/12/14 12:50:11 $
*/
#include "uttypesu.h"
#include "utmacrou.h"
#include "utmemryu.h"
#include "utstrngu.h"
#include "utsarryp.h"
#include "utsarryu.h"
#include "utstriou.h"

/*********************************************
 * create an Sarray and return a pointer to it
 * the sarray is initialized to default values
 *********************************************/

#ifdef __STDC__
pSarray utCreateSarray()
#else
pSarray utCreateSarray()
#endif
{
   pSarray newSarray;

   newSarray = (pSarray)utGetBytes(sizeof(Sarray));
   utInitializeSarray(newSarray);

   return(newSarray);
}

/******************************************
 * initialize an Sarray to dafault values.
 * This is a helper function used by other
 * functions. User should not directly call
 ******************************************/

#ifdef __STDC__
void utInitializeSarray(pSarray sarray)
#else
void utInitializeSarray(sarray)
pSarray sarray;
#endif
{
   sarray->StructType = STRUCTTYPE_SARRAY;
   sarray->nStrings   = 0;
   sarray->nMaxSize   = 0;
   sarray->strings    = NULL;
}

/************************************************
 * Allocate memory to hold a specified maximum
 * number of strings. This avoids expense of
 * growing the array for each addition. Use
 * this function if you know the number at start.
 ************************************************/

#ifdef __STDC__
pSarray utCreateSarrayOfSize(int nMaxSize)
#else
pSarray utCreateSarrayOfSize(nMaxSize)
int nMaxSize;
#endif
{
   int     i1;
   pSarray sarray = utCreateSarray();

   sarray->nMaxSize = nMaxSize;
   sarray->strings = (char **)utGetBytes(nMaxSize*sizeof(char *));

   for(i1=0 ; i1<nMaxSize ; i1++)
      sarray->strings[i1] = NULL;

   return(sarray);
}

/*******************************************
 * create an Sarray and copy the contents of
 * a given Sarray to it. Return a pointer to
 * the copy.
 *******************************************/

#ifdef __STDC__
pSarray utAssignSarray(pSarray sarray)
#else
pSarray utAssignSarray(sarray)
pSarray sarray;
#endif
{
   pSarray newSarray;
   int     i1;

   newSarray = utCreateSarrayOfSize(sarray->nMaxSize);

   newSarray->nStrings = sarray->nStrings;
   for(i1=0 ; i1<sarray->nStrings ; i1++)
      newSarray->strings[i1] = utAssignString(sarray->strings[i1]);

   return(newSarray);
}

/*******************************************
 * Add a string to the end of an Sarray. The
 * array will grow if necessary. The array
 * will make a local copy of the string.
 *******************************************/

#ifdef __STDC__
void utAddToSarray(pSarray sarray,
                   char *cstring)
#else
void utAddToSarray(sarray,
                   cstring)
pSarray  sarray;
char    *cstring;
#endif
{
   int    nSize      = sarray->nStrings;
   int    nMaxSize   = sarray->nMaxSize;
   char **newStrings = NULL;

   if(nSize < nMaxSize) {
     /* no need to create room */
      sarray->nStrings += 1;
      sarray->strings[nSize] = utAssignString(cstring);
   } else {  /* nSize = nMaxSize */
     /* need to create room */
      newStrings = (char **)utGetBytes((nMaxSize+SARRAY_GROWBY_SIZE) * sizeof (char *));
      sarray->nMaxSize = nMaxSize+SARRAY_GROWBY_SIZE;
      sarray->nStrings = nSize+1;

      if(nSize > 0)
         memcpy(newStrings,sarray->strings,nSize*sizeof(char *));

      newStrings[nSize] = utAssignString(cstring);

      utFreeBytes((pVoid)sarray->strings);
      sarray->strings = newStrings;
   }
}

/********************************************
 * insert a string at the ith position
 * of an Sarray. The function checks for
 * valid range before insertion.
 * If you want
 * to insert at the end with the possibility
 * of growing, use Add operation.
 ********************************************/

#ifdef __STDC__
void utInsertInSarray(pSarray sarray,
                      int ith,
                      char *cstring)
#else
void utInsertInSarray(sarray,
                      ith,
                      cstring)
pSarray  sarray;
int      ith;
char    *cstring;
#endif
{
   int    nSize      = sarray->nStrings;
   int    nMaxSize   = sarray->nMaxSize;
   char **newStrings = NULL;

   assert(ith >= 0);
   assert(ith < nSize);

   if(nSize < nMaxSize) {
      newStrings = (char **)utGetBytes(nMaxSize*sizeof(char *));
      sarray->nStrings = nSize+1;
   } else {
      newStrings = (char **)utGetBytes((nMaxSize+SARRAY_GROWBY_SIZE)*sizeof(char *));
      sarray->nStrings = nSize+1;
      sarray->nMaxSize = nMaxSize+SARRAY_GROWBY_SIZE;
   }

   if(ith != 0)
      memcpy(newStrings,sarray->strings,ith*sizeof(char *));

   newStrings[ith] = utAssignString(cstring);

   memcpy(&(newStrings[ith+1]),&(sarray->strings[ith]),(nSize-ith)*sizeof(char *));

   utFreeBytes((pVoid)sarray->strings);
   sarray->strings = newStrings;
}

/************************************
 * deletes an ith item in the Sarray.
 * Checks the range to make sure.
 ************************************/

#ifdef __STDC__
void utDeleteFromSarray(pSarray sarray,
                        int ith)
#else
void utDeleteFromSarray(sarray,
                        ith)
pSarray sarray;
int     ith;
#endif
{
   int    nSize      = sarray->nStrings;
   int    nMaxSize   = sarray->nMaxSize;
   char **newStrings = NULL;

   assert (ith >= 0);
   assert (ith < nSize);

   newStrings = (char **)utGetBytes(nMaxSize*sizeof(char *));
   sarray->nStrings = nSize-1;
   sarray->nMaxSize = nMaxSize;

   memcpy(newStrings,sarray->strings,ith*sizeof(char *));

   memcpy(&(newStrings[ith]),&(sarray->strings[ith+1]),(nSize-ith-1)*sizeof(char *));

   utFreeBytes((pVoid)sarray->strings[ith]);
   utFreeBytes((pVoid)sarray->strings);
   sarray->strings = newStrings;
}


#ifdef __STDC__
void utRemoveFromSarray(pSarray sarray,
                        char *value)
#else
void utRemoveFromSarray(sarray,
                        value)
pSarray  sarray;
char    *value;
#endif
{
   int position = utStringPositionInSarray(value,sarray);

   if(position >= 0)
      utDeleteFromSarray(sarray,position);
}

/*********************************************
 * empty the contents of an sarray. This frees
 * the memory for strings.
 *********************************************/

#ifdef __STDC__
void utEmptySarray(pSarray sarray)
#else
void utEmptySarray(sarray)
pSarray sarray;
#endif
{
   int i1;

   for(i1=0 ; i1<sarray->nStrings ; i1++)
      utFreeBytes((pVoid)sarray->strings[i1]);

   utFreeBytes((pVoid)sarray->strings);
   utInitializeSarray(sarray);
}

/*********************************************
 * free the Sarray object.  This frees all the
 * strings inside and then frees the memory of
 * the Sarray object itself.
 *********************************************/

#ifdef __STDC__
void utFreeSarray(pSarray *sarray)
#else
void utFreeSarray(sarray)
pSarray *sarray;
#endif
{
   if(*sarray == NULL)
      return;

   utEmptySarray(*sarray);

   utFreeBytes((pVoid)*sarray);
   *sarray = NULL;
}

/**************************************
 * compare two strings in an Sarray.
 * This is a helper function used by
 * the SortSarray function. If you need
 * to compare two cstrings use the
 * general utility utEqualStrings
 **************************************/

#ifdef __STDC__
int utCompareStrings(pConstVoid s1,
                     pConstVoid s2)
#else
int utCompareStrings(s1,
                     s2)
pVoid s1;
pVoid s2;
#endif
{
   return(strcmp(*(char **)s1,*(char **)s2));
}

/**********************************************
 * sorts a string array in a case sensitive way.
 **********************************************/

#ifdef __STDC__
void utSortSarray(pSarray sarray)
#else
void utSortSarray(sarray)
pSarray sarray;
#endif
{
   int nStrings = sarray->nStrings;

   if(nStrings > 1)
      qsort((pVoid)sarray->strings,nStrings,sizeof(char *),utCompareStrings);
}

/*************************************
 * checks if a cstring is in an Sarray
 * returns non-zero if it is.
 * returns zero if it is not.
 *************************************/

#ifdef __STDC__
utLogical utStringIsInSarray(char *cstring,
                             pSarray sarray)
#else
utLogical utStringIsInSarray(cstring,
                             sarray)
char    *cstring;
pSarray  sarray;
#endif
{
   int i1;

   for(i1=0 ; i1<sarray->nStrings ; i1++)
      if(utEqualStrings(cstring,sarray->strings[i1]))
         return(utTrue);

   return(utFalse);
}


#ifdef __STDC__
int utStringPositionInSarray(char *cString,
                             pSarray sarray)
#else
int utStringPositionInSarray(cString,
                             sarray)
char    *cString;
pSarray  sarray;
#endif
{
   int i1;

   if(cString == NULL)
      return(-1);

   for(i1=0 ; i1<sarray->nStrings ; i1++)
      if(utEqualStrings(cString,sarray->strings[i1]))
         return(i1);

   return(-1);
}

/**********************************************
 * Returns a new sarray which has only the
 * unique elements in the Sarray passed.
 * uses utEqualStrings to check for uniqueness.
 **********************************************/

#ifdef __STDC__
pSarray utGetUniqueSarray(pSarray sarray)
#else
pSarray utGetUniqueSarray(sarray)
pSarray sarray;
#endif
{
   pSarray copySarray;
   pSarray uniqueSarray;
   int     i1,indx;

  /*****************************************
   * if the original sarray had zero strings
   * return a blank sarray
   *****************************************/

   if(sarray->nStrings == 0)
      return(utCreateSarray());

  /*****************************************
   * we need to sort sarray. Since we do not
   * want to change the original sarray, we
   * make a copy and then sort it.
   *****************************************/

   copySarray = utAssignSarray(sarray);
   utSortSarray(copySarray);

  /********************************************
   * create a blank uniqueSarray. Add a string
   * from copySarray only if it is not a
   * duplicate string.
   ********************************************/

   uniqueSarray = utCreateSarray();
   indx = 0;
   utAddToSarray(uniqueSarray,copySarray->strings[indx]);

   for(i1=1 ; i1<copySarray->nStrings ; i1++) {
      if(!utEqualStrings(copySarray->strings[indx],copySarray->strings[i1])) {
         indx = i1;
         utAddToSarray(uniqueSarray,copySarray->strings[indx]);
      }
   }
   utFreeSarray(&copySarray);

   return(uniqueSarray);
}


#ifdef __STDC__
int utGetCountSarray(pSarray sarray)
#else
int utGetCountSarray(sarray)
pSarray sarray;
#endif
{
   assert(sarray != NULL);

   return(sarray->nStrings);
}

/*
 * A word of caution: When you convert a CstringArray to an
 * sarray, be clear as to where the memory should be freed.
 */

#ifdef __STDC__
pSarray utCreateSarrayFromCstringArray(int nCount,
                                       char ***cStringArray)
#else
pSarray utCreateSarrayFromCstringArray(nCount,
                                       cStringArray)
int     nCount;
char ***cStringArray;
#endif
{
   pSarray sarray;

   assert(cStringArray != NULL);
   assert(nCount > 0);

   sarray = utCreateSarray();
   sarray->nStrings = nCount;
   sarray->nMaxSize = nCount;
   sarray->strings = *cStringArray;
   *cStringArray = NULL;

   return(sarray);
}


#ifdef __STDC__
char *utGetCstringAtPositionInSarray(pSarray sarray,
                                     int position)
#else
char *utGetCstringAtPositionInSarray(sarray,
                                     position)
pSarray sarray;
int     position;
#endif
{
   assert(sarray != NULL);
   assert((position >= 0) && (position <= (sarray->nStrings-1)));

   return(sarray->strings[position]);
}


#ifdef __STDC__
void utSetCstringAtPositionInSarray(pSarray sarray,
                                    int position,
                                    char *cString)
#else
void utSetCstringAtPositionInSarray(sarray,
                                    position,
                                    cString)
pSarray  sarray;
int      position;
char    *cString;
#endif
{
   char **newStrings = NULL;
   int    nSize = sarray->nStrings;

   assert(sarray != NULL);
   assert(position >= 0);

   if(position < sarray->nMaxSize) {
      utFreeBytes((pVoid)sarray->strings[position]);
      sarray->strings[position] = utAssignString(cString);
      sarray->nStrings = MAX(sarray->nStrings,position+1);
   } else {
      newStrings = (char **)utGetBytes((position+SARRAY_GROWBY_SIZE)*sizeof(char *));
      sarray->nMaxSize = position+SARRAY_GROWBY_SIZE;
      sarray->nStrings = position+1;

      if(nSize > 0)
         memcpy(newStrings,sarray->strings,nSize*sizeof(char *));

      newStrings[position] = utAssignString(cString);

      utFreeBytes((pVoid)sarray->strings);
      sarray->strings = newStrings;
   }
}


#ifdef __STDC__
void utMergeSarrays(pSarray sarray1,
                    pSarray sarray2)
#else
void utMergeSarrays(sarray1,
                    sarray2)
pSarray sarray1;
pSarray sarray2;
#endif
{
   int   nCount2;
   int   i1;
   char *cString;

   assert(sarray1 != NULL);
   assert(sarray2 != NULL);

   nCount2 = utGetCountSarray(sarray2);
   for(i1=0 ; i1<nCount2 ; i1++) {
      cString = utGetCstringAtPositionInSarray(sarray2,i1);
      utAddToSarray(sarray1,cString);
   }
}


#ifdef __STDC__
void utSubtractSarrayFromSarray(pSarray sarray1,
                                pSarray sarray2)
#else
void utSubtractSarrayFromSarray(sarray1,
                                sarray2)
pSarray sarray1;
pSarray sarray2;
#endif
{
   int   nStrings = utGetCountSarray(sarray2);
   int   i1;
   char *string;
   int   position;

   for(i1=0 ; i1<nStrings ; i1++) {
      string = utGetCstringAtPositionInSarray(sarray2,i1);
      position = utStringPositionInSarray(string,sarray1);
      if(position >= 0)
         utDeleteFromSarray(sarray1,position);
   }
}


#ifdef __STDC__
pSarray utIntersectSarrays(pSarray sarray1,
                           pSarray sarray2)
#else
pSarray utIntersectSarrays(sarray1,
                           sarray2)
pSarray sarray1;
pSarray sarray2;
#endif
{
   pSarray  intersection;
   int      nCount1 = utGetCountSarray(sarray1);
   int      nCount2 = utGetCountSarray(sarray2);
   int      maximumIntersectionCount = MIN(nCount1,nCount2);
   int      i1;
   char    *string;

   intersection = utCreateSarrayOfSize(maximumIntersectionCount);

   if(nCount1 < nCount2) {
      for(i1=0 ; i1<nCount1 ; i1++) {
         string = utGetCstringAtPositionInSarray(sarray1,i1);
         if(utStringIsInSarray(string,sarray2))
            utAddToSarray(intersection,string);
      }
   } else {
      for(i1=0 ; i1<nCount2 ; i1++) {
         string = utGetCstringAtPositionInSarray(sarray2,i1);
         if(utStringIsInSarray(string,sarray1))
            utAddToSarray(intersection,string);
      }
   }

   return(intersection);
}


#ifdef __STDC__
int utGetLongestStringSizeInSarray(pSarray sarray)
#else
int utGetLongestStringSizeInSarray(sarray)
pSarray sarray;
#endif
{
   int   maxLength = 0;
   int   i1,nStrings;
   char *string;

   assert(sarray != NULL);

   nStrings = utGetCountSarray(sarray);
   if(nStrings > 0) {
      string = utGetCstringAtPositionInSarray(sarray,0);
      maxLength = utStringLength(string);
      for(i1=1 ; i1<nStrings ; i1++) {
         string = utGetCstringAtPositionInSarray(sarray,i1);
         maxLength = MAX(maxLength,utStringLength(string));
      }
   }

   return(maxLength);
}


#ifdef __STDC__
pSarray utAdjoinSarrays(pSarray sarray1,
                        int widthSarray1,
                        pSarray sarray2,
                        int widthSarray2,
                        pSarray sarray3,
                        int widthSarray3,
                        pSarray sarray4,
                        int widthSarray4,
                        pSarray sarray5,
                        int widthSarray5)
#else
pSarray utAdjoinSarrays(sarray1,
                        widthSarray1,
                        sarray2,
                        widthSarray2,
                        sarray3,
                        widthSarray3,
                        sarray4,
                        widthSarray4,
                        sarray5,
                        widthSarray5)
pSarray sarray1;
int     widthSarray1;
pSarray sarray2;
int     widthSarray2;
pSarray sarray3;
int     widthSarray3;
pSarray sarray4;
int     widthSarray4;
pSarray sarray5;
int     widthSarray5;
#endif
{
   pSarray  sarray;
   char     buf[MAX_STRING_LENGTH+1];
   int      nRows = utGetCountSarray(sarray1);
   int      i1;
   char    *row1;
   char    *row2;
   char    *row3;
   char    *row4;
   char    *row5;

   sarray = utCreateSarrayOfSize(nRows);

   for(i1=0 ; i1<nRows ; i1++) {
      utClearString(buf);
      row1 = utGetCstringAtPositionInSarray(sarray1,i1);
      utCatToString(buf,row1,widthSarray1);

      if(sarray2 != NULL) {
         row2 = utGetCstringAtPositionInSarray(sarray2,i1);
         utCatToString(buf,row2,widthSarray2);
         if(sarray3 != NULL) {
            row3 = utGetCstringAtPositionInSarray(sarray3,i1);
            utCatToString(buf,row3,widthSarray3);
            if(sarray4 != NULL) {
               row4 = utGetCstringAtPositionInSarray(sarray4,i1);
               utCatToString(buf,row4,widthSarray4);
               if(sarray5 != NULL) {
                  row5 = utGetCstringAtPositionInSarray(sarray5,i1);
                  utCatToString(buf,row5,widthSarray5);
               }
            }
         }
      }

      utAddToSarray(sarray,buf);
   }

   return(sarray);
}


#ifdef __STDC__
void utCleaveSarrays(pSarray sarray,
                     pSarray *sarray1,
                     int widthSarray1,
                     pSarray *sarray2,
                     int widthSarray2,
                     pSarray *sarray3,
                     int widthSarray3,
                     pSarray *sarray4,
                     int widthSarray4,
                     pSarray *sarray5,
                     int widthSarray5)
#else
void utCleaveSarrays(sarray,
                     sarray1,
                     widthSarray1,
                     sarray2,
                     widthSarray2,
                     sarray3,
                     widthSarray3,
                     sarray4,
                     widthSarray4,
                     sarray5,
                     widthSarray5)
pSarray  sarray;
pSarray *sarray1;
int      widthSarray1;
pSarray *sarray2;
int      widthSarray2;
pSarray *sarray3;
int      widthSarray3;
pSarray *sarray4;
int      widthSarray4;
pSarray *sarray5;
int      widthSarray5;
#endif
{
   char  buf[MAX_STRING_LENGTH+1];
   int   nRows = utGetCountSarray(sarray);
   int   i1;
   char *sarrayRow;

   *sarray1 = utCreateSarrayOfSize(nRows);
   if(widthSarray2 >= 0) {
      *sarray2 = utCreateSarrayOfSize(nRows);
      if(widthSarray3 >= 0) {
         *sarray3 = utCreateSarrayOfSize(nRows);
         if(widthSarray4 >= 0) {
            *sarray4 = utCreateSarrayOfSize(nRows);
            if(widthSarray5 >= 0) {
               *sarray5 = utCreateSarrayOfSize(nRows);
            }
         }
      }
   }

   for(i1=0 ; i1<nRows ; i1++) {
      sarrayRow = utGetCstringAtPositionInSarray(sarray,i1);
      utGetNextColumn(sarrayRow,widthSarray1,buf);
      utAddToSarray(*sarray1,buf);
      if(widthSarray2 >= 0) {
         utGetNextColumn(NULL,widthSarray2,buf);
         utAddToSarray(*sarray2,buf);
         if(widthSarray3 >= 0) {
            utGetNextColumn(NULL,widthSarray3,buf);
            utAddToSarray(*sarray3,buf);
            if(widthSarray4 >= 0) {
               utGetNextColumn(NULL,widthSarray4,buf);
               utAddToSarray(*sarray4,buf);
               if(widthSarray5 >= 0) {
                  utGetNextColumn(NULL,widthSarray5,buf);
                  utAddToSarray(*sarray5,buf);
               }
            }
         }
      }
   }
}


#ifdef __STDC__
utLogical utEqualSarrays(pSarray sarray1,
                         pSarray sarray2)
#else
utLogical utEqualSarrays(sarray1,
                         sarray2)
pSarray sarray1;
pSarray sarray2;
#endif
{
   utLogical  areSarraysEqual = utFalse;
   int        nCount1 = utGetCountSarray(sarray1);
   int        nCount2 = utGetCountSarray(sarray2);
   int        i1;
   char      *string1;
   char      *string2;

   if(nCount1 == nCount2) {
      areSarraysEqual = utTrue;
      for(i1=0 ; i1<nCount1 ; i1++) {
         string1 = utGetCstringAtPositionInSarray(sarray1,i1);
         string2 = utGetCstringAtPositionInSarray(sarray2,i1);
         if(!utEqualStrings(string1,string2)) {
            areSarraysEqual = utFalse;
            break;
         }
      }
   }

   return(areSarraysEqual);
}

/******************************************
 * returns a single cstring which is a cat
 * of the Sarray passed. It does not change
 * the Sarray argument.
 ******************************************/

#ifdef __STDC__
char *utCreateSingleStringFromSarray(pSarray sarray,
                                     char *delimiter)
#else
char *utCreateSingleStringFromSarray(sarray,
                                     delimiter)
pSarray  sarray;
char    *delimiter;
#endif
{
   int   i1,nSize;
   int   lengthDelimiter = utStringLength(delimiter);
   int   nStrings = sarray->nStrings;
   char *string = NULL;

   nSize = 0;
   for(i1=0 ; i1<nStrings ; i1++)
      nSize += utStringLength(sarray->strings[i1]);

   if(nSize == 0)
      return(utCreateNullString());

   nSize += (nStrings*lengthDelimiter)+1;  /* delimiters and NULL */

   string = (char *)utGetBytes(nSize*sizeof(char));

   strcpy(string,sarray->strings[0]);

   if(nStrings > 1) {
      strcat(string,delimiter);
      for(i1=1 ; i1<nStrings-1 ; i1++) {
         strcat(string,sarray->strings[i1]);
         strcat(string,delimiter);
      }
      strcat(string,sarray->strings[nStrings-1]);
   }
   strcat(string,"\0");

   return(string);
}


#ifdef __STDC__
pSarray utCreateSarrayFromSingleString(char *string,
                                       char *delimiter)
#else
pSarray utCreateSarrayFromSingleString(string,
                                       delimiter)
char *string;
char *delimiter;
#endif
{
   pSarray  sarray = utCreateSarray();
   char    *stringCopy;
   char    *field;
   int      lengthDelimiter = utStringLength(delimiter);

   assert(lengthDelimiter == 1);

   if(!utHasAllBlanks(string)) {
      stringCopy = utAssignString(string);
      field = strtok(stringCopy,delimiter);
      while(field != NULL) {
         utAddToSarray(sarray,field);
         field = strtok(NULL,delimiter);
      }
      utFreeBytes((pVoid)stringCopy);
   }

   return(sarray);
}


#ifdef __STDC__
void utDumpSarray(pSarray sarray,
                  FILE *fp)
#else
void utDumpSarray(sarray,
                  fp)
pSarray  sarray;
FILE    *fp;
#endif
{
   int nCount = sarray->nStrings;
   int i1;

/*
   for(i1=0 ; i1<nCount ; i1++)
      utWritelnFortranString(fp,sarray->strings[i1]);
*/
}


#ifdef __STDC__
void utSaveSarray(pSarray sarray,
                  FILE *fp)
#else
void utSaveSarray(sarray,
                  fp)
pSarray  sarray;
FILE    *fp;
#endif
{
   int nCount = sarray->nStrings;
   int i1;

/*
   fprintf(fp," %d\n",nCount);
   for(i1=0 ; i1<nCount ; i1++)
      utWritelnFortranString(fp,sarray->strings[i1]);
*/
}


#ifdef __STDC__
pSarray utLoadSarray(FILE *fp)
#else
pSarray utLoadSarray(fp)
FILE *fp;
#endif
{
   pSarray sarray;
   char    buf[MAX_STRING_LENGTH+1];
   int     i1,nCount;

   fscanf(fp," %d\n",&nCount);
   sarray = utCreateSarrayOfSize(nCount);
   for(i1=0 ; i1<nCount ; i1++) {
      utReadFortranString(fp,buf,MAX_STRING_LENGTH);
      utAddToSarray(sarray,buf);
   }

   return(sarray);
}


#ifdef __STDC__
void utApendStringAtPositionInSarray(pSarray sarray,
                                     int position,
                                     char *cString,
                                     char *delimiter)
#else
void utApendStringAtPositionInSarray(sarray,
                                     position,
                                     cString,
                                     delimiter)
pSarray  sarray;
int      position;
char    *cString;
char    *delimiter;
#endif
{
   char *newString;

   assert(sarray != NULL);
   assert(position >= 0);

   if(position < sarray->nMaxSize) {
      if(utIsNullString(sarray->strings[position])) {
         utSetCstringAtPositionInSarray(sarray,
                                        position,
                                        cString);
      } else {
         newString = utCatStringPlusString(sarray->strings[position],delimiter,cString);
         utFreeBytes((pVoid)sarray->strings[position]);
         sarray->strings[position] = newString;
         sarray->nStrings = MAX(sarray->nStrings,position+1);
      }
   } else {
      utSetCstringAtPositionInSarray(sarray,
                                     position,
                                     cString);
   }
}


#ifdef __STDC__
pSarray utMatchStringsInSarray(pSarray sarray,
                               char *patternString)
#else
pSarray utMatchStringsInSarray(sarray,
                               patternString)
pSarray  sarray;
char    *patternString;
#endif
{
   pSarray  matchSarray = NULL;
   int      i1;
   int      nStrings = utGetCountSarray(sarray);
   char    *string;

   matchSarray = utCreateSarrayOfSize(nStrings);
   for(i1=0 ; i1<nStrings ; i1++) {
      string = utGetCstringAtPositionInSarray(sarray,i1);
      if(utDoesStringMatchPattern(string,patternString,utFalse))
         utAddToSarray(matchSarray,string);
   }

   return(matchSarray);
}


#ifdef __STDC__
pSarray utGetUniqueMatchedList(pSarray list,
                               pSarray choiceList,
                               utLogical sortList)
#else
pSarray utGetUniqueMatchedList(list,
                               choiceList,
                               sortList)
pSarray   list;
pSarray   choiceList;
utLogical sortList;
#endif
{
   pSarray  uniqueMatchedList = NULL;
   int      i1;
   int      nItemsInList = utGetCountSarray(list);
   char    *listString;
   pSarray  matchedList = utCreateSarray();
   pSarray  itemMatchList;

   for(i1=0 ; i1<nItemsInList ; i1++) {
      listString = utGetCstringAtPositionInSarray(list,i1);
      if(utIsWildcardString(listString)) {
         itemMatchList = utMatchStringsInSarray(choiceList,listString);
         utMergeSarrays(matchedList,itemMatchList);
         utFreeSarray(&itemMatchList);
      } else {
         if(utStringIsInSarray(listString,choiceList))
            utAddToSarray(matchedList,listString);
      }
   }

   if(sortList)
      uniqueMatchedList = utGetUniqueSarray(matchedList);
   else {
      uniqueMatchedList = utCreateSarray();
      nItemsInList = utGetCountSarray(matchedList);
      for(i1=0 ; i1<nItemsInList ; i1++) {
         listString = utGetCstringAtPositionInSarray(matchedList,i1);
         if(!utStringIsInSarray(listString,uniqueMatchedList))
            utAddToSarray(uniqueMatchedList,listString);
      }
   }
   utFreeSarray(&matchedList);

   return(uniqueMatchedList);
}


#ifdef __STDC__
void utWriteFormattedSarray(pSarray sarray,
                            int firstLineInitialOffset,
                            int initialOffset,
                            int stringWidth,
                            int stringsPerLine,
                            FILE *fp)
#else
void utWriteFormattedSarray(sarray,
                            firstLineInitialOffset,
                            initialOffset,
                            stringWidth,
                            stringsPerLine,
                            fp)
pSarray  sarray;
int      firstLineInitialOffset;
int      initialOffset;
int      stringWidth;
int      stringsPerLine;
FILE    *fp;
#endif
{
   int   nStrings = sarray->nStrings;
   int   i1;
   int   nFirstLineStrings;
   char *initialBlanks;
   char *string;

   assert(stringsPerLine > 0);

   if(nStrings > 0) {
      if(firstLineInitialOffset > 0) {
         initialBlanks = utPadStringToLength("",firstLineInitialOffset);
         fprintf(fp,"%s",initialBlanks);
         utFreeBytes((pVoid)initialBlanks);
      }

      nFirstLineStrings = MIN(nStrings,stringsPerLine);
      for(i1=0 ; i1<nFirstLineStrings ; i1++) {
         if(stringWidth > 0) {
            string = utPadStringToLength(sarray->strings[i1],stringWidth);
            fprintf(fp,"%s",string);
            utFreeBytes((pVoid)string);
         } else {
            string = sarray->strings[i1];
            fprintf(fp,"%s",string);
         }
      }
      fprintf(fp,"\n");

      if(nStrings > nFirstLineStrings) {
         if(initialOffset > 0)
            initialBlanks = utPadStringToLength("",initialOffset);
         for(i1=nFirstLineStrings ; i1<nStrings ; i1++) {
            if((i1%stringsPerLine == 0) && (initialOffset > 0))
               fprintf(fp,"%s",initialBlanks);
            if(stringWidth > 0) {
               string = utPadStringToLength(sarray->strings[i1],stringWidth);
               fprintf(fp,"%s",string);
               utFreeBytes((pVoid)string);
            } else {
               string = sarray->strings[i1];
               fprintf(fp,"%s",string);
            }
      
            if((i1+1)%stringsPerLine == 0)
               fprintf(fp,"\n");
         }
         if(nStrings%stringsPerLine != 0)
            fprintf(fp,"\n");
      
         if(initialOffset > 0)
            utFreeBytes((pVoid)initialBlanks);
      }
   }
}


#ifdef __STDC__
void utWriteSarrayToTXTFile(pSarray sarray,
                            int stringsPerLine,
                            int increment,
                            FILE *fp)
#else
void utWriteSarrayToTXTFile(sarray,
                            stringsPerLine,
                            increment,
                            fp)
pSarray  sarray;
int      stringsPerLine;
int      increment;
FILE    *fp;
#endif
{
   int nStrings = sarray->nStrings;
   int stringsInRow;
   int firstPosition = 0;
   int position;

   assert(stringsPerLine > 0);
   assert(increment > 0);

   if(nStrings > 0) {
      position = firstPosition;
      while(position < nStrings/stringsPerLine) {
         stringsInRow = 0;
         while(stringsInRow < stringsPerLine) {
               fprintf(fp,"%s\t",sarray->strings[position]);
            position += increment;
            stringsInRow++;
         }
         fprintf(fp,"\n");
         firstPosition++;
         position = firstPosition;
      }
   }
}


