/*
  $RCSfile: utstrcvu.c,v $       $Revision: 1.12 $
  $State: V7_1 $                  $Date: 2004/11/05 11:24:22 $
*/
#include <ctype.h>

#include "uttypesu.h"
#include "utmacrou.h"
#include "utmemryu.h"
#include "utstrngu.h"
#include "utstrcvu.h"

/*****************************************
 * get the roundoff of the machine in
 * double precision. This is the smallest
 * positive machine number U such that
 * (1.0 + U) != 1.0 (in double precision)
 *****************************************/

static double utstrcvMachinePrecision = 0.;

#ifdef __STDC__
double utMachinePrecision()
#else
double utMachinePrecision()
#endif
{
   double u,comp;

   if(utstrcvMachinePrecision == 0.) {
      u = 1.0;
      do {
         u = u*0.5;
         comp = 1.0+u;
      } while (comp != 1.0);
      utstrcvMachinePrecision = u*2.0;
   }

   return(utstrcvMachinePrecision);
}


#ifdef __STDC__
double utMachineTolerance()
#else
double utMachineTolerance()
#endif
{
   double machineTolerance;
   double exponent = 2./3.;

   machineTolerance = 5.*pow(utMachinePrecision(),exponent);

   return(machineTolerance);
}


#ifdef __STDC__
char *utIntToString(int integer)
#else
char *utIntToString(integer)
int integer;
#endif
{
   static char  buf[MAX_INTEGER_STRING_DIGITS+1];
   int          nPrint;
   char        *string;

   nPrint = sprintf(buf,"%d",integer);
   assert(nPrint <= MAX_INTEGER_STRING_DIGITS);

   string = utAssignString(buf);

   return(string);
}


#ifdef __STDC__
int utStringToInt(char *sInt)
#else
int utStringToInt(sInt)
char *sInt;
#endif
{
   int integer = 0;

   assert(sInt);

   if(!utIsNullString(sInt))
      sscanf(sInt,"%d",&integer);
   else
      integer = 0;

   return(integer);
}


#ifdef __STDC__
char *utLongToString(long integer)
#else
char *utLongToString(integer)
long integer;
#endif
{
   static char  buf[MAX_INTEGER_STRING_DIGITS+1];
   int          nPrint;
   char        *string;

   nPrint = sprintf(buf,"%ld",integer);
   assert(nPrint <= MAX_INTEGER_STRING_DIGITS);

   string = utAssignString(buf);

   return(string);
}

#ifdef __STDC__
long utStringToLong(char *sLong)
#else
long utStringToLong(sLong)
char *sLong;
#endif
{
   long integer = 0;

   assert(sLong);

   if(!utIsNullString(sLong))
      sscanf(sLong,"%ld",&integer);
   else
      integer = 0;

   return(integer);
}


#ifdef __STDC__
char *utDoubleToString(double value)
#else
char *utDoubleToString(value)
double value;
#endif
{
   char        *string;
   static char  buf[MAX_DOUBLE_STRING_DIGITS+1];
   static char  sDbl[MAX_DOUBLE_STRING_DIGITS+1];
   int          i1,nStart;
   utLogical    hasAlpha = utFalse;
   utLogical    hasDot   = utFalse;
   int          nPrint;

   utClearString(sDbl);

   if(fabs(value) > utMachinePrecision())
      nPrint = sprintf(buf,"%#.10g",value);
   else {
      strcpy(buf,"0.0");
      nPrint = 3;
   }
   assert(nPrint <= MAX_DOUBLE_STRING_DIGITS);

   for(i1=0 ; i1<utStringLength(buf) ; i1++) {
      if(isalpha(buf[i1]))
         hasAlpha = utTrue;
      if(buf[i1] == '.')
         hasDot   = utTrue;
   }

  /*
   * %#g ensures '.' is always there
   * but %g format can skip '.' !!
   */

   if(!hasDot) {
      strcpy(sDbl,buf);
   } else {
      if(hasAlpha) {
         for(i1=utStringLength(buf)-1 ; i1>=0 ; i1--)
            if(isalpha(buf[i1]))
               break;

         nStart = i1-1;
      } else
         nStart = utStringLength(buf)-1;

      if(buf[nStart] == '0') {
         for(i1=nStart ; i1>=0 ; i1--)
            if(buf[i1] != '0')
               break;
         i1++;
      } else
         i1 = nStart;

      strncpy(sDbl,buf,i1+1);
      sDbl[i1+1] = '\0';

      if(hasAlpha)
         strcat(sDbl,&buf[nStart+1]);
   }
   string = utAssignString(sDbl);

   return(string);
}


#ifdef __STDC__
double utStringToDouble(char *sDbl)
#else
double utStringToDouble(sDbl)
char *sDbl;
#endif
{
   double value = 0.;

   assert(sDbl);

   if(!utIsNullString(sDbl))
      sscanf(sDbl,"%lg",&value);
   else
      value = 0.;

   return(value);
}


/**********************************************
** if a string holding a valid double is larger
** than 15 characters, it will be returned in a
** form with a maximum of 15 characters.
** string s1 has the original double. The new
** form is in string s2.
***********************************************/

#ifdef __STDC__
void utDblStrToLimitedDblStr(char *s1,
                             char *s2)
#else
void utDblStrToLimitedDblStr(s1,
                             s2)
char *s1;
char *s2;
#endif
{
   double d;

   d = atof(s1);
   sprintf(s2,"%.10g",d);
   utStripTrailingBlanks(s2);
}


#ifdef __STDC__
char *utDoubleToFormattedString(double value,
                                int width,
                                int fPrecision,
                                int gPrecision)
#else
char *utDoubleToFormattedString(value,
                                width,
                                fPrecision,
                                gPrecision)
double value;
int    width;
int    fPrecision;
int    gPrecision;
#endif
{
   char   *string;
   char    format[MAX_DOUBLE_STRING_DIGITS+1];
   double  lowerBound,upperBound;

   if(width > 0) {
      lowerBound = pow((double)10.,(double)(-fPrecision));
      upperBound = pow((double)10.,(double)(width-(fPrecision+2)))-
                   pow((double)10.,(double)(-fPrecision))/2.;

      if((gPrecision > 0) && (((value > 0.) && (fabs(value) < lowerBound)) || (fabs(value) >= upperBound)))
         sprintf(format,"%%#%d.%dlg",width,
                                     gPrecision);
      else
         sprintf(format,"%%#%d.%dlf",width,
                                     fPrecision);
   } else
      sprintf(format,"%%#.%df",fPrecision);

   string = utFormatDoubleToString(value,format);

   return(string);
}


#ifdef __STDC__
char *utFormatDoubleToString(double value,
                             char *format)
#else
char *utFormatDoubleToString(value,
                             format)
double  value;
char  *format;
#endif
{
   char *string;
   char  localFormat[MAX_DOUBLE_STRING_DIGITS+1];
   char  buf[MAX_STRING_LENGTH+1];

   if(format[0] != '%') {
      sprintf(localFormat,"%%%s",format);
      sprintf(buf,localFormat,value);
   } else
      sprintf(buf,format,value);

   utStripTrailingBlanks(buf);

   string = utAssignString(buf);

   return(string);
}


#ifdef __STDC__
char *utIntToFormattedString(int integer,
                             int width)
#else
char *utIntToFormattedString(integer,
                             width)
int integer;
int width;
#endif
{
   char  format[MAX_INTEGER_STRING_DIGITS+1];
   char  buf[MAX_STRING_LENGTH+1];
   int   nPrint;
   char *string;

   if(width > 0) {
      sprintf(format,"%%%d.%dd",width,
                                width);
      nPrint = sprintf(buf,format,integer);
   } else
      nPrint = sprintf(buf,"%d",integer);

   assert(nPrint <= MAX_INTEGER_STRING_DIGITS);

   string = utAssignString(buf);

   return(string);
}


#ifdef __STDC__
char *utFormatIntToString(int value,
                          char *format)
#else
char *utFormatIntToString(value,
                          format)
int   value;
char *format;
#endif
{
   char *string;
   char  localFormat[MAX_INTEGER_STRING_DIGITS+1];
   char  buf[MAX_STRING_LENGTH+1];

   if(format[0] != '%') {
      sprintf(localFormat,"%%%s",format);
      sprintf(buf,localFormat,value);
   } else
      sprintf(buf,format,value);

   utStripTrailingBlanks(buf);

   string = utAssignString(buf);

   return(string);
}


#ifdef __STDC__
char *utLongToFormattedString(long value,
                              int width)
#else
char *utLongToFormattedString(value,
                              width)
long value;
int  width;
#endif
{
   char  format[MAX_INTEGER_STRING_DIGITS+1];
   char  buf[MAX_STRING_LENGTH+1];
   int   nPrint;
   char *string;

   if(width > 0) {
      sprintf(format,"%%%d.%dld",width,
                                 width);
      nPrint = sprintf(buf,format,value);
   } else
      nPrint = sprintf(buf,"%ld",value);

   assert(nPrint <= MAX_INTEGER_STRING_DIGITS);

   string = utAssignString(buf);

   return(string);
}


#ifdef __STDC__
void utWriteFormattedDoubleToFile(FILE *fp,
                                  char *prefix,
                                  double value,
                                  int width,
                                  int fPrecision,
                                  int gPrecision,
                                  char *delimeter,
                                  char *suffix,
                                  utLogical newLine)
#else
void utWriteFormattedDoubleToFile(fp,
                                  prefix,
                                  value,
                                  width,
                                  fPrecision,
                                  gPrecision)
                                  char *delimeter,
                                  char *suffix,
                                  utLogical newLine)
FILE      *fp;
char      *prefix;
double     value;
int        width;
int        fPrecision;
int        gPrecision;
char      *delimeter;
char      *suffix;
utLogical  newLine;
#endif
{
   char *valueString;

   assert(width > 0);

   if(!utIsNullString(prefix))
      fprintf(fp,"%s",prefix);

   valueString = utDoubleToFormattedString(value,
                                           width,
                                           fPrecision,
                                           gPrecision);
   fprintf(fp,"%s",valueString);
   utFreeBytes((pVoid)valueString);

   if(!utIsNullString(delimeter))
      fprintf(fp,"%s",delimeter);
   if(!utIsNullString(suffix))
      fprintf(fp,"%s",suffix);
   if(newLine)
      fprintf(fp,"\n");
}


