/*
  $RCSfile: utstriou.c,v $       $Revision: 1.7 $
  $State: V6_2 $                  $Date: 2000/08/18 19:08:03 $
*/
#include "utmacrou.h"
#include "utstriou.h"
#include "utstrngu.h"
#include "utfilesu.h"
#include "utsarryu.h"

#ifdef __STDC__
utLogical utReadLine(FILE *fp,
                     char *buf,
                     int bufSize)
#else
utLogical utReadLine(fp,
                     buf,
                     bufSize)
FILE *fp;
char *buf;
int   bufSize;
#endif
{
   utLogical lineWasRead = utTrue;
   int       c;
   int       i1 = 0;

   while((i1 < bufSize) &&
         ((c = getc(fp)) != EOF) && (c != '\n') && (c != '\r')) buf[i1++] = c;
   buf[i1] = '\0';
   if(c == EOF)
      lineWasRead = utFalse;

   return(lineWasRead);
}


#ifdef __STDC__
char *utReadUnlimitedLine(FILE *fp)
#else
char *utReadUnlimitedLine(fp)
FILE *fp;
#endif
{
   char      *string;
   utLogical  done = utFalse;
   char       buf[MAX_STRING_LENGTH+1];
   int        c;
   int        i1;
   pSarray    sections = utCreateSarray();

   while(!done) {
      i1 = 0;
      while((i1 < MAX_STRING_LENGTH) &&
            ((c = getc(fp)) != EOF) && (c != '\n') && (c != '\r')) buf[i1++] = c;
      buf[i1] = '\0';
      if(i1 > 0)
         utAddToSarray(sections,buf);
      if((c == EOF) || (c == '\n') || (c == '\r'))
         done = utTrue;

      if       (c == '\n') {
         c = getc(fp);
         if(c != '\r')
            ungetc(c,fp);
      } else if(c == '\r') {
         c = getc(fp);
         if(c != '\n')
            ungetc(c,fp);
      }
   }

   if(utGetCountSarray(sections) > 0)
      string = utCreateSingleStringFromSarray(sections,"");
   else
      string = utCreateNullString();

   utFreeSarray(&sections);

   return(string);
}

/***************************************************************
 *   Read quoted string from fp.  A maximum of n
 *   characters are read into s. The remainder is discarded.
 *   Does not return leading and trailing quotes (') or spaces.
 *   Returns NULL string if first non-white-space character is
 *   not a quote, or if EOF is encountered before entire
 *   string is read in, or if a closing quote has not been
 *   found.
 **************************************************************/

#ifdef __STDC__
void utReadFortranString(FILE *fp,
                         char *string,
                         int maximumLength)
#else
void utReadFortranString(fp,
                         string,
                         maximumLength)
FILE *fp;
char *string;
int   maximumLength;
#endif
{
   int c;
   char *cp;

   /*
    * expect to read a "'" or a white-space char
    */

   while((c = getc(fp)) != '\'') {
      if(c == EOF) {
         utClearString(string);
         return;
      }
      if(c != '\n' && c != '\r' && c != '\t' && c != ' ') {
         utClearString(string);
         assert(utFalse);
         return;
      }
   }

   cp = string;
   while(1) {
      while((c = getc(fp)) != '\'') {
         if(c == EOF) {
            utClearString(string);
            return;
         }
         if(cp-string < maximumLength) *cp++ = c;
      }
      if((c = getc(fp)) != '\''){
         ungetc(c,fp);
         goto done;
      } else
         if(cp-string < maximumLength) *cp++ = c;
   }
done:
   *cp = '\0';
   utStripBlanks(string);
}


#ifdef __STDC__
void utWriteCSVFieldString(FILE *fp, char *buf)
#else
void utWriteCSVFieldString(fp,buf)
FILE *fp;
char *buf;
#endif
{
   size_t i1;
   size_t len = utStringLength(buf);

   if(len == 0) return;

   if(utStringContainsCharacter(buf,',')) 
      fprintf(fp,"\"%s\"",buf);
   else
      fprintf(fp,"%s",buf);
}


#ifdef __STDC__
void utWriteFortranString(FILE *fp, char *buf)
#else
void utWriteFortranString(fp,buf)
FILE *fp;
char *buf;
#endif
{
   size_t i1;
   size_t len = utStringLength(buf);

   if(len == 0) {
      fprintf(fp," ' '");
      return;
   }

   fprintf(fp," '");   /* first quote */

   for(i1=0 ; i1<len ; i1++) {
      if(buf[i1] == '\'')
         fprintf(fp,"''");
      else
         fprintf(fp,"%c",buf[i1]);
   }
   fprintf(fp,"'");   /* second quote */
}


#ifdef __STDC__
void utWritelnFortranString(FILE *fp, char *buf)
#else
void utWritelnFortranString(fp,buf)
FILE *fp;
char *buf;
#endif
{
   utWriteFortranString(fp,buf);
   fprintf(fp,"\n");
}


#ifdef __STDC__
utLogical utIsNextFieldFortranString(FILE *fp)
#else
utLogical utIsNextFieldFortranString(fp)
FILE *fp;
#endif
{
   utLogical     nextFieldIsFortranString = utFalse;
   pFilePosition filePosition = utGetFilePosition(fp);
   utLogical     filePositionReset;
   int           c;

   if(filePosition != NULL) {
      while((c = getc(fp)) != '\'') {
         if(c != '\n' && c != '\r' && c != '\t' && c != ' ')
            break;
      }
      if(c == '\'') {
         while(1) {
            while((c = getc(fp)) != '\'') {
               if(c == EOF)
                  break;
            }
            if(c == '\'') {
               if((c = getc(fp)) != '\'') {
                  nextFieldIsFortranString = utTrue;
                  break;
               }
            } else
               break;
         }
      }
      filePositionReset = utSetFilePosition(fp,
                                            filePosition);
      if(nextFieldIsFortranString)
         nextFieldIsFortranString = filePositionReset;

      utFreeFilePosition(&filePosition);
   }

   return(nextFieldIsFortranString);
}


#ifdef __STDC__
void utPeekAtFortranString(FILE *fp,
                           char *string,
                           int maximumLength)
#else
void utPeekAtFortranString(fp,
                           string,
                           maximumLength)
FILE *fp;
char *string;
int   maximumLength;
#endif
{
   pFilePosition filePosition = utGetFilePosition(fp);

   utReadFortranString(fp,
                       string,
                       maximumLength);
   if(!utSetFilePosition(fp,filePosition))
      utClearString(string);

   utFreeFilePosition(&filePosition);
}

/*********************************************************
 *
 * String that has been read before these encounters
 * in file fp is placed in string s
 * All leading blanks are ignored when reading s
 *
 *********************************************************/

#ifdef __STDC__
utReadTerminateCharacter utReadUntilChar(FILE *fp,
                                         char *string,
                                         int limit)
#else
utReadTerminateCharacter utReadUntilChar(fp,
                                         string,
                                         limit)
FILE *fp;
char *string;
int   limit;
#endif
{
   utReadTerminateCharacter endCondition = ReadTerminateCharacterUnknown;
   int                      c,counter;

   while((c = getc(fp)) == ' ');

   counter = 0;
   while((endCondition == ReadTerminateCharacterUnknown) &&
         (counter < limit-1)) {
      switch(c) {
         case EOF:
            endCondition = ReadTerminateCharacterEOF;
            break;
         case '\n':
         case '\r':
            endCondition = ReadTerminateCharacterEOL;
            break;
         case '\t':
            endCondition = ReadTerminateCharacterTab;
            break;
         default:
            string[counter] = c;
            counter++;
            c = getc(fp);
            break;
      }
   }
   string[counter] = '\0';

   return(endCondition);
}


#ifdef __STDC__
int utSkipLines(FILE *fp,
                int nLines)
#else
int utSkipLines(fp,
                nLines)
FILE *fp;
int   nLines;
#endif
{
   char  buf[MAX_STRING_LENGTH+1];
   char *status;
   int   nLinesSkipped = 0;

   while(((status = fgets(buf,MAX_STRING_LENGTH,fp)) != NULL) && (nLinesSkipped < nLines))
      nLinesSkipped++;

   return(nLinesSkipped);
}


#ifdef __STDC__
utLogical utSkipLinesUntil(FILE *fp,
                           char *string)
#else
utLogical utSkipLinesUntil(fp,
                           string)
FILE *fp;
char *string;
#endif
{
   char  buf[MAX_STRING_LENGTH+1];
   char *status;

   while(status = fgets(buf,MAX_STRING_LENGTH,fp)) {
      if(utStringContainsSubstring(buf,string))
         return(utTrue);
   }

   return(utFalse);
}


#ifdef __STDC__
utLogical utIsSequentialEOF(FILE *fp)
#else
utLogical utIsSequentialEOF(fp)
FILE *fp;
#endif
{
   int c;

   while((c = getc(fp)) == '\n' || c == '\r' || c == '\t' || c == ' ');
   if(c == EOF) {
      return(utTrue);
   } else {
      ungetc(c,fp);
      return(utFalse);
   }
}


