/*
  $RCSfile: utstrngu.c,v $       $Revision: 1.19 $
  $State: V7_1 $                  $Date: 2005/01/06 18:06:52 $
*/
#include <ctype.h>

#include "utmacrou.h"
#include "utmemryu.h"
#include "utstrngu.h"

#ifdef __STDC__
int utStringLength(char *string)
#else
int utStringLength(string)
char *string;
#endif
{
   int stringLength = 0;

   if(!utIsNullString(string))
      stringLength = strlen(string);

   return(stringLength);
}


#ifdef __STDC__
utLogical utEqualStrings(char *s1,
                         char *s2)
#else
utLogical utEqualStrings(s1,s2)
char *s1;
char *s2;
#endif
{
   if     (utIsNullString(s1) && utIsNullString(s2))
      return(utTrue);
   else if(utIsNullString(s1) || utIsNullString(s2))
      return(utFalse);
   else if(strcmp(s1,s2) == 0)
      return(utTrue);
   else
      return(utFalse);
}


#ifdef __STDC__
utLogical utIsNullString(char *s)
#else
utLogical utIsNullString(s)
char *s;
#endif
{
   if((s == NULL) || (s[0] == '\0'))
      return utTrue;
   else
      return utFalse;
}

/**********************************************
 * return utTrue if the string has only blanks if
 * at all. return utFalse otherwise.
 **********************************************/

#ifdef __STDC__
utLogical utHasAllBlanks(char *s)
#else
utLogical utHasAllBlanks(s)
char *s;
#endif
{
   int i1;

   for(i1=0 ; i1<utStringLength(s) ; i1++)
      if(s[i1] != ' ')
         return utFalse;

   return utTrue;
}

/***************************************
 * make a copy of the string and return
 * a pointer to new string.
 ***************************************/

#ifdef __STDC__
char *utAssignString(char *buf)
#else
char *utAssignString(buf)
char *buf;
#endif
{
   char   *newBuf;
   size_t  size;

   if(buf == NULL) return NULL;

   size = utStringLength(buf)+1;
   newBuf = (char *)utGetBytes(size);
   strcpy(newBuf,buf);

   return(newBuf);
}


#ifdef __STDC__
void utClearString(char *s)
#else
void utClearString(s)
char *s;
#endif
{
   assert(s != NULL);

   s[0] = '\0';
}

/********************************************************
 * nSpace is the total space - use blank padding on s2 to
 * meet this fixed length requirement.  Ofcourse, we
 * assume s2 has length <= nSpace !!
 ********************************************************/

#ifdef __STDC__
void utCatToString(char *s1,
                   char *s2,
                   int nSpace)
#else
void utCatToString(s1,
                   s2,
                   nSpace)
char *s1;
char *s2;
int   nSpace;
#endif
{
   int len1,len2;
   int i1;

   assert(s1 != NULL);
   assert(s2 != NULL);

   len1 = utStringLength(s1);
   len2 = utStringLength(s2);
   if(nSpace > 0) assert (utStringLength(s2) <= nSpace);

   strcat(s1,s2);
   for(i1=0 ; i1<(nSpace-len2) ; i1++)
      s1[len1+len2+i1] = ' ';
   s1[len1+len2+i1] = '\0';
}


#ifdef __STDC__
char *utCatStringPlusString(char *s1,
                            char *delimiter,
                            char *s2)
#else
char *utCatStringPlusString(s1,
                            delimiter,
                            s2)
char *s1;
char *delimiter;
char *s2;
#endif
{
   char   *string;
   size_t  len1;
   size_t  len2;
   size_t  sizeString;

   assert(s1 != NULL);
   assert(s2 != NULL);

   len1 = utStringLength(s1);
   len2 = utStringLength(s2);
   sizeString = len1+len2;

   if(utIsNullString(delimiter)) {
      string = (char *)utGetBytes(sizeString+1);
      strcpy(string,s1);
      strcat(string,s2);
   } else {
      sizeString = sizeString+utStringLength(delimiter);
      string = (char *)utGetBytes(sizeString+1);
      strcpy(string,s1);
      strcat(string,delimiter);
      strcat(string,s2);
   }

   return(string);
}


#ifdef __STDC__
char *utCatCStringsWithDelimiter(char *delimiter,
                                 char *s1,char *s2,
                                 char *s3,char *s4,char *s5)
#else
char *utCatCStringsWithDelimiter(delimiter,
                                 s1,s2,
                                 s3,s4,s5)
char *delimiter;
char *s1;
char *s2;
char *s3;
char *s4;
char *s5;
#endif
{
   char   *string;
   size_t  lenDelimiter = utStringLength(delimiter);
   size_t  lens1 = utStringLength(s1);
   size_t  lens2 = utStringLength(s2);
   size_t  lens3 = utStringLength(s3);
   size_t  lens4 = utStringLength(s4);
   size_t  lens5 = utStringLength(s5);
   size_t  sizeString = lens1+lens2+lens3+lens4+lens5+4*lenDelimiter;

   string = (char *)utGetBytes(sizeString+1);
   if(utIsNullString(delimiter)) {
      if(lens1 > 0)
         strcpy(string,s1);
      if(lens2 > 0)
         strcat(string,s2);
      if(lens3 > 0)
         strcat(string,s3);
      if(lens4 > 0)
         strcat(string,s4);
      if(lens5 > 0)
         strcat(string,s5);
   } else {
      if(lens1 > 0)
         strcpy(string,s1);
      if(lens2+lens3+lens4+lens5 > 0) {
         strcat(string,delimiter);
         if(lens2 > 0)
            strcat(string,s2);
      }
      if(lens3+lens4+lens5 > 0) {
         strcat(string,delimiter);
         if(lens3 > 0)
            strcat(string,s3);
      }
      if(lens4+lens5 > 0) {
         strcat(string,delimiter);
         if(lens4 > 0)
            strcat(string,s4);
      }
      if(lens5 > 0) {
         strcat(string,delimiter);
         if(lens5 > 0)
            strcat(string,s5);
      }
   }

   return(string);
}

/************************************************
 * copies next column from a parseString to buf
 * if parse_string is NULL is operates on the
 * previous string. nLength is the length of the
 * column to be copied.
 * if nLength is 0, return the rest of the string
 ************************************************/

#ifdef __STDC__
void utGetNextColumn(char *parseString,
                     int nLength,
                     char *buf)
#else
void utGetNextColumn(parseString,
                     nLength,
                     buf)
char *parseString;
int   nLength;
char *buf;
#endif
{
   static int   nStart;
   static char *string = NULL;
   int          nCharactersRemaining,nCopy;

   if(buf == NULL) {
      utFreeBytes((pVoid)string);
      string = NULL;
   } else {
      assert(string || parseString);

      if(nLength == 0)
         assert(parseString == NULL);

      if(parseString != NULL) {
         nStart = 0;
         utFreeBytes((pVoid)string);
         string = utAssignString(parseString);
      }

      utClearString(buf);
      nCharactersRemaining = utStringLength(string)-nStart;
      if(nLength) {
         nCopy = MIN(nCharactersRemaining,nLength);
         if(nCopy > 0) {
            strncpy(buf,&string[nStart],nCopy);
            buf[nCopy] = '\0';
            utStripTrailingBlanks(buf);
            nStart += nCopy;
         }
      } else {
         if(nCharactersRemaining > 0)
            strcpy(buf,&string[nStart]);  /* return remaining string */
         utFreeBytes((pVoid)string);
         string = NULL;                   /* no more string left */
         nStart = 0;
      }
   }
}

/***************************************
 * strip any trailing blanks in a string
 ***************************************/

#ifdef __STDC__
void utStripTrailingBlanks(char *buf)
#else
void utStripTrailingBlanks(buf)
char *buf;
#endif
{
   int i1,nLen;

   nLen = utStringLength(buf)-1;
   for(i1=nLen ; i1>=0 ; i1--)
      if(buf[i1] != ' ')
         break;
      else
         buf[i1] = '\0';
}

/**************************************
 * strip any leading blanks in a string
 **************************************/

#ifdef __STDC__
void utStripLeadingBlanks(char *s)
#else
void utStripLeadingBlanks(s)
char *s;
#endif
{
   int i1;
   int nonBlank = 0;

   for(i1=0 ; i1<utStringLength(s) ; i1++) {
      if(s[i1] != ' ') {
         nonBlank = i1;
         break;
      }
   }

   if(nonBlank > 0) {
      for(i1=nonBlank ; i1<utStringLength(s) ; i1++)
         s[i1-nonBlank] = s[i1];
      s[i1-nonBlank] = '\0';
   }
}

/***************************************************
 * strip any leading and trailing blanks in a string
 ***************************************************/

#ifdef __STDC__
void utStripBlanks(char *s)
#else
void utStripBlanks(s)
char *s;
#endif
{
   utStripLeadingBlanks(s);
   utStripTrailingBlanks(s);
}

/************************************
 * create a null string and return it
 ************************************/

#ifdef __STDC__
char *utCreateNullString()
#else
char *utCreateNullString()
#endif
{
   char *nullString = (char *)utGetBytes(1);

   nullString[0] = '\0';

   return(nullString);
}

/****************************
 * set a buffer to all blanks
 ****************************/

#ifdef __STDC__
void utMakeAllBlanks(char *s,
                     int size)
#else
void utMakeAllBlanks(s,
                     size)
char *s;
int size;
#endif
{
   utMakeAllCharacter(s,size,' ');
}


#ifdef __STDC__
void utMakeAllCharacter(char *s,
                        int size,
                        char character)
#else
void utMakeAllCharacter(s,
                        size,
                        character)
char *s;
int   size;
char  character;
#endif
{
   int i1;

   for(i1=0 ; i1<size ; i1++)
      s[i1] = character;
   s[i1] = '\0';
}

/*************************************************
 * search a "string" for a "search_string".
 * given the direction of search (forward/reverse)
 * starting point of search, and
 * case type (sensitive/insensitive)
 *************************************************/

#ifdef __STDC__
char *utSearchString(char *string,
                     char *searchFor,
                     int position,
                     utCaseType caseType,
                     utSearchType searchType)
#else
char *utSearchString(string,
                     searchFor,
                     position,
                     caseType,
                     searchType)
char         *string;
char         *searchFor;
int           position;
utCaseType    caseType;
utSearchType  searchType;
#endif
{
   switch(searchType) {
      case SearchTypeForward:
         return(utForwardSearchString(string,searchFor,position,caseType));
      case SearchTypeReverse:
         return(utReverseSearchString(string,searchFor,position,caseType));
      default:
         return(NULL);
   }
}

/*******************************************************
** reverse search in string "string" for "search_string"
** starting from the position "position". If found,
** return the pointer to that location. Return NULL
** otherwise.
********************************************************/

#ifdef __STDC__
char *utReverseSearchString(char *string,
                            char *searchFor,
                            int position,
                            utCaseType caseType)
#else
char *utReverseSearchString(string,
                            searchFor,
                            position,
                            caseType)
char       *string;
char       *searchFor;
int         position;
utCaseType  caseType;
#endif
{
   int       i1;
   int       len = utStringLength(searchFor);
   utLogical found;

   position -= len;

   while(position >= 0) {
      found = utTrue;
      for(i1=0 ; i1<len ; i1++) {
         switch (caseType) {
            case CaseTypeSensitive:
               found = (string[position+i1] == searchFor[i1]);
               break;
            case CaseTypeInsensitive:
               found = (tolower(string[position+i1]) == tolower(searchFor[i1]));
               break;
         }
         if(!found)
            break;
      }
      if(found)
         return(&string[position]);
      else
         position--;
   }

   return(NULL);
}

/*******************************************************
** search forward in string "string" for "search_string"
** starting from the position "position". If found,
** return the pointer to that location. Return NULL
** otherwise.
********************************************************/

#ifdef __STDC__
char *utForwardSearchString(char *string,
                            char *searchFor,
                            int position,
                            utCaseType caseType)
#else
char *utForwardSearchString(string,
                            searchFor,
                            position,
                            caseType)
char       *string;
char       *searchFor;
int         position;
utCaseType  caseType;
#endif
{
   int       i1;
   utLogical found;
   int       len = utStringLength(searchFor);
   int       remain = utStringLength(string) - position;

   while((remain-len) >= 0) {
      found = utTrue;
      for(i1=0 ; i1<len ; i1++) {
         switch(caseType) {
            case CaseTypeSensitive:
               found = (string[position+i1] == searchFor[i1]);
               break;
            case CaseTypeInsensitive:
               found = (tolower(string[position+i1]) == tolower(searchFor[i1]));
               break;
          }
          if(!found)
             break;
      }
      if(found)
         return(&string[position]);
      else {
         position++;
         remain--;
      }
  }

  return(NULL);
}

/***********************************
 * convert a character to upper case
 * and return the upper case version
 ***********************************/

#ifdef __STDC__
int utUpper(char c)
#else
int utUpper(c)
char c;
#endif
{
   return((islower(c)) ? (toupper(c)) : c);
}

/***********************************
 * convert a character to lower case
 * and return the lower case version
 ***********************************/

#ifdef __STDC__
int utLower(char c)
#else
int utLower(c)
char c;
#endif
{
   return((isupper(c)) ? (tolower(c)) : c);
}

/***********************************
 * compare two strings ignoring case
 ***********************************/

#ifdef __STDC__
int utStrCmpNoCase(char *s,
                   char *t)
#else
int utStrCmpNoCase(s,
                   t)
char *s;
char *t;
#endif
{
   if(!s && !t) return 1;
   if(!s || !t) return 0;

   for( ; utUpper(*s) == utUpper(*t) ; s++, t++)
      if(*s == '\0')
         return(0);

   return(utUpper(*s) - utUpper(*t));
}

/**********************************
 * compare to strings ignoring case
 * upto a certain specified length
 **********************************/

#ifdef __STDC__
int utStrNCmpNoCase(char *s,
                    char *t,
                    int compareLength)
#else
int utStrNCmpNoCase(s,
                    t,
                    compareLength)
char *s;
char *t;
int   compareLength;
#endif
{
   register int i1;

   if(compareLength < 1)
      return(0);
   for(i1=0 ; utUpper(*s) == utUpper(*t) ; s++, t++, i1++)
      if(*s == '\0' || i1 == compareLength-1)
         return(0);

   return(utUpper(*s) - utUpper(*t));
}


#ifdef __STDC__
void utStringToUpper(char *s)
#else
void utStringToUpper(s)
char *s;
#endif
{
   char c;

   if(!utIsNullString(s))
      while(c = *s)
         *s++ = utUpper(c);
}


#ifdef __STDC__
void utStringToLower(char *s)
#else
void utStringToLower(s)
char *s;
#endif
{
   char c;

   if(!utIsNullString(s))
      while(c = *s)
         *s++ = utLower(c);
}


#ifdef __STDC__
void utFreeCstringArray(char **cstringarray,
                        int nStrings)
#else
void utFreeCstringArray(cstringarray,
                        nStrings)
char **cstringarray;
int    nStrings;
#endif
{
   int i1;

   assert(nStrings > 0);

   for(i1=0 ; i1<nStrings ; i1++)
      utFreeBytes((pVoid)cstringarray[i1]);

   utFreeBytes((pVoid)cstringarray);
}


#ifdef __STDC__
void utCopyColumnFromCstring(char *cstring,
                             char *column,
                             int iColumn,
                             int *columnSizes)
#else
void utCopyColumnFromCstring(cstring,
                             column,
                             iColumn,
                             columnSizes)
char *cstring;
char *column;
int   iColumn;
int  *columnSizes;
#endif
{
   int i1;
   int cumulSize = 0;

   assert(cstring != NULL);
   assert(column != NULL);
   assert(iColumn > 0);
   assert(columnSizes != NULL);

   for(i1=0 ; i1<iColumn ; i1++)
      cumulSize += columnSizes[i1];
   assert(cumulSize < utStringLength(cstring));

   for(i1=0 ; i1<columnSizes[iColumn] ; i1++)
      column[i1] = cstring[cumulSize+i1];
   column[i1] = '\0';
}


#ifdef __STDC__
void utAddColumnToCstring(char *cstring,
                          char *column,
                          int iColumn,
                          int *columnSizes)
#else
void utAddColumnToCstring(cstring,
                          column,
                          iColumn,
                          columnSizes)
char *cstring;
char *column;
int   iColumn;
int  *columnSizes;
#endif
{
   int nColumns;
   int i1,len;
   int cumulSize = 0;
   int nWhiteSpaces;

   assert(cstring != NULL);
   assert(column != NULL);
   assert(iColumn > 0);
   assert(columnSizes != NULL);

   nColumns = utNumber(columnSizes);
   assert(iColumn <= nColumns);

  /******************************************
   * only the last column can have zero width
   ******************************************/

   if(iColumn != nColumns)  {
      assert(columnSizes[iColumn] > 0);
   } else {
      nWhiteSpaces = columnSizes[iColumn]-utStringLength(column);
      assert(nWhiteSpaces >= 0);
   }

  /**************************************************
   * make sure cstring is consistent with columnSizes
   **************************************************/

   for(i1=0 ; i1<iColumn ; i1++)
      cumulSize += columnSizes[i1];
   assert(cumulSize == utStringLength(cstring));

   strcat(cstring,column);
   len = utStringLength(cstring);
   for(i1=0 ; i1<nWhiteSpaces ; i1++)
      cstring[len+i1] = ' ';
   cstring[len+i1] = '\0';
}


#ifdef __STDC__
static utLogical utDoesStringMatchPatternPrivate(char *string,
                                                 char *pattern,
                                                 utLogical ignoreCase)
#else
static utLogical utDoesStringMatchPatternPrivate(string,
                                                 pattern,
                                                 ignoreCase)
char      *string;
char      *pattern;
utLogical  ignoreCase;
#endif
{
   char      c,p,l;
   utLogical matchRange;

   for( ; ; ) {
      switch(p = *pattern++) {
         case 0:                                /* end of pattern               */
            return(*string ? utFalse : utTrue); /* if end of string utTrue      */
         case '*':
            while(*string) {                    /* match zero or more char      */
               if(utDoesStringMatchPattern(string++,pattern,ignoreCase))
                  return(utTrue);
            }
            return(utDoesStringMatchPattern(string,pattern,ignoreCase));
         case '?':
            if(*string++ == 0)                  /* match any one char           */
               return(utFalse);                 /* not end of string            */
            break;
         case '[':
            if((c = *string++) == 0)            /* match char set               */
               return(utFalse);                 /* syntax                       */

            if(*pattern == '^') {
               p = *pattern++;
               matchRange = utFalse;
            } else
               matchRange = utTrue;

            if(ignoreCase)
               c = toupper(c);
            l = 0;
            while(p = *pattern++) {
               if(p == ']')                     /* if end of char set, then     */
                  break;                        /* no match found               */

               if(p == '-') {                   /* check a range of chars?      */
                  p = *pattern;                 /* get high limit of range      */
                  if((p == 0) || (p == ']'))
                     return(utFalse);           /* syntax                       */

                  if(ignoreCase)
                     p = toupper(p);
                  if((c >= l) && (c <= p))
                     break;                     /* if in range, move on         */
               }

               if(ignoreCase)
                  p = toupper(p);
               l = p;

               if(c == p)                       /* if char matches this element */
                  break;                        /* move on                      */
            }

            if(matchRange && (p == ']'))
               return(utFalse);
            if(!matchRange && (p != ']'))
               return(utFalse);

            while(p && (p != ']'))              /* got a match in char set      */
               p = *pattern++;                  /* skip to end of set           */
            break;
         default:
            c = *string++;
            if(ignoreCase) {
               c = toupper(c);
               p = toupper(p);
            }
            if(c != p)                          /* check for exact char         */
               return(utFalse);                 /* not a match                  */
            break;
      }
   }
}


#ifdef __STDC__
utLogical utDoesStringMatchPattern(char *string,
                                   char *pattern,
                                   utLogical ignoreCase)
#else
utLogical utDoesStringMatchPattern(string,
                                   pattern,
                                   ignoreCase)
char      *string;
char      *pattern;
utLogical  ignoreCase;
#endif
{
   utLogical doesStringMatchPattern;

   if     (utIsNullString(string) && utIsNullString(pattern))
      doesStringMatchPattern = utTrue;
   else if(utIsNullString(string) || utIsNullString(pattern))
      doesStringMatchPattern = utFalse;
   else 
      doesStringMatchPattern = utDoesStringMatchPatternPrivate(string,
                                                               pattern,
                                                               ignoreCase);

   return(doesStringMatchPattern);
}


#ifdef __STDC__
char *utGetDelimitedField(char *string,
                          char *delimiters,
                          int fieldNumber)
#else
char *utGetDelimitedField(string,
                          delimiters,
                          fieldNumber)
char *string;
char *delimiters;
int   fieldNumber;
#endif
{
   char *stringCopy = NULL;
   char *field;
   char *requestedField = NULL;
   int   i1;

   if(!utIsNullString(string)) {
      stringCopy = utAssignString(string);
      field = strtok(stringCopy,delimiters);
      for(i1=1 ; i1<fieldNumber ; i1++) {
         field = strtok(NULL,delimiters);
      }
      requestedField = utAssignString(field);
   }
   utFreeBytes((pVoid)stringCopy);

   return(requestedField);
}


#ifdef __STDC__
char *utGetBracketedField(char *string,
                          char *leadingBracket,
                          char *trailingBracket)
#else
char *utGetBracketedField(string,
                          leadingBracket,
                          trailingBracket)
char *string;
char *leadingBracket;
char *trailingBracket;
#endif
{
   char      *requestedField = NULL;
   utLogical  stringContainsLeadingBracket;
   utLogical  stringContainsTrailingBracket;
   char      *leadingCharacter;
   char      *trailingCharacter;
   int        lengthBracketedString;

   if(!utIsNullString(string)) {
      stringContainsLeadingBracket = utStringContainsSubstring(string,leadingBracket);
      stringContainsTrailingBracket = utStringContainsSubstring(string,trailingBracket);
      if(stringContainsLeadingBracket && stringContainsTrailingBracket) {
         leadingCharacter = strstr(string,leadingBracket);
         trailingCharacter = strstr(string,trailingBracket);
         lengthBracketedString = trailingCharacter-leadingCharacter+1;
         requestedField = (char *)utGetBytes((lengthBracketedString+1)*sizeof(char));
         strncpy(requestedField,leadingCharacter,lengthBracketedString);
         strcat(requestedField,"\0");
      }
   }

   return(requestedField);
}


#ifdef __STDC__
int utCountCharacterInString(char *string,
                             char character)
#else
int utCountCharacterInString(string,
                             character)
char *string;
char  character;
#endif
{
   int    nCount = 0;
   size_t i1,length;

   if(!utIsNullString(string)) {
      length = utStringLength(string);
      for(i1=0 ; i1<length ; i1++) {
         if(string[i1] == character)
            nCount++;
      }
   }

   return(nCount);
}


#ifdef __STDC__
utLogical utStringContainsCharacter(char *string,
                                    char character)
#else
utLogical utStringContainsCharacter(string,
                                    character)
char *string;
char  character;
#endif
{
   utLogical stringContainsCharacter = utFalse;
   size_t    i1,length;

   if(!utIsNullString(string)) {
      length = utStringLength(string);
      for(i1=0 ; i1<length ; i1++) {
         if(string[i1] == character) {
            stringContainsCharacter = utTrue;
            break;
         }
      }
   }

   return(stringContainsCharacter);
}


#ifdef __STDC__
void utSubstituteCharacterInString(char *string,
                                   char characterBefore,
                                   char characterAfter)
#else
void utSubstituteCharacterInString(string,
                                   characterBefore,
                                   characterAfter)
char *string;
char  characterBefore;
char  characterAfter;
#endif
{
   char *cp;

   cp = string;
   while(cp = strchr(cp,characterBefore))
      *cp = characterAfter;
}


#ifdef __STDC__
utLogical utStringContainsSubstring(char *string,
                                    char *substring)
#else
utLogical utStringContainsSubstring(string,
                                    substring)
char *string;
char *substring;
#endif
{
   utLogical  stringContainsSubstring = utFalse;
   char      *status;

   if       (utIsNullString(string) && utIsNullString(substring)) {
      stringContainsSubstring = utTrue;
   } else if(utIsNullString(string) || utIsNullString(substring)) {
      stringContainsSubstring = utFalse;
   } else {
      status = strstr(string,substring);
      if(status != NULL)
         stringContainsSubstring = utTrue;
   }

   return(stringContainsSubstring);
}


#ifdef __STDC__
char *utSubstituteSubstringInString(char *string,
                                    char *subStringBefore,
                                    char *subStringAfter)
#else
char *utSubstituteSubstringInString(string,
                                    subStringBefore,
                                    subStringAfter)
char *string;
char *subStringBefore;
char *subStringAfter;
#endif
{
   char   *newString = NULL;
   size_t  originalLength = utStringLength(string);
   size_t  beforeLength = utStringLength(subStringBefore);
   size_t  afterLength = 0;
   size_t  newLength;
   char   *location;
   int     position;

   location = strstr(string,subStringBefore);
   if(location != NULL) {
      if(!utIsNullString(subStringAfter))
         afterLength = utStringLength(subStringAfter);
      newLength = originalLength-beforeLength+afterLength;
      position = location-string;
      newString = (char *)utGetBytes(newLength+1);
      utClearString(newString);
      if(position > 0)
         strncat(newString,string,position);
      if(!utIsNullString(subStringAfter))
         strcat(newString,subStringAfter);
      if(position+beforeLength < originalLength)
         strcat(newString,&string[position+beforeLength]);
   } else
      newString = utAssignString(string);

   return(newString);
}


#ifdef __STDC__
void utSubstituteAllStringsInString(char **buf,
                                    char *stringBefore,
                                    char *stringAfter)
#else
void utSubstituteAllStringsInString(buf,
                                    stringBefore,
                                    stringAfter)
char **buf;
char  *stringBefore;
char  *stringAfter;
#endif
{
   char *oldString = *buf;
   char *newString = NULL;

   newString = utSubstituteSubstringInString(oldString,
                                             stringBefore,
                                             stringAfter);
   while(!utEqualStrings(newString,oldString)) {
      utFreeBytes((pVoid)oldString);
      oldString = newString;
      newString = utSubstituteSubstringInString(oldString,
                                                stringBefore,
                                                stringAfter);
   }
   utFreeBytes((pVoid)oldString);

   *buf = newString;
}


#ifdef __STDC__
void utReplaceAllStringsInString(char *buf,
                                 int nBuf,
                                 char *stringBefore,
                                 char *stringAfter)
#else
void utReplaceAllStringsInString(buf,
                                 nBuf,
                                 stringBefore,
                                 stringAfter)
char *buf;
int   nBuf;
char *stringBefore;
char *stringAfter;
#endif
{
   char *newString = utAssignString(buf);
   int   stringLength;

   utSubstituteAllStringsInString(&newString,
                                  stringBefore,
                                  stringAfter);
   stringLength = utStringLength(newString);
   utClearString(buf);
   if(stringLength < nBuf)
      strcpy(buf,newString);

   utFreeBytes((pVoid)newString);
}


#ifdef __STDC__
char *utStripWhiteSpace(char *string)
#else
char *utStripWhiteSpace(string)
char *string;
#endif
{
   char *strippedString;
   int   c;
   char *cp;
   int   i1,stringLength;

   if(!utIsNullString(string)) {
      stringLength = utStringLength(string);
      strippedString = (char *)utGetBytes((stringLength+1)*sizeof(char));
      cp = strippedString;
      for(i1=0 ; i1<stringLength ; i1++) {
         c = string[i1];
         if(c != '\n' && c != '\r' && c != '\t' && c != ' ')
            *cp++ = c;
      }
      *cp = '\0';
   } else
      strippedString = utCreateNullString();

   return(strippedString);
}


#ifdef __STDC__
char *utPadStringToLength(char *string,
                          int paddedLength)
#else
char *utPadStringToLength(string,
                          paddedLength)
char *string;
int   paddedLength;
#endif
{
   char *paddedString = (char *)utGetBytes((paddedLength+1)*sizeof(char));
   int   i1,stringLength;

   if(utIsNullString(string)) {
      for(i1=0 ; i1<paddedLength ; i1++)
         paddedString[i1] = ' ';
   } else {
      stringLength = utStringLength(string);
      if(stringLength >= paddedLength)
         strncpy(paddedString,string,paddedLength);
      else {
         strcpy(paddedString,string);
         for(i1=stringLength ; i1<paddedLength ; i1++)
            paddedString[i1] = ' ';
      }
   }
   paddedString[paddedLength] = '\0';

   return(paddedString);
}


#ifdef __STDC__
char *utCenterString(char *string,
                     int returnStringWidth)
#else
char *utCenterString(string,
                     returnStringWidth)
char *string;
int returnStringWidth;
#endif
{
   char *returnString;
   int   stringLength = utStringLength(string);
   int   nLeadingBlanks,nTrailingBlanks;
   char *leadingBlanks;
   char *trailingBlanks;

   assert(returnStringWidth > 0);

   if(stringLength >= returnStringWidth) {
      returnString = utPadStringToLength(string,returnStringWidth);
   } else {
      nLeadingBlanks  = (returnStringWidth-stringLength)/2;
      nTrailingBlanks = returnStringWidth-nLeadingBlanks-stringLength;
      leadingBlanks  = utPadStringToLength(NULL,nLeadingBlanks);
      trailingBlanks = utPadStringToLength(NULL,nTrailingBlanks);

      returnString = utCatStringPlusString(leadingBlanks,
                                           string,
                                           trailingBlanks);
      utFreeBytes((pVoid)trailingBlanks);
      utFreeBytes((pVoid)leadingBlanks);
   }

   return(returnString);
}


#ifdef __STDC__
char *utCenterBracketedString(char *string,
                              char *leadingBracket,
                              char *trailingBracket,
                              int returnStringWidth)
#else
char *utCenterBracketedString(string,
                              leadingBracket,
                              trailingBracket,
                              returnStringWidth)
char *string;
char *leadingBracket;
char *trailingBracket;
int   returnStringWidth;
#endif
{
   char *returnString;
   char *bracketedString;

   assert(returnStringWidth > 0);

   bracketedString = utCatStringPlusString(leadingBracket,
                                           string,
                                           trailingBracket);
   returnString = utCenterString(bracketedString,
                                 returnStringWidth);
   utFreeBytes((pVoid)bracketedString);

   return(returnString);
}


#ifdef __STDC__
utLogical utIsStringUpper(char *string)
#else
utLogical utIsStringUpper(string)
char *string;
#endif
{
   utLogical stringIsUpper = utTrue;
   char      c;

   if(!utIsNullString(string))
      while(c = *string)
         if(!isupper(c)) {
            stringIsUpper = utFalse;
            break;
         } else
            *string++;

   return(stringIsUpper);
}


#ifdef __STDC__
utLogical utIsStringLower(char *string)
#else
utLogical utIsStringLower(string)
char *string;
#endif
{
   utLogical stringIsLower = utTrue;
   char      c;

   if(!utIsNullString(string))
      while(c = *string)
         if(!islower(c)) {
            stringIsLower = utFalse;
            break;
         } else
            *string++;

   return(stringIsLower);
}


#ifdef __STDC__
char *utCreateStringOfChar(char character,
                           int length)
#else
char *utCreateStringOfChar(character,
                           length)
char character;
int  length;
#endif
{
   char *string;

   assert(length >= 0);
   
   string = utPadStringToLength("",length);
   utMakeAllCharacter(string,length,character);

   return(string);
}

typedef enum {
    CSVSearchForTypeQuote,
    CSVSearchForTypeDoubleQuote,
    CSVSearchForTypeComma,
    CSVSearchForTypeTab
} utCSVSearchForType;

#ifdef __STDC__
char *utGetSingleQuotedCSVField(char *string,
                                int fieldNumber)
#else
char *utGetSingleQuotedCSVField(string,
                                fieldNumber)
char *string;
int   fieldNumber;
#endif
{
   char               *field = utCreateNullString();
   char               *start;
   char               *end;
   char               *pQuote;
   char               *pComma;
   utCSVSearchForType  searchForType;
   int                 i1;
   int                 length;

   if(!utIsNullString(string)) {
      start = string;
      for(i1=1 ; i1<fieldNumber ; i1++) {
         while(*start == ' ') start++;
         pQuote = strchr(start,'\'');
         pComma = strchr(start,',');
         if((pQuote != NULL) && (pComma != NULL)) {
            if((int)(pQuote-pComma) > 0)
               searchForType = CSVSearchForTypeComma;
            else
               searchForType = CSVSearchForTypeQuote;
         } else if(pQuote != NULL) {
            searchForType = CSVSearchForTypeQuote;
         } else if(pComma != NULL) {
            searchForType = CSVSearchForTypeComma;
         } else
            return(field);

         switch(searchForType) {
            case CSVSearchForTypeComma:
               start = pComma;
               break;
            case CSVSearchForTypeQuote:
               start = pQuote;
               start++;
               start = strchr(start,'\'');
               /* jump to comma if used to separate fields */
               end = start;
               end++;
               pQuote = strchr(end,'\'');
               pComma = strchr(end,',');
               if((pQuote != NULL) && (pComma != NULL)) {
                  if((int)(pQuote-pComma) > 0)
                     start = pComma;
               } else if(pComma != NULL)
                  start = pComma;
               break;
         }
         if(start != NULL) start++;
      }

      while(*start == ' ') start++;
      pQuote = strchr(start,'\'');
      pComma = strchr(start,',');
      if((pQuote != NULL) && (pComma != NULL)) {
         if((int)(pQuote-pComma) > 0)
            searchForType = CSVSearchForTypeComma;
         else
            searchForType = CSVSearchForTypeQuote;
      } else if(pQuote != NULL) {
         searchForType = CSVSearchForTypeQuote;
      } else if(pComma != NULL) {
         searchForType = CSVSearchForTypeComma;
      } else {
         utFreeBytes((pVoid)field);
         field = utAssignString(start);
         return(field);
      }

      switch(searchForType) {
         case CSVSearchForTypeComma:
            end = pComma;
            break;
         case CSVSearchForTypeQuote:
            start = pQuote;
            start++;
            end = strchr(start,'\'');
            break;
      }
      
      if(start != NULL) {
         while((*start == ' ') && ((int)(end-start) > 1)) start++;
         end--;
         while((*end == ' ') && ((int)(end-start) > 0)) end--;
         end++;

         length = (int)(end-start);
         utFreeBytes((pVoid)field);
         field = utCreateStringOfChar('\0',length);
         strncpy(field,start,length);
         return(field);
      } else
         return(field);
   } else
      return(field);
}


#ifdef __STDC__
char *utGetDoubleQuotedCSVField(char *string,
                                int fieldNumber)
#else
char *utGetDoubleQuotedCSVField(string,
                                fieldNumber)
char *string;
int   fieldNumber;
#endif
{
   char               *field = utCreateNullString();
   char               *start;
   char               *end;
   char               *currentPosition;
   char               *pDoubleQuote;
   char               *pComma;
   utCSVSearchForType  searchForType;
   int                 i1;
   int                 length;
   utLogical           found;

   if(!utIsNullString(string)) {
      start = string;
      for(i1=1 ; i1<fieldNumber ; i1++) {
         while(*start == ' ') start++;
         pDoubleQuote = strchr(start,'"');
         pComma = strchr(start,',');
         if       ((pDoubleQuote != NULL) && (pComma != NULL)) {
            if((int)(pDoubleQuote-pComma) > 0)
               searchForType = CSVSearchForTypeComma;
            else
               searchForType = CSVSearchForTypeDoubleQuote;
         } else if(pDoubleQuote != NULL) {
            searchForType = CSVSearchForTypeDoubleQuote;
         } else if(pComma != NULL) {
            searchForType = CSVSearchForTypeComma;
         } else
            return(field);

         switch(searchForType) {
            case CSVSearchForTypeComma:
               start = pComma;
               break;
            case CSVSearchForTypeDoubleQuote:
               currentPosition = pDoubleQuote;
               currentPosition++;
               found = utFalse;
               while(!found) {
                  currentPosition = strchr(currentPosition,'"');
                  if(currentPosition[0] == '"' && currentPosition[1] == '"')
                     currentPosition += 2;
                  else
                     found = utTrue;
               }
               end = currentPosition;
               /* jump to comma if used to separate fields */
               end++;
               pDoubleQuote = strchr(end,'"');
               pComma = strchr(end,',');
               if((pDoubleQuote != NULL) && (pComma != NULL)) {
                  if((int)(pDoubleQuote-pComma) > 0)
                     start = pComma;
               } else if(pComma != NULL)
                  start = pComma;
               break;
         }
         if(start != NULL) start++;
      }

      while(*start == ' ') start++;
      pDoubleQuote = strchr(start,'"');
      pComma = strchr(start,',');
      if       ((pDoubleQuote != NULL) && (pComma != NULL)) {
         if((int)(pDoubleQuote-pComma) > 0)
            searchForType = CSVSearchForTypeComma;
         else
            searchForType = CSVSearchForTypeDoubleQuote;
      } else if(pDoubleQuote != NULL) {
         searchForType = CSVSearchForTypeDoubleQuote;
      } else if(pComma != NULL) {
         searchForType = CSVSearchForTypeComma;
      } else {
         utFreeBytes((pVoid)field);
         field = utAssignString(start);
         return(field);
      }

      switch(searchForType) {
         case CSVSearchForTypeComma:
            end = pComma;
            break;
         case CSVSearchForTypeDoubleQuote:
            start = pDoubleQuote;
            start++;
            currentPosition = start;
            found = utFalse;
            while(!found) {
               currentPosition = strchr(currentPosition,'"');
               if(currentPosition[0] == '"' && currentPosition[1] == '"')
                  currentPosition += 2;
               else
                  found = utTrue;
            }
            end = currentPosition;
            break;
      }

      if(start != NULL) {
         while((*start == ' ') && ((int)(end-start) > 1)) start++;
         end--;
         while((*end == ' ') && ((int)(end-start) > 0)) end--;
         end++;

         length = (int)(end-start);
         utFreeBytes((pVoid)field);
         field = utCreateStringOfChar('\0',length);
         strncpy(field,start,length);
         utSubstituteAllStringsInString(&field,
                                        "\"\"",
                                        "\"");
         return(field);
      } else
         return(field);
   } else
      return(field);
}


#ifdef __STDC__
char *utGetSingleQuotedTXTField(char *string,
                                int fieldNumber)
#else
char *utGetSingleQuotedTXTField(string,
                                fieldNumber)
char *string;
int   fieldNumber;
#endif
{
   char               *field = utCreateNullString();
   char               *start;
   char               *end;
   char               *pQuote;
   char               *pTab;
   utCSVSearchForType  searchForType;
   int                 i1;
   int                 length;

   if(!utIsNullString(string)) {
      start = string;
      for(i1=1 ; i1<fieldNumber ; i1++) {
         while(*start == ' ') start++;
         pQuote = strchr(start,'\'');
         pTab = strchr(start,'\t');
         if((pQuote != NULL) && (pTab != NULL)) {
            if((int)(pQuote-pTab) > 0)
               searchForType = CSVSearchForTypeTab;
            else
               searchForType = CSVSearchForTypeQuote;
         } else if(pQuote != NULL) {
            searchForType = CSVSearchForTypeQuote;
         } else if(pTab != NULL) {
            searchForType = CSVSearchForTypeTab;
         } else
            return(field);

         switch(searchForType) {
            case CSVSearchForTypeTab:
               start = pTab;
               break;
            case CSVSearchForTypeQuote:
               start = pQuote;
               start++;
               start = strchr(start,'\'');
               /* jump to tab if used to separate fields */
               end = start;
               end++;
               pQuote = strchr(end,'\'');
               pTab = strchr(end,'\t');
               if((pQuote != NULL) && (pTab != NULL)) {
                  if((int)(pQuote-pTab) > 0)
                     start = pTab;
               } else if(pTab != NULL)
                  start = pTab;
               break;
         }
         if(start != NULL) start++;
      }

      while(*start == ' ') start++;
      pQuote = strchr(start,'\'');
      pTab = strchr(start,'\t');
      if((pQuote != NULL) && (pTab != NULL)) {
         if((int)(pQuote-pTab) > 0)
            searchForType = CSVSearchForTypeTab;
         else
            searchForType = CSVSearchForTypeQuote;
      } else if(pQuote != NULL) {
         searchForType = CSVSearchForTypeQuote;
      } else if(pTab != NULL) {
         searchForType = CSVSearchForTypeTab;
      } else {
         utFreeBytes((pVoid)field);
         field = utAssignString(start);
         return(field);
      }

      switch(searchForType) {
         case CSVSearchForTypeTab:
            end = pTab;
            break;
         case CSVSearchForTypeQuote:
            start = pQuote;
            start++;
            end = strchr(start,'\'');
            break;
      }
      
      if(start != NULL) {
         while((*start == ' ') && ((int)(end-start) > 1)) start++;
         end--;
         while((*end == ' ') && ((int)(end-start) > 0)) end--;
         end++;

         length = (int)(end-start);
         utFreeBytes((pVoid)field);
         field = utCreateStringOfChar('\0',length);
         strncpy(field,start,length);
         return(field);
      } else
         return(field);
   } else
      return(field);
}


#ifdef __STDC__
char *utGetDoubleQuotedTXTField(char *string,
                                int fieldNumber)
#else
char *utGetDoubleQuotedTXTField(string,
                                fieldNumber)
char *string;
int   fieldNumber;
#endif
{
   char               *field = utCreateNullString();
   char               *start;
   char               *end;
   char               *currentPosition;
   char               *pDoubleQuote;
   char               *pTab;
   utCSVSearchForType  searchForType;
   int                 i1;
   int                 length;
   utLogical           found;

   if(!utIsNullString(string)) {
      start = string;
      for(i1=1 ; i1<fieldNumber ; i1++) {
         while(*start == ' ') start++;
         pDoubleQuote = strchr(start,'"');
         pTab = strchr(start,'\t');
         if       ((pDoubleQuote != NULL) && (pTab != NULL)) {
            if((int)(pDoubleQuote-pTab) > 0)
               searchForType = CSVSearchForTypeTab;
            else
               searchForType = CSVSearchForTypeDoubleQuote;
         } else if(pDoubleQuote != NULL) {
            searchForType = CSVSearchForTypeDoubleQuote;
         } else if(pTab != NULL) {
            searchForType = CSVSearchForTypeTab;
         } else
            return(field);

         switch(searchForType) {
            case CSVSearchForTypeTab:
               start = pTab;
               break;
            case CSVSearchForTypeDoubleQuote:
               currentPosition = pDoubleQuote;
               currentPosition++;
               found = utFalse;
               while(!found) {
                  currentPosition = strchr(currentPosition,'"');
                  if(currentPosition[0] == '"' && currentPosition[1] == '"')
                     currentPosition += 2;
                  else
                     found = utTrue;
               }
               end = currentPosition;
               /* jump to tab if used to separate fields */
               end++;
               pDoubleQuote = strchr(end,'"');
               pTab = strchr(end,'\t');
               if((pDoubleQuote != NULL) && (pTab != NULL)) {
                  if((int)(pDoubleQuote-pTab) > 0)
                     start = pTab;
               } else if(pTab != NULL)
                  start = pTab;
               break;
         }
         if(start != NULL) start++;
      }

      while(*start == ' ') start++;
      pDoubleQuote = strchr(start,'"');
      pTab = strchr(start,'\t');
      if       ((pDoubleQuote != NULL) && (pTab != NULL)) {
         if((int)(pDoubleQuote-pTab) > 0)
            searchForType = CSVSearchForTypeTab;
         else
            searchForType = CSVSearchForTypeDoubleQuote;
      } else if(pDoubleQuote != NULL) {
         searchForType = CSVSearchForTypeDoubleQuote;
      } else if(pTab != NULL) {
         searchForType = CSVSearchForTypeTab;
      } else {
         utFreeBytes((pVoid)field);
         field = utAssignString(start);
         return(field);
      }

      switch(searchForType) {
         case CSVSearchForTypeTab:
            end = pTab;
            break;
         case CSVSearchForTypeDoubleQuote:
            start = pDoubleQuote;
            start++;
            currentPosition = start;
            found = utFalse;
            while(!found) {
               currentPosition = strchr(currentPosition,'"');
               if(currentPosition[0] == '"' && currentPosition[1] == '"')
                  currentPosition += 2;
               else
                  found = utTrue;
            }
            end = currentPosition;
            break;
      }

      if(start != NULL) {
         while((*start == ' ') && ((int)(end-start) > 1)) start++;
         end--;
         while((*end == ' ') && ((int)(end-start) > 0)) end--;
         end++;

         length = (int)(end-start);
         utFreeBytes((pVoid)field);
         field = utCreateStringOfChar('\0',length);
         strncpy(field,start,length);
         utSubstituteAllStringsInString(&field,
                                        "\"\"",
                                        "\"");
         return(field);
      } else
         return(field);
   } else
      return(field);
}


#ifdef __STDC__
char *utExtractString(char *string,
                      int startPosition,
                      int endPosition)
#else
char *utExtractString(string,
                      startPosition,
                      endPosition)
char *string;
int   startPosition;
int   endPosition;
#endif
{
   char *requestedString = NULL;
   int   lengthString;

   if(!utIsNullString(string)) {
      if(endPosition >= startPosition) {
         lengthString = endPosition-startPosition+1;
         requestedString = (char *)utGetBytes((lengthString+1)*sizeof(char));
         strncpy(requestedString,&string[startPosition],lengthString);
         strcat(requestedString,"\0");
      }
   }

   return(requestedString);
}


#ifdef __STDC__
utLogical utIsWildcardString(char *string)
#else
utLogical utIsWildcardString(string)
char *string;
#endif
{
   utLogical isWildcardString = utFalse;
   int       i1,j1,length;

   if       (utStringContainsCharacter(string,'*')) {
      isWildcardString = utTrue;
   } else if(utStringContainsCharacter(string,'?')) {
      isWildcardString = utTrue;
   } else {
      length = utStringLength(string);
      for(i1=0 ; (i1<length) && !isWildcardString ; i1++) {
         if(string[i1] == '[') {
            for(j1=i1+1 ; (j1<length) && !isWildcardString ; j1++) {
               if(string[j1] == ']')
                  isWildcardString = utTrue;
            }
         }
      }
   }

   return(isWildcardString);
}


#ifdef __STDC__
int utPositionOfSubstringInString(char *string,
                                  char *subString)
#else
int utPositionOfSubstringInString(char *string,
                                  subString)
char *string;
char *subString;
#endif
{
   int   position = -1;
   char *location;

   location = strstr(string,subString);
   if(location != NULL)
      position = location-string;

   return(position);
}


