/*
  $RCSfile: uttableu.c,v $       $Revision: 1.18 $
  $State: V7_0 $                  $Date: 2003/10/01 13:46:56 $
*/
#include "utmemryu.h"
#include "utmacrou.h"
#include "uttableu.h"
#include "utstrngu.h"
#include "utstriou.h"
#include "utiarryu.h"

#ifdef __STDC__
pTable utCreateTable()
#else
pTable utCreateTable()
#endif
{
   pTable newTable;

   newTable = (pTable)utGetBytes(sizeof(Table));
   utInitializeTable(newTable);

   return(newTable);
}


#ifdef __STDC__
pTable utCreateTableWithNames(pSarray rowNames,
                              pSarray columnNames)
#else
pTable utCreateTableWithNames(rowNames,
                              columnNames)
pSarray rowNames;
pSarray columnNames;
#endif
{
   pTable newTable = utCreateTable();

   utFreeSarray(&newTable->RowNames);
   utFreeSarray(&newTable->ColumnNames);

   newTable->RowNames    = utAssignSarray(rowNames);
   newTable->ColumnNames = utAssignSarray(columnNames);

   return(newTable);
}


#ifdef __STDC__
pTable utCreateTableOfSize(int nRows,
                           int nColumns)
#else
pTable utCreateTableOfSize(nRows,
                           nColumns)
int nRows;
int nColumns;
#endif
{
   pTable newTable = utCreateTable();
   int    i1;

   if(nRows > 0) {
      utFreeSarray(&newTable->RowNames);
      newTable->RowNames = utCreateSarrayOfSize(nRows);
      for(i1=0 ; i1<nRows ; i1++)
         utAddToSarray(newTable->RowNames,"");
   }

   if(nColumns > 0) {
      utFreeSarray(&newTable->ColumnNames);
      newTable->ColumnNames = utCreateSarrayOfSize(nColumns);
      for(i1=0 ; i1<nColumns ; i1++)
         utAddToSarray(newTable->ColumnNames,"");
   }

   return(newTable);
}


#ifdef __STDC__
void utInitializeTable(pTable newTable)
#else
void utInitializeTable(newTable)
pTable newTable;
#endif
{
   newTable->StructType  = STRUCTTYPE_TABLE;
   newTable->RowNames    = utCreateSarray();
   newTable->ColumnNames = utCreateSarray();
   newTable->CellValues  = utCreateSarray();
}


#ifdef __STDC__
void utFreeTable(pTable *table)
#else
void utFreeTable(table)
pTable *table;
#endif
{
   if(*table == NULL)
      return;

   utFreeSarray(&(*table)->RowNames);
   utFreeSarray(&(*table)->ColumnNames);
   utFreeSarray(&(*table)->CellValues);

   utFreeBytes((pVoid)*table);
   *table = NULL;
}


#ifdef __STDC__
void utAddTableRow(pTable table,
                   char *rowName)
#else
void utAddTableRow(table,
                   rowName)
pTable  table;
char   *rowName;
#endif
{
   int nRowNames;
   int i1,nColumnNames;
   int location;

   utAddToSarray(table->RowNames,rowName);

   if(utGetCountSarray(table->CellValues) > 0) {
      nRowNames = utGetCountSarray(table->RowNames);
      nColumnNames = utGetCountSarray(table->ColumnNames);
      location = nRowNames-1;
      for(i1=0 ; i1<nColumnNames ; i1++) {
         if(location < utGetCountSarray(table->CellValues))
            utInsertInSarray(table->CellValues,
                             location,
                             "");
         location += nRowNames;
      }
   }
}


#ifdef __STDC__
void utAddTableColumn(pTable table,
                      char *columnName)
#else
void utAddTableColumn(table,
                      columnName)
pTable  table;
char   *columnName;
#endif
{
   utAddToSarray(table->ColumnNames,columnName);
}


#ifdef __STDC__
pTable utAssignTable(pTable table)
#else
pTable utAssignTable(table)
pTable table;
#endif
{
   pTable newTable = NULL;

   if(table != NULL) {
      newTable = utCreateTable();
      utFreeSarray(&newTable->RowNames);
      utFreeSarray(&newTable->ColumnNames);
      utFreeSarray(&newTable->CellValues);

      newTable->RowNames    = utAssignSarray(table->RowNames);
      newTable->ColumnNames = utAssignSarray(table->ColumnNames);
      newTable->CellValues  = utAssignSarray(table->CellValues);
   }

   return(newTable);
}


#ifdef __STDC__
void utSaveTable(pTable table,
                 FILE *fp)
#else
void utSaveTable(table,
                 fp)
pTable  table;
FILE   *fp;
#endif
{
   utWritelnFortranString(fp,"TABLE VERSION 1.0");

   utSaveSarray(table->RowNames,fp);
   utSaveSarray(table->ColumnNames,fp);
   utSaveSarray(table->CellValues,fp);
}


#ifdef __STDC__
pTable utLoadTable(FILE *fp)
#else
pTable utLoadTable(fp)
FILE *fp;
#endif
{
   pTable table;
   char   version[MAX_STRING_LENGTH+1];

   utReadFortranString(fp,version,MAX_STRING_LENGTH);
   assert(utEqualStrings(version,"TABLE VERSION 1.0"));

   table = utCreateTable();

   utFreeSarray(&table->RowNames);
   utFreeSarray(&table->ColumnNames);
   utFreeSarray(&table->CellValues);

   table->RowNames    = utLoadSarray(fp);
   table->ColumnNames = utLoadSarray(fp);
   table->CellValues  = utLoadSarray(fp);

   return(table);
}


#ifdef __STDC__
char *utGetTableCellValue(pTable table,
                          int rowNumber,
                          int columnNumber)
#else
char *utGetTableCellValue(table,
                          rowNumber,
                          columnNumber)
pTable table;
int    rowNumber;
int    columnNumber;
#endif
{
   int   nRowNames = utGetCountSarray(table->RowNames);
   int   location = columnNumber*nRowNames+rowNumber;
   char *cellValue = NULL;
   
   if(location < utGetCountSarray(table->CellValues))
      cellValue = utGetCstringAtPositionInSarray(table->CellValues,location);

   return(cellValue);
}


#ifdef __STDC__
void utSetTableCellValue(pTable table,
                         int rowNumber,
                         int columnNumber,
                         char *cellValue)
#else
void utSetTableCellValue(table,
                         rowNumber,
                         columnNumber,
                         cellValue)
pTable  table;
int     rowNumber;
int     columnNumber;
char   *cellValue;
#endif
{
   int nRowNames = utGetCountSarray(table->RowNames);
   int location = columnNumber*nRowNames+rowNumber;

   utSetCstringAtPositionInSarray(table->CellValues,
                                  location,
                                  cellValue);
}


#ifdef __STDC__
void utAppendTableCellValue(pTable table,
                            int rowNumber,
                            int columnNumber,
                            char *cellValue,
                            char *delimiter)
#else
void utAppendTableCellValue(table,
                            rowNumber,
                            columnNumber,
                            cellValue,
                            delimiter)
pTable  table;
int     rowNumber;
int     columnNumber;
char   *cellValue;
char   *delimiter;
#endif
{
   int nRowNames = utGetCountSarray(table->RowNames);
   int location = columnNumber*nRowNames+rowNumber;

   utApendStringAtPositionInSarray(table->CellValues,
                                   location,
                                   cellValue,
                                   delimiter);
}


#ifdef __STDC__
pTable utMapTableForNewRowNames(pTable table,
                                pSarray rowNames)
#else
pTable utMapTableForNewRowNames(table,
                                rowNames)
pTable  table;
pSarray rowNames;
#endif
{
   pTable  newTable = utCreateTable();
   int     nColumnNames = utGetCountSarray(table->ColumnNames);
   int     nRowNames = utGetCountSarray(rowNames);
   int     oldRowNamesIndex;
   int     i1,j1;
   char   *cellValue;
   char   *rowName;

   utFreeSarray(&newTable->RowNames);
   utFreeSarray(&newTable->ColumnNames);

   newTable->ColumnNames = utAssignSarray(table->ColumnNames);
   newTable->RowNames    = utAssignSarray(rowNames);

   for(i1=0 ; i1<nRowNames ; i1++) {
      rowName = utGetCstringAtPositionInSarray(rowNames,i1);
      oldRowNamesIndex = utStringPositionInSarray(rowName,table->RowNames);
      if(oldRowNamesIndex >= 0) {
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            oldRowNamesIndex,
                                            j1);
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                cellValue);
         }
      } else {
         for(j1=0 ; j1<nColumnNames ; j1++) {
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                "");
         }
      }
   }

   return(newTable);
}


#ifdef __STDC__
pTable utMapTableForNewColumnNames(pTable table,
                                   pSarray columnNames)
#else
pTable utMapTableForNewColumnNames(table,
                                   columnNames)
pTable  table;
pSarray columnNames;
#endif
{
   pTable  newTable = utCreateTable();
   int     nColumnNames = utGetCountSarray(columnNames);
   int     nRowNames = utGetCountSarray(table->RowNames);
   int     oldColumnIndex;
   int     i1,j1;
   char   *cellValue;
   char   *columnName;

   utFreeSarray(&newTable->RowNames);
   utFreeSarray(&newTable->ColumnNames);

   newTable->ColumnNames = utAssignSarray(columnNames);
   newTable->RowNames    = utAssignSarray(table->RowNames);

   for(j1=0 ; j1<nColumnNames ; j1++) {
      columnName = utGetCstringAtPositionInSarray(columnNames,j1);
      oldColumnIndex = utStringPositionInSarray(columnName,table->ColumnNames);
      if(oldColumnIndex >= 0) {
         for(i1=0 ; i1<nRowNames ; i1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,
                                            oldColumnIndex);
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                cellValue);
         }
      } else {
         for(i1=0 ; i1<nRowNames ; i1++) {
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                "");
         }
      }
   }

   return(newTable);
}


#ifdef __STDC__
pTable utMapRecipeTableForNewColumnNames(pTable oldT,pTable newT)
#else
pTable utMapRecipeTableForNewColumnNames(oldT,newT)
pTable  oldT;
pTable  newT;
#endif
{
   pTable  newTable = utCreateTable();
   int     nOldColumnNames = utGetCountSarray(oldT->ColumnNames);
   int     nOldRowNames = utGetCountSarray(oldT->RowNames);
   int     nNewColumnNames = utGetCountSarray(newT->ColumnNames);
   int     oldColumnIndex;
   int     i1,j1;
   char   *cellValue;
   char   *columnName;
   pSarray newColumnNames = newT->ColumnNames;

   utFreeSarray(&newTable->RowNames);
   utFreeSarray(&newTable->ColumnNames);

   newTable->ColumnNames = utAssignSarray(newT->ColumnNames);
   newTable->RowNames    = utAssignSarray(oldT->RowNames);

   for(j1=0 ; j1<nNewColumnNames ; j1++) {
      columnName = utGetCstringAtPositionInSarray(newColumnNames,j1);
      oldColumnIndex = utStringPositionInSarray(columnName,
                                                oldT->ColumnNames);
      if(oldColumnIndex >= 0) {
         for(i1=0 ; i1<nOldRowNames ; i1++) {
            cellValue = utGetTableCellValue(oldT,
                                            i1,
                                            oldColumnIndex);
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                cellValue);
         }
      } else {
         for(i1=0 ; i1<3 ; i1++) {
            cellValue = utGetTableCellValue(newT,
                                            i1,
                                            j1);
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                cellValue);
         }
         for(i1=3 ; i1<nOldRowNames ; i1++) {
            utSetTableCellValue(newTable,
                                i1,
                                j1,
                                "");
         }
      }
   }

   return(newTable);
}


#ifdef __STDC__
void utFillTableCellsWithValue(pTable table,
                               char *cellValue)
#else
void utFillTableCellsWithValue(table,
                               cellValue)
pTable  table;
char   *cellValue;
#endif
{
   int nRowNames    = utGetCountSarray(table->RowNames);
   int nColumnNames = utGetCountSarray(table->ColumnNames);
   int nCells       = nRowNames*nColumnNames;
   int i1;
   
   utFreeSarray(&table->CellValues);
   table->CellValues = utCreateSarrayOfSize(nCells);

   for(i1=0 ; i1<nCells ; i1++)
      utAddToSarray(table->CellValues,cellValue);
}


#ifdef __STDC__
int utGetCountTableRows(pTable table)
#else
int utGetCountTableRows(table)
pTable table;
#endif
{
   int nRows = 0; 

   if(table->RowNames != NULL)
      nRows = utGetCountSarray(table->RowNames);
   
   return(nRows);
}


#ifdef __STDC__
int utGetCountTableColumns(pTable table)
#else
int utGetCountTableColumns(table)
pTable table;
#endif
{
   int nColumns = 0;

   if(table->ColumnNames != NULL)
      nColumns = utGetCountSarray(table->ColumnNames);
   
   return(nColumns);
}


#ifdef __STDC__
char *utGetTableRowName(pTable table,
                        int rowNumber)
#else
char *utGetTableRowName(table,
                        rowNumber)
pTable table;
int    rowNumber;
#endif
{
   char *rowName = utGetCstringAtPositionInSarray(table->RowNames,rowNumber);

   return(rowName);
}


#ifdef __STDC__
void utSetTableRowName(pTable table,
                       int rowNumber,
                       char *rowName)
#else
void utSetTableRowName(table,
                       rowNumber,
                       rowName)
pTable  table;
int     rowNumber;
char   *rowName;
#endif
{
   utSetCstringAtPositionInSarray(table->RowNames,
                                  rowNumber,
                                  rowName);
}


#ifdef __STDC__
int utGetTableRowNumber(pTable table,
                        char *rowName)
#else
int utGetTableRowNumber(table,
                        rowName)
pTable  table;
char   *rowName;
#endif
{
   int rowNumber = utStringPositionInSarray(rowName,table->RowNames);

   return(rowNumber);
}


#ifdef __STDC__
char *utGetTableColumnName(pTable table,
                           int columnNumber)
#else
char *utGetTableColumnName(table,
                           columnNumber)
pTable table;
int    columnNumber;
#endif
{
   char *columnName = utGetCstringAtPositionInSarray(table->ColumnNames,columnNumber);

   return(columnName);
}


#ifdef __STDC__
void utSetTableColumnName(pTable table,
                          int columnNumber,
                          char *columnName)
#else
void utSetTableColumnName(table,
                          columnNumber,
                          columnName)
pTable  table;
int     columnNumber;
char   *columnName;
#endif
{
   utSetCstringAtPositionInSarray(table->ColumnNames,
                                  columnNumber,
                                  columnName);
}


#ifdef __STDC__
int utGetTableColumnNumber(pTable table,
                           char *columnName)
#else
int utGetTableColumnNumber(table,
                           columnName)
pTable  table;
char   *columnName;
#endif
{
   int columnNumber = utStringPositionInSarray(columnName,table->ColumnNames);

   return(columnNumber);
}


#ifdef __STDC__
pTable utRemoveEmptyRowsFromTable(pTable oldTable,
                                  char *emptyCellValue)
#else
pTable utRemoveEmptyRowsFromTable(oldTable,
                                  emptyCellValue)
pTable  oldTable;
char   *emptyCellValue;
#endif
{
   pTable   newTable = NULL;
   pSarray  newRowNames;
   int      nColumnNames = utGetCountSarray(oldTable->ColumnNames);
   int      nRowNames = utGetCountSarray(oldTable->RowNames);
   int      i1,j1;
   char    *cellValue;
   char    *rowName;

   newRowNames = utCreateSarrayOfSize(nRowNames);
   for(i1=0 ; i1<nRowNames ; i1++) {
      rowName = utGetCstringAtPositionInSarray(oldTable->RowNames,i1);
      for(j1=0 ; j1<nColumnNames ; j1++) {
         cellValue = utGetTableCellValue(oldTable,
                                         i1,
                                         j1);
         if(!utEqualStrings(cellValue,emptyCellValue)) {
            utAddToSarray(newRowNames,rowName);
            break;
         }
      }
   }

   if(utGetCountSarray(newRowNames) != nRowNames)
      newTable = utMapTableForNewRowNames(oldTable,newRowNames);
   else
      newTable = utAssignTable(oldTable);

   utFreeSarray(&newRowNames);

   return(newTable);
}


#ifdef __STDC__
pTable utRemoveEmptyColumnsFromTable(pTable oldTable,
                                     char *emptyCellValue)
#else
pTable utRemoveEmptyColumnsFromTable(oldTable,
                                     emptyCellValue)
pTable  oldTable;
char   *emptyCellValue;
#endif
{
   pTable   newTable = NULL;
   pSarray  newColumnNames;
   int      nColumnNames = utGetCountSarray(oldTable->ColumnNames);
   int      nRowNames = utGetCountSarray(oldTable->RowNames);
   int      i1,j1;
   char    *cellValue;
   char    *columnName;

   newColumnNames = utCreateSarrayOfSize(nColumnNames);
   for(j1=0 ; j1<nColumnNames ; j1++) {
      columnName = utGetCstringAtPositionInSarray(oldTable->ColumnNames,j1);
      for(i1=0 ; i1<nRowNames ; i1++) {
         cellValue = utGetTableCellValue(oldTable,
                                         i1,
                                         j1);
         if(!utEqualStrings(cellValue,emptyCellValue)) {
            utAddToSarray(newColumnNames,columnName);
            break;
         }
      }
   }

   if(utGetCountSarray(newColumnNames) != nColumnNames)
      newTable = utMapTableForNewColumnNames(oldTable,newColumnNames);
   else
      newTable = utAssignTable(oldTable);

   utFreeSarray(&newColumnNames);

   return(newTable);
}


#ifdef __STDC__
void utMergeRowAfterRowInTable(pTable table,
                               int addAfterRowNumber,
                               char *rowName,
                               pSarray row,
                               char *emptyCellValue)
#else
void utMergeRowAfterRowInTable(table,
                               addAfterRowNumber,
                               rowName,
                               row,
                               emptyCellValue)
pTable   table;
int      addAfterRowNumber;
char    *rowName,
pSarray  row,
char    *emptyCellValue;
#endif
{
   pTable  originalTable = utAssignTable(table);
   int     nOldRows,nOldColumns;
   int     nNewColumns;
   int     i1,j1;
   char   *cellValue;

   nOldRows = utGetCountSarray(originalTable->RowNames);
   if(addAfterRowNumber == nOldRows-1)
      utAddToSarray(table->RowNames,rowName);
   else
      utInsertInSarray(table->RowNames,addAfterRowNumber+1,rowName);

   nOldColumns = utGetCountSarray(originalTable->ColumnNames);
   nNewColumns = utGetCountSarray(row);
   for(j1=nOldColumns ; j1<nNewColumns ; j1++)
      utAddToSarray(table->ColumnNames,"");

   utFillTableCellsWithValue(table,emptyCellValue);

   for(i1=0 ; i1<=addAfterRowNumber ; i1++) {
      for(j1=0 ; j1<nOldColumns ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table,
                             i1,j1,
                             cellValue);
      }
   }

   for(i1=addAfterRowNumber+1 ; i1<nOldRows ; i1++) {
      for(j1=0 ; j1<nOldColumns ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table,
                             i1+1,j1,
                             cellValue);
      }
   }

   for(j1=0 ; j1<nNewColumns ; j1++) {
      cellValue = utGetCstringAtPositionInSarray(row,j1);
      utSetTableCellValue(table,
                          addAfterRowNumber+1,j1,
                          cellValue);
   }

   utFreeTable(&originalTable);
}


#ifdef __STDC__
void utMergeColumnAfterColumnInTable(pTable table,
                                     int addAfterColumnNumber,
                                     char *columnName,
                                     pSarray column,
                                     char *emptyCellValue)
#else
void utMergeColumnAfterColumnInTable(table,
                                     addAfterColumnNumber,
                                     columnName,
                                     column,
                                     emptyCellValue)
pTable   table;
int      addAfterColumnNumber;
char    *columnName;
pSarray  column;
char    *emptyCellValue;
#endif
{
   pTable  originalTable = utAssignTable(table);
   int     nOldRows,nOldColumns;
   int     nNewRows;
   int     i1,j1;
   char   *cellValue;

   nOldColumns = utGetCountSarray(originalTable->ColumnNames);
   if(addAfterColumnNumber == nOldColumns-1)
      utAddToSarray(table->ColumnNames,columnName);
   else
      utInsertInSarray(table->ColumnNames,addAfterColumnNumber+1,columnName);

   nOldRows = utGetCountSarray(originalTable->RowNames);
   nNewRows = utGetCountSarray(column);
   for(i1=nOldRows ; i1<nNewRows ; i1++)
      utAddToSarray(table->RowNames,"");

   utFillTableCellsWithValue(table,emptyCellValue);

   for(i1=0 ; i1<nOldRows ; i1++) {
      for(j1=0 ; j1<=addAfterColumnNumber ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table,
                             i1,j1,
                             cellValue);
      }
   }

   for(i1=0 ; i1<nOldRows ; i1++) {
      for(j1=addAfterColumnNumber+1 ; j1<nOldColumns ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table,
                             i1,j1+1,
                             cellValue);
      }
   }

   for(i1=0 ; i1<nNewRows ; i1++) {
      cellValue = utGetCstringAtPositionInSarray(column,i1);
      utSetTableCellValue(table,
                          i1,addAfterColumnNumber+1,
                          cellValue);
   }

   utFreeTable(&originalTable);
}


#ifdef __STDC__
void utMergeTableAfterRowInTable(pTable table1,
                                 int addAfterRowNumber,
                                 pTable table2,
                                 char *emptyCellValue)
#else
void utMergeTableAfterRowInTable(table1,
                                 addAfterRowNumber,
                                 table2,
                                 emptyCellValue)
pTable  table1;
int     addAfterRowNumber;
pTable  table2;
char   *emptyCellValue;
#endif
{
   pTable  originalTable = utAssignTable(table1);
   int     nOldRows,nOldColumns;
   int     nNewRows,nNewColumns;
   int     i1,j1;
   char   *cellValue;
   char   *rowName;
   char   *columnName;
   int     columnNumber;

   nOldRows = utGetCountSarray(originalTable->RowNames);
   nNewRows = utGetCountSarray(table2->RowNames);
   if(addAfterRowNumber == nOldRows-1) {
      for(i1=0 ; i1<nNewRows ; i1++) {
         rowName = utGetCstringAtPositionInSarray(table2->RowNames,i1);
         utAddToSarray(table1->RowNames,rowName);
      }
   } else {
      for(i1=nNewRows-1 ; i1>=0 ; i1--) {
         rowName = utGetCstringAtPositionInSarray(table2->RowNames,i1);
         utInsertInSarray(table1->RowNames,addAfterRowNumber+1,rowName);
      }
   }

   nOldColumns = utGetCountSarray(originalTable->ColumnNames);
   nNewColumns = utGetCountSarray(table2->ColumnNames);
   for(j1=0 ; j1<nNewColumns ; j1++) {
      columnName = utGetCstringAtPositionInSarray(table2->ColumnNames,j1);
      columnNumber = utGetTableColumnNumber(originalTable,columnName);
      if(columnNumber < 0)
         utAddToSarray(table1->ColumnNames,columnName);
   }

   utFillTableCellsWithValue(table1,emptyCellValue);

   for(i1=0 ; i1<=addAfterRowNumber ; i1++) {
      for(j1=0 ; j1<nOldColumns ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table1,
                             i1,j1,
                             cellValue);
      }
   }

   for(i1=addAfterRowNumber+1 ; i1<nOldRows ; i1++) {
      for(j1=0 ; j1<nOldColumns ; j1++) {
         cellValue = utGetTableCellValue(originalTable,
                                         i1,j1);
         utSetTableCellValue(table1,
                             i1+nNewRows,j1,
                             cellValue);
      }
   }

   for(j1=0 ; j1<nNewColumns ; j1++) {
      columnName = utGetCstringAtPositionInSarray(table2->ColumnNames,j1);
      columnNumber = utGetTableColumnNumber(table1,columnName);
      for(i1=0 ; i1<nNewRows ; i1++) {
         cellValue = utGetTableCellValue(table2,
                                         i1,j1);
         utSetTableCellValue(table1,
                             addAfterRowNumber+1+i1,columnNumber,
                             cellValue);
      }
   }

   utFreeTable(&originalTable);
}


#ifdef __STDC__
int utGetLongStringSizeTableColumn(pTable table,
                                   int columnNumber,
                                   utLogical includeColumnHeader)
#else
int utGetLongStringSizeTableColumn(table,
                                   columnNumber,
                                   includeColumnHeader)
pTable    table;
int       columnNumber;
utLogical includeColumnHeader;
#endif
{
   int   longestStringSize = 0;
   int   nRowNames = utGetCountSarray(table->RowNames);
   int   i1;
   char *cellValue;
   char *columnName;

   if(includeColumnHeader) {
      columnName = utGetCstringAtPositionInSarray(table->ColumnNames,columnNumber);
      longestStringSize = utStringLength(columnName);
   }

   for(i1=0 ; i1<nRowNames ; i1++) {
      cellValue = utGetTableCellValue(table,
                                      i1,columnNumber);
      longestStringSize = MAX(longestStringSize,utStringLength(cellValue));
   }

   return(longestStringSize);
}


#ifdef __STDC__
void utWriteTableToFile(pTable table,
                        utLogical writeColumnHeaders,
                        char *rowNameColumnHeader,
                        int identationSize,
                        int columnSpacing,
                        char *emptyCellValue,
                        utLogical writeEmptyCellsAsBlank,
                        utLogical writeEmptyRows,
                        utLogical centerCellValues,
                        FILE *fp)
#else
void utWriteTableToFile(table,
                        writeColumnHeaders,
                        rowNameColumnHeader,
                        identationSize,
                        columnSpacing,
                        emptyCellValue,
                        writeEmptyCellsAsBlank,
                        writeEmptyRows,
                        centerCellValues,
                        fp)
pTable     table;
utLogical  writeColumnHeaders;
char      *rowNameColumnHeader;
int        identationSize;
int        columnSpacing;
char      *emptyCellValue;
utLogical  writeEmptyCellsAsBlank;
utLogical  writeEmptyRows;
utLogical  centerCellValues;
FILE      *fp;
#endif
{
/*
   int        nRowNames = utGetCountSarray(table->RowNames);
   int        nColumnNames = utGetCountSarray(table->ColumnNames);
   int        i1,j1;
   char      *cellValue;
   char      *rowName;
   int        rowNameColumnWidth;
   char      *columnName;
   pIarray    columnWidths = utCreateIarrayOfSize(nColumnNames);
   int        columnWidth;
   char      *printedString;
   char      *identationString = utPadStringToLength("",identationSize);
   char      *columnSpacingString = utPadStringToLength("",columnSpacing);
   utLogical  writeRow;

   rowNameColumnWidth = utGetLongestStringSizeInSarray(table->RowNames);
   if(writeColumnHeaders)
      rowNameColumnWidth = MAX(rowNameColumnWidth,
                               utStringLength(rowNameColumnHeader));

   for(j1=0 ; j1<nColumnNames ; j1++) {
      columnWidth = utGetLongStringSizeTableColumn(table,j1,writeColumnHeaders);
      utAddToIarray(columnWidths,columnWidth);
   }

   if(writeColumnHeaders) {
      printedString = utPadStringToLength(rowNameColumnHeader,rowNameColumnWidth);
      fprintf(fp,"%s%s",identationString,
                        printedString);
      utFreeBytes((pVoid)printedString);

      for(j1=0 ; j1<nColumnNames ; j1++) {
         columnName = utGetCstringAtPositionInSarray(table->ColumnNames,j1);
         columnWidth = utGetIntAtPositionInIarray(columnWidths,j1);
         printedString = utCenterString(columnName,columnWidth);
         fprintf(fp,"%s%s",columnSpacingString,
                           printedString);
         utFreeBytes((pVoid)printedString);
      }
      fprintf(fp,"\n");

      printedString = utCreateStringOfChar('-',rowNameColumnWidth);
      fprintf(fp,"%s%s",identationString,
                        printedString);
      utFreeBytes((pVoid)printedString);

      for(j1=0 ; j1<nColumnNames ; j1++) {
         columnWidth = utGetIntAtPositionInIarray(columnWidths,j1);
         printedString = utCreateStringOfChar('-',columnWidth);
         fprintf(fp,"%s%s",columnSpacingString,
                           printedString);
         utFreeBytes((pVoid)printedString);
      }
      fprintf(fp,"\n");
   }

   for(i1=0 ; i1<nRowNames ; i1++) {
      writeRow = utTrue;
      if(!writeEmptyRows) {
         writeRow = utFalse;
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if(!utHasAllBlanks(cellValue)) {
               writeRow = utTrue;
               break;
            }
         }
      }
      if(writeRow) {
         rowName = utGetCstringAtPositionInSarray(table->RowNames,i1);
         printedString = utPadStringToLength(rowName,rowNameColumnWidth);
         fprintf(fp,"%s%s",identationString,
                           printedString);
         utFreeBytes((pVoid)printedString);

         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            columnWidth = utGetIntAtPositionInIarray(columnWidths,j1);
            if(centerCellValues)
               printedString = utCenterString(cellValue,columnWidth);
            else
               printedString = utPadStringToLength(cellValue,columnWidth);
            fprintf(fp,"%s%s",columnSpacingString,
                              printedString);
            utFreeBytes((pVoid)printedString);
         }
         fprintf(fp,"\n");
      }
   }

   utFreeBytes((pVoid)columnSpacingString);
   utFreeBytes((pVoid)identationString);
*/
}


#ifdef __STDC__
void utWriteTableToFileAsHTML(pTable table,
                              utLogical writeRowNames,
                              utLogical writeColumnNames,
                              char *rowNameColumnHeader,
                              char *emptyCellValue,
                              utLogical writeEmptyCellsAsBlank,
                              utLogical writeEmptyRows,
                              FILE *fp)
#else
void utWriteTableToFileAsHTML(table,
                              writeRowNames,
                              writeColumnNames,
                              rowNameColumnHeader,
                              emptyCellValue,
                              writeEmptyCellsAsBlank,
                              writeEmptyRows,
                              fp)
pTable     table;
utLogical  writeRowNames;
utLogical  writeColumnNames;
char      *rowNameColumnHeader;
char      *emptyCellValue;
utLogical  writeEmptyCellsAsBlank;
utLogical  writeEmptyRows;
FILE      *fp;
#endif
{
   int        nRowNames = utGetCountSarray(table->RowNames);
   int        nColumnNames = utGetCountSarray(table->ColumnNames);
   int        i1,j1;
   char      *cellValue;
   char      *rowName;
   char      *columnName;
   utLogical  writeRow;

   fprintf(fp,"<table cellpadding=\"0\" cellspacing=\"1\">\n");
   if(writeColumnNames) {
      fprintf(fp,"   <tr>\n");
      if(writeRowNames)
         fprintf(fp,"      <th>%s</th>\n",rowNameColumnHeader);

      for(j1=0 ; j1<nColumnNames ; j1++) {
         columnName = utGetCstringAtPositionInSarray(table->ColumnNames,j1);
         fprintf(fp,"      <th>%s</th>\n",columnName);
      }
      fprintf(fp,"   </tr>\n");
   }

   for(i1=0 ; i1<nRowNames ; i1++) {
      writeRow = utTrue;
      if(!writeEmptyRows) {
         writeRow = utFalse;
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if(!utHasAllBlanks(cellValue)) {
               writeRow = utTrue;
               break;
            }
         }
      }
      if(writeRow) {
         fprintf(fp,"   <tr>\n");
         if(writeRowNames) {
            rowName = utGetCstringAtPositionInSarray(table->RowNames,i1);
            fprintf(fp,"      <td>%s</td>\n",rowName);
         }

         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            fprintf(fp,"      <td>%s</td>\n",cellValue);
         }
         fprintf(fp,"   </tr>\n");
      }
   }
   fprintf(fp,"</table>\n");
}


#ifdef __STDC__
void utWriteTableToTXTFile(pTable table,
                           utLogical writeRowNames,
                           utLogical writeColumnHeaders,
                           char *rowNameColumnHeader,
                           char *emptyCellValue,
                           utLogical writeEmptyCellsAsBlank,
                           utLogical writeEmptyRows,
                           FILE *fp)
#else
void utWriteTableToTXTFile(table,
                           writeRowNames,
                           writeColumnHeaders,
                           rowNameColumnHeader,
                           emptyCellValue,
                           writeEmptyCellsAsBlank,
                           writeEmptyRows,
                           fp)
pTable     table;
utLogical  writeRowNames;
utLogical  writeColumnHeaders;
char      *rowNameColumnHeader;
char      *emptyCellValue;
utLogical  writeEmptyCellsAsBlank;
utLogical  writeEmptyRows;
FILE      *fp;
#endif
{
   int        nRowNames = utGetCountSarray(table->RowNames);
   int        nColumnNames = utGetCountSarray(table->ColumnNames);
   int        i1,j1;
   char      *cellValue;
   char      *rowName;
   char      *columnName;
   utLogical  writeRow;

   if(writeColumnHeaders) {
      if(writeRowNames)
         fprintf(fp,"%s",rowNameColumnHeader);
      for(j1=0 ; j1<nColumnNames ; j1++) {
         columnName = utGetCstringAtPositionInSarray(table->ColumnNames,j1);
         if((j1 == 0) && !writeRowNames)
            fprintf(fp,"%s",columnName);
         else
            fprintf(fp,"\t%s",columnName);
      }
      fprintf(fp,EOLSTRING);
   }

   for(i1=0 ; i1<nRowNames ; i1++) {
      writeRow = utTrue;
      if(!writeEmptyRows) {
         writeRow = utFalse;
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if(!utHasAllBlanks(cellValue)) {
               writeRow = utTrue;
               break;
            }
         }
      }
      if(writeRow) {
         if(writeRowNames) {
            rowName = utGetCstringAtPositionInSarray(table->RowNames,i1);
            fprintf(fp,"%s",rowName);
         }
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(utIsNullString(cellValue))
               cellValue = "";
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if((j1 == 0) && !writeRowNames)
               fprintf(fp,"%s",cellValue);
            else
               fprintf(fp,"\t%s",cellValue);
         }
         fprintf(fp,EOLSTRING);
      }
   }
}


#ifdef __STDC__
void utWriteTableToCSVFile(pTable table,
                           utLogical writeRowNames,
                           utLogical writeColumnHeaders,
                           char *rowNameColumnHeader,
                           char *emptyCellValue,
                           utLogical writeEmptyCellsAsBlank,
                           utLogical writeEmptyRows,
                           FILE *fp)
#else
void utWriteTableToCSVFile(table,
                           writeRowNames,
                           writeColumnHeaders,
                           rowNameColumnHeader,
                           emptyCellValue,
                           writeEmptyCellsAsBlank,
                           writeEmptyRows,
                           fp)
pTable     table;
utLogical  writeRowNames;
utLogical  writeColumnHeaders;
char      *rowNameColumnHeader;
char      *emptyCellValue;
utLogical  writeEmptyCellsAsBlank;
utLogical  writeEmptyRows;
FILE      *fp;
#endif
{
   int        nRowNames = utGetCountSarray(table->RowNames);
   int        nColumnNames = utGetCountSarray(table->ColumnNames);
   int        i1,j1;
   char      *cellValue;
   char      *rowName;
   char      *columnName;
   utLogical  writeRow;

   if(writeColumnHeaders) {
      if(writeRowNames)
         fprintf(fp,"%s",rowNameColumnHeader);
      for(j1=0 ; j1<nColumnNames ; j1++) {
         columnName = utGetCstringAtPositionInSarray(table->ColumnNames,j1);
         if((j1 == 0) && !writeRowNames)
            fprintf(fp,"%s",columnName);
         else
            fprintf(fp,",%s",columnName);
      }
      fprintf(fp,EOLSTRING);
   }

   for(i1=0 ; i1<nRowNames ; i1++) {
      writeRow = utTrue;
      if(!writeEmptyRows) {
         writeRow = utFalse;
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,
                                            i1,j1);
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if(!utHasAllBlanks(cellValue)) {
               writeRow = utTrue;
               break;
            }
         }
      }
      if(writeRow) {
         if(writeRowNames) {
            rowName = utGetCstringAtPositionInSarray(table->RowNames,i1);
            fprintf(fp,"%s",rowName);
         }
         for(j1=0 ; j1<nColumnNames ; j1++) {
            cellValue = utGetTableCellValue(table,i1,j1);
            if(utIsNullString(cellValue))
               cellValue = "";
            if(writeEmptyCellsAsBlank) {
               if(utEqualStrings(cellValue,emptyCellValue))
                  cellValue = " ";
            }
            if((j1 == 0) && !writeRowNames) {
//             fprintf(fp,"%s",cellValue);
               utWriteCSVFieldString(fp,cellValue);
            } else {
//             fprintf(fp,",%s",cellValue);
               fprintf(fp,",");
               utWriteCSVFieldString(fp,cellValue);
            }
         }
         fprintf(fp,EOLSTRING);
      }
   }
}


#ifdef __STDC__
pTable utLoadCSVStringsAsTable(FILE *fp)
#else
pTable utLoadCSVStringsAsTable(fp)
FILE *fp;
#endif
{
   pTable   table = utCreateTable();
   char    *row;
   char    *field;
   int      fieldNumber;
   int      rowNumber = 0;
   pSarray  cellValues = utCreateSarray();
   int      i1,nRows;
   int      j1,nColumns;

   row = utReadUnlimitedLine(fp);
   if(!utHasAllBlanks(row)) {
      fieldNumber = 1;
      field = utGetDoubleQuotedCSVField(row,fieldNumber);
      while(!utIsNullString(field)) {
         if(fieldNumber > 1)
            utSetTableColumnName(table,
                                 fieldNumber-2,
                                 field);
         utFreeBytes((pVoid)field);
         fieldNumber++;
         field = utGetDoubleQuotedCSVField(row,fieldNumber);
      }
      utFreeBytes((pVoid)field);
      utFreeBytes((pVoid)row);

      nColumns = utGetCountTableColumns(table);
      row = utReadUnlimitedLine(fp);
      while(!utHasAllBlanks(row)) {
         fieldNumber = 1;
         field = utGetDoubleQuotedCSVField(row,fieldNumber);
         utSetTableRowName(table,
                           rowNumber,
                           field);
         utFreeBytes((pVoid)field);

         while(fieldNumber <= nColumns) {
            fieldNumber++;
            field = utGetDoubleQuotedCSVField(row,fieldNumber);
            utAddToSarray(cellValues,field);
            utFreeBytes((pVoid)field);
         }

         utFreeBytes((pVoid)row);
         row = utReadUnlimitedLine(fp);
         rowNumber++;
      }

      nRows = utGetCountTableRows(table);
      fieldNumber = 0;
      for(i1=0 ; i1<nRows ; i1++) {
         for(j1=0 ; j1<nColumns ; j1++) {
            field = utGetCstringAtPositionInSarray(cellValues,fieldNumber);
            utSetTableCellValue(table,
                                i1,
                                j1,
                                field);
            fieldNumber++;
         }
      }
      utFreeSarray(&cellValues);
   }
   utFreeBytes((pVoid)row);

   return(table);
}


#ifdef __STDC__
pTable utLoadCSVFileAsTable(FILE *fp)
#else
pTable utLoadCSVFileAsTable(fp)
FILE *fp;
#endif
{
   pTable   table = utCreateTable();
   char    *row;
   char    *field;
   int      fieldNumber;
   int      rowNumber = 0;
   pSarray  cellValues = utCreateSarray();
   int      i1,nRows;
   int      j1,nColumns;

   row = utReadUnlimitedLine(fp);
   if(!utHasAllBlanks(row)) {
      fieldNumber = 1;
      field = utGetDoubleQuotedCSVField(row,fieldNumber);
      while(!utIsNullString(field)) {
         utSetTableColumnName(table,fieldNumber-1,field);
         utFreeBytes((pVoid)field);
         fieldNumber++;
         field = utGetDoubleQuotedCSVField(row,fieldNumber);
      }
      utFreeBytes((pVoid)field);
      //utFreeBytes((pVoid)row);

      nColumns = utGetCountTableColumns(table);
      //row = utReadUnlimitedLine(fp);
      while(!utHasAllBlanks(row)) {
         fieldNumber = 1;
         field = utGetDoubleQuotedCSVField(row,fieldNumber);
         utSetTableRowName(table,
                           rowNumber,
                           field);
         utFreeBytes((pVoid)field);

         while(fieldNumber <= nColumns) {
            field = utGetDoubleQuotedCSVField(row,fieldNumber);
            utAddToSarray(cellValues,field);
            utFreeBytes((pVoid)field);
            fieldNumber++;
         }

         utFreeBytes((pVoid)row);
         row = utReadUnlimitedLine(fp);
         rowNumber++;
      }

      nRows = utGetCountTableRows(table);
      fieldNumber = 0;
      for(i1=0 ; i1<nRows ; i1++) {
         for(j1=0 ; j1<nColumns ; j1++) {
            field = utGetCstringAtPositionInSarray(cellValues,fieldNumber);
            utSetTableCellValue(table,
                                i1,
                                j1,
                                field);
            fieldNumber++;
         }
      }
      utFreeSarray(&cellValues);
   }
   utFreeBytes((pVoid)row);

   return(table);
}


#ifdef __STDC__
pTable utLoadCSVFortranStringsAsTable(FILE *fp)
#else
pTable utLoadCSVFortranStringsAsTable(fp)
FILE *fp;
#endif
{
   pTable   table = utCreateTable();
   char    *row;
   char    *field;
   int      fieldNumber;
   int      rowNumber = 0;
   pSarray  cellValues = utCreateSarray();
   int      i1,nRows;
   int      j1,nColumns;

   row = utReadUnlimitedLine(fp);
   if(!utHasAllBlanks(row)) {
      fieldNumber = 1;
      field = utGetSingleQuotedCSVField(row,fieldNumber);
      while(!utIsNullString(field)) {
         if(fieldNumber > 1)
            utSetTableColumnName(table,
                                 fieldNumber-2,
                                 field);
         utFreeBytes((pVoid)field);
         fieldNumber++;
         field = utGetSingleQuotedCSVField(row,fieldNumber);
      }
      utFreeBytes((pVoid)field);
      utFreeBytes((pVoid)row);

      nColumns = utGetCountTableColumns(table);
      row = utReadUnlimitedLine(fp);
      while(!utHasAllBlanks(row)) {
         fieldNumber = 1;
         field = utGetSingleQuotedCSVField(row,fieldNumber);
         utSetTableRowName(table,
                           rowNumber,
                           field);
         utFreeBytes((pVoid)field);

         while(fieldNumber <= nColumns) {
            fieldNumber++;
            field = utGetSingleQuotedCSVField(row,fieldNumber);
            utAddToSarray(cellValues,field);
            utFreeBytes((pVoid)field);
         }

         utFreeBytes((pVoid)row);
         row = utReadUnlimitedLine(fp);
         rowNumber++;
      }

      nRows = utGetCountTableRows(table);
      fieldNumber = 0;
      for(i1=0 ; i1<nRows ; i1++) {
         for(j1=0 ; j1<nColumns ; j1++) {
            field = utGetCstringAtPositionInSarray(cellValues,fieldNumber);
            utSetTableCellValue(table,
                                i1,
                                j1,
                                field);
            fieldNumber++;
         }
      }
      utFreeSarray(&cellValues);
   }
   utFreeBytes((pVoid)row);

   return(table);
}


#ifdef __STDC__
pIarray utGetMatchingTableColumns(pTable table,
                                  pIarray rowNumbers,
                                  pSarray rowValues,
                                  utLogical allBlankValuesAreEqual)
#else
pIarray utGetMatchingTableColumns(table,
                                  rowNumbers,
                                  rowValues,
                                  allBlankValuesAreEqual)
pTable    table;
pIarray   rowNumbers;
pSarray   rowValues;
utLogical allBlankValuesAreEqual;
#endif
{
/*
   pIarray    columnNumbers = utCreateIarray();
   int        rowNumber;
   char      *rowValue;
   char      *cellValue;
   int        k1,nKeys;
   int        j1,nColumns;
   utLogical  addColumn;

   nKeys = utGetCountIarray(rowNumbers);
   nColumns = utGetCountTableColumns(table);
   for(j1=0 ; j1<nColumns ; j1++) {
      addColumn = utTrue;
      for(k1=0 ; k1<nKeys ; k1++) {
         rowNumber = utGetIntAtPositionInIarray(rowNumbers,k1);
         rowValue = utGetCstringAtPositionInSarray(rowValues,k1);
         cellValue = utGetTableCellValue(table,
                                         rowNumber,
                                         j1);
         if(!utEqualStrings(cellValue,rowValue)) {
            if(!allBlankValuesAreEqual || !utHasAllBlanks(cellValue) || !utHasAllBlanks(rowValue)) {
               addColumn = utFalse;
               break;
            }
         }
      }
      if(addColumn)
         utAddToIarray(columnNumbers,j1);
   }

   return(columnNumbers);
*/
}


#ifdef __STDC__
pIarray utGetMatchingTableRows(pTable table,
                               pIarray columnNumbers,
                               pSarray columnValues,
                               utLogical allBlankValuesAreEqual)
#else
pIarray utGetMatchingTableRows(table,
                               columnNumbers,
                               columnValues,
                               allBlankValuesAreEqual)
pTable    table;
pIarray   columnNumbers;
pSarray   columnValues;
utLogical allBlankValuesAreEqual;
#endif
{
/*
   pIarray    rowNumbers = utCreateIarray();
   int        columnNumber;
   char      *columnValue;
   char      *cellValue;
   int        k1,nKeys;
   int        i1,nRows;
   utLogical  addRow;

   nKeys = utGetCountIarray(columnNumbers);
   nRows = utGetCountTableRows(table);
   for(i1=0 ; i1<nRows ; i1++) {
      addRow = utTrue;
      for(k1=0 ; k1<nKeys ; k1++) {
         columnNumber = utGetIntAtPositionInIarray(columnNumbers,k1);
         columnValue = utGetCstringAtPositionInSarray(columnValues,k1);
         cellValue = utGetTableCellValue(table,
                                         i1,
                                         columnNumber);
         if(!utEqualStrings(cellValue,columnValue)) {
            if(!allBlankValuesAreEqual || !utHasAllBlanks(cellValue) || !utHasAllBlanks(columnValue)) {
               addRow = utFalse;
               break;
            }
         }
      }
      if(addRow)
         utAddToIarray(rowNumbers,i1);
   }

   return(rowNumbers);
*/
}


#ifdef __STDC__
void utMoveTableRowNamesToColumn1(pTable table,
                                  char *column1Name,
                                  pSarray newRowNames)
#else
void utMoveTableRowNamesToColumn1(table,
                                  column1Name,
                                  newRowNames)
pTable   table;
char    *column1Name;
pSarray  newRowNames;
#endif
{
   int   nRowNames = utGetCountSarray(table->RowNames);
   int   i1;
   char *rowName;

   if(utGetCountSarray(newRowNames) == nRowNames) {
      for(i1=nRowNames-1 ; i1>=0 ; i1--) {
         rowName = utGetCstringAtPositionInSarray(table->RowNames,i1);
         utInsertInSarray(table->CellValues,0,rowName);
      }
      utFreeSarray(&table->RowNames);
      table->RowNames = utAssignSarray(newRowNames);
      utInsertInSarray(table->ColumnNames,0,column1Name);
   }
}


#ifdef __STDC__
void utSubstituteTableCellValues(pTable table,
                                 char *cellValueBefore,
                                 char *cellValueAfter)
#else
void utSubstituteTableCellValues(table,
                                 cellValueBefore,
                                 cellValueAfter)
pTable  table;
char   *cellValueBefore;
char   *cellValueAfter;
#endif
{
   int   i1;
   int   nCellValues = utGetCountSarray(table->CellValues);
   char *cellValue;

   for(i1=0 ; i1<nCellValues ; i1++) {
      cellValue = utGetCstringAtPositionInSarray(table->CellValues,i1);
      if(utEqualStrings(cellValue,cellValueBefore))
         utSetCstringAtPositionInSarray(table->CellValues,
                                        i1,
                                        cellValueAfter);
   }
}


#ifdef __STDC__
utLogical utAreTablesEqual(pTable table1,
                           pTable table2)
#else
utLogical utAreTablesEqual(table1,
                           table2)
pTable table1;
pTable table2;
#endif
{
   utLogical areTablesEqual = utFalse;

   if(utEqualSarrays(table1->RowNames,table2->RowNames)) {
      if(utEqualSarrays(table1->ColumnNames,table2->ColumnNames)) {
         if(utEqualSarrays(table1->CellValues,table2->CellValues))
            areTablesEqual = utTrue;
      }
   }

   return(areTablesEqual);
}


