/*
   $RCSfile: uttableu.h,v $       $Revision: 1.14 $
   $State: V7_0 $                  $Date: 2003/10/01 13:47:09 $
*/
#ifndef _UTTABLEU_H
#define _UTTABLEU_H

#ifdef __cplusplus
   extern "C" {
#endif

#include "utiarryu.h"
#include "utsarryu.h"

#define STRUCTTYPE_TABLE 1049

typedef struct {
   int     StructType;
   pSarray RowNames;
   pSarray ColumnNames;
   pSarray CellValues;
} Table, *pTable;

#ifdef __STDC__
   extern pTable utCreateTable();
   extern pTable utCreateTableWithNames(pSarray rowNames,
                                        pSarray columnNames);
   extern pTable utCreateTableOfSize(int nRows,
                                     int nColumns);
   extern void utInitializeTable(pTable newTable);
   extern void utFreeTable(pTable *table);
   extern void utAddTableRow(pTable table,
                             char *rowName);
   extern void utAddTableColumn(pTable table,
                                char *columnName);
   extern pTable utAssignTable(pTable table);
   extern void utSaveTable(pTable table,
                           FILE *fp);
   extern pTable utLoadTable(FILE *fp);
   extern char *utGetTableCellValue(pTable table,
                                    int rowNumber,
                                    int columnNumber);
   extern void utSetTableCellValue(pTable table,
                                   int rowNumber,
                                   int columnNumber,
                                   char *cellValue);
   extern void utAppendTableCellValue(pTable table,
                                      int rowNumber,
                                      int columnNumber,
                                      char *cellValue,
                                      char *delimiter);
   extern pTable utMapTableForNewRowNames(pTable table,
                                          pSarray rowNames);
   extern pTable utMapTableForNewColumnNames(pTable table,
                                             pSarray columnNames);
   extern pTable utMapRecipeTableForNewColumnNames(pTable oldT,
                                                   pTable newT);
   extern void utFillTableCellsWithValue(pTable table,
                                         char *cellValue);
   extern int utGetCountTableRows(pTable table);
   extern int utGetCountTableColumns(pTable table);
   extern char *utGetTableRowName(pTable table,
                                  int rowNumber);
   extern void utSetTableRowName(pTable table,
                                 int rowNumber,
                                 char *rowName);
   extern int utGetTableRowNumber(pTable table,
                                  char *rowName);
   extern char *utGetTableColumnName(pTable table,
                                     int columnNumber);
   extern void utSetTableColumnName(pTable table,
                                    int columnNumber,
                                    char *columnName);
   extern int utGetTableColumnNumber(pTable table,
                                     char *columnName);
   extern pTable utRemoveEmptyRowsFromTable(pTable oldTable,
                                            char *emptyCellValue);
   extern pTable utRemoveEmptyColumnsFromTable(pTable oldTable,
                                               char *emptyCellValue);
   extern void utMergeRowAfterRowInTable(pTable table,
                                         int addAfterRowNumber,
                                         char *rowName,
                                         pSarray row,
                                         char *emptyCellValue);
   extern void utMergeColumnAfterColumnInTable(pTable table,
                                               int addAfterColumnNumber,
                                               char *columnName,
                                               pSarray column,
                                               char *emptyCellValue);
   extern void utMergeTableAfterRowInTable(pTable table1,
                                           int addAfterRowNumber,
                                           pTable table2,
                                           char *emptyCellValue);
   extern int utGetLongStringSizeTableColumn(pTable table,
                                             int columnNumber,
                                             utLogical includeColumnHeader);
   extern void utWriteTableToFile(pTable table,
                                  utLogical writeColumnHeaders,
                                  char *rowNameColumnHeader,
                                  int identationSize,
                                  int columnSpacing,
                                  char *emptyCellValue,
                                  utLogical writeEmptyCellsAsBlank,
                                  utLogical writeEmptyRows,
                                  utLogical centerCellValues,
                                  FILE *fp);
   extern void utWriteTableToFileAsHTML(pTable table,
                                        utLogical writeRowNames,
                                        utLogical writeColumnNames,
                                        char *rowNameColumnHeader,
                                        char *emptyCellValue,
                                        utLogical writeEmptyCellsAsBlank,
                                        utLogical writeEmptyRows,
                                        FILE *fp);
   extern void utWriteTableToTXTFile(pTable table,
                                     utLogical writeRowNames,
                                     utLogical writeColumnHeaders,
                                     char *rowNameColumnHeader,
                                     char *emptyCellValue,
                                     utLogical writeEmptyCellsAsBlank,
                                     utLogical writeEmptyRows,
                                     FILE *fp);
   extern void utWriteTableToCSVFile(pTable table,
                                     utLogical writeRowNames,
                                     utLogical writeColumnHeaders,
                                     char *rowNameColumnHeader,
                                     char *emptyCellValue,
                                     utLogical writeEmptyCellsAsBlank,
                                     utLogical writeEmptyRows,
                                     FILE *fp);
   extern pTable utLoadCSVStringsAsTable(FILE *fp);
   extern pTable utLoadCSVFileAsTable(FILE *fp);
   extern pTable utLoadCSVFortranStringsAsTable(FILE *fp);
   extern pIarray utGetMatchingTableColumns(pTable table,
                                            pIarray rowNumbers,
                                            pSarray rowValues,
                                            utLogical allBlankValuesAreEqual);
   extern pIarray utGetMatchingTableRows(pTable table,
                                         pIarray columnNumbers,
                                         pSarray columnValues,
                                         utLogical allBlankValuesAreEqual);
   extern void utMoveTableRowNamesToColumn1(pTable table,
                                            char *column1Name,
                                            pSarray newRowNames);
   extern void utSubstituteTableCellValues(pTable table,
                                           char *cellValueBefore,
                                           char *cellValueAfter);
   extern utLogical utAreTablesEqual(pTable table1,
                                     pTable table2);
#else
   extern pTable utCreateTable();
   extern pTable utCreateTableWithNames();
   extern pTable utCreateTableOfSize();
   extern void utInitializeTable();
   extern void utFreeTable();
   extern void utAddTableRow();
   extern void utAddTableColumn();
   extern pTable utAssignTable();
   extern void utSaveTable();
   extern pTable utLoadTable();
   extern char *utGetTableCellValue();
   extern void utSetTableCellValue();
   extern void utAppendTableCellValue();
   extern pTable utMapTableForNewRowNames();
   extern pTable utMapTableForNewColumnNames();
   extern pTable utMapRecipeTableForNewColumnNames();
   extern void utFillTableCellsWithValue();
   extern int utGetCountTableRows();
   extern int utGetCountTableColumns();
   extern char *utGetTableRowName();
   extern void utSetTableRowName();
   extern int utGetTableRowNumber();
   extern char *utGetTableColumnName();
   extern void utSetTableColumnName();
   extern int utGetTableColumnNumber();
   extern pTable utRemoveEmptyRowsFromTable();
   extern pTable utRemoveEmptyColumnsFromTable();
   extern void utMergeRowAfterRowInTable();
   extern void utMergeColumnAfterColumnInTable();
   extern void utMergeTableAfterRowInTable();
   extern int utGetLongStringSizeTableColumn();
   extern void utWriteTableToFile();
   extern void utWriteTableToFileAsHTML();
   extern void utWriteTableToTXTFile();
   extern void utWriteTableToCSVFile();
   extern pTable utLoadCSVStringsAsTable();
   extern pTable utLoadCSVFileAsTable();
   extern pTable utLoadCSVFortranStringsAsTable();
   extern pIarray utGetMatchingTableColumns();
   extern pIarray utGetMatchingTableRows();
   extern void utMoveTableRowNamesToColumn1();
   extern void utSubstituteTableCellValues();
   extern utLogical utAreTablesEqual();
#endif

#ifdef __cplusplus
   }
#endif

#endif
